Package: spc
Version: 0.7.2
Date: 2025-09-25
Title: Statistical Process Control -- Calculation of ARL and Other
        Control Chart Performance Measures
Authors@R: person(given = "Sven",
                    family = "Knoth",
                    role = c("aut", "cre"),
                    email = "Sven.Knoth@gmx.de")
Depends: R (>= 1.8.0)
Description: Evaluation of control charts by means of
        the zero-state, steady-state ARL (Average Run Length) and RL quantiles.
        Setting up control charts for given in-control ARL. The control charts
        under consideration are one- and two-sided EWMA, CUSUM, and
        Shiryaev-Roberts schemes for monitoring the mean or variance of normally
        distributed independent data. ARL calculation of the same set of schemes under drift (in the mean) are added.
        Eventually, all ARL measures for the multivariate EWMA (MEWMA) are provided.
License: GPL (>= 2)
URL: https://www.r-project.org
NeedsCompilation: yes
Packaged: 2025-09-23 19:56:28 UTC; knoth
Author: Sven Knoth [aut, cre]
Maintainer: Sven Knoth <Sven.Knoth@gmx.de>
Repository: CRAN
Date/Publication: 2025-09-24 05:10:09 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 14:03:02 UTC; unix
Archs: spc.so.dSYM

useDynLib(spc, .registration=TRUE)
## imports
import("stats")
## exports
export("xshewhartrunsrules.ad", "xshewhartrunsrules.arl", "xshewhartrunsrules.crit", "xshewhartrunsrules.matrix",
       "xshewhart.ar1.arl", "xtshewhart.ar1.arl",
       "xewma.ad", "xewma.arl", "xewma.arl.f", "xewma.crit", "xewma.q", "xewma.sf", "xewma.q.crit",
       "xcusum.ad", "xcusum.arl", "xcusum.crit.L0h", "xcusum.crit.L0L1", "xcusum.crit", "xcusum.q", "xcusum.sf",
       "xgrsr.ad", "xgrsr.arl", "xgrsr.crit",
       "sewma.arl", "sewma.crit", "sewma.sf", "sewma.q.crit", "sewma.q",
       "sewma.sf.prerun", "sewma.arl.prerun", "sewma.q.crit.prerun", "sewma.q.prerun", "sewma.crit.prerun",
       "lns2ewma.arl", "lns2ewma.crit",
       "scusum.arl", "scusum.crit", "scusums.arl",
       "mewma.arl", "mewma.crit", "mewma.psi", "mewma.arl.f", "mewma.ad",
       "xsewma.arl", "xsewma.crit", "xsewma.sf", "xsewma.q.crit", "xsewma.q",
       "xewma.arl.prerun", "xewma.q.prerun", "xewma.sf.prerun",
       "xewma.crit.prerun", "xewma.q.crit.prerun",
       "xtewma.arl", "xtewma.ad", "xtewma.sf", "xtewma.q", "xtewma.q.crit",
       "xtcusum.arl",
       "xDcusum.arl", "xDewma.arl", "xDgrsr.arl", "xDshewhartrunsrules.arl", "xDshewhartrunsrulesFixedm.arl",
       "p.ewma.arl", "tewma.arl", "euklid.ewma.arl",
       "pois.ewma.arl", "pois.ewma.crit", "pois.ewma.ad",
       "pois.cusum.arl", "pois.cusum.crit", "pois.cusum.crit.L0L1",
       "imr.arl", "imr.Ru_Mgiven", "imr.Ru_Mgiven", "imr.Rl_Mgiven", "imr.MandRu",
       "imr.MandRuRl", "imr.RuRl_alone_tail", "imr.RuRl_alone_s3", "imr.RuRl_alone", "imr.Ru_Rlgiven", "imr.Rl_Rugiven",
       "phat.ewma.arl", "phat.ewma.crit", "phat.ewma.lambda", "dphat", "pphat", "qphat",
       "s.res.ewma.arl", "x.res.ewma.arl", "xs.res.ewma.arl", "xs.res.ewma.pms",
       "quadrature.nodes.weights",   
       "tol.lim.fac")

# Computation of res-EWMA PMS (simultaneous mean & variance monitoring)
# PMS = probability of misleading signal
xs.res.ewma.pms <- function(lx, cx, ls, csu, mu, sigma, type="3", alpha=0, n=5,
                       hsx=0, rx=40, hss=1, rs=40, qm=30) {
  if ( lx <= 0 || lx > 1 ) 
    stop("lx has to be between 0 and 1")
  if ( ls <= 0 || ls > 1 )
    stop("ls has to be between 0 and 1")
  if ( cx <= 0 )
    stop("cx has to be positive")
  if ( csu <= 0 ) 
    stop("csu has to be positive")
  if ( sigma <= 0 )
    stop("sigma must be positive")
  if ( !(type %in% c("3", "4")) )
    stop("wrong PMS type")
  vice_versa <- as.numeric(type) - 3
  if ( abs(alpha) > 1 )
    warning("nonstationary AR(1) process")
  if ( n < 1 )
    warning("n is too small")
  n <- round(n)
  if ( abs(hsx) > cx )
    stop("wrong headstart hsx")
  if ( hss < 0 | hss > csu ) 
    stop("wrong headstart hss")
  if ( rx < 5 )
    stop("rx is too small")
  if ( rs <10 ) 
    stop("rs is too small")
  if ( qm < 5 ) 
    stop("qm is too small")
  ctyp <- 1 # later more
  pms <- .C("xsewma_res_pms",as.double(alpha),as.integer(n-1),as.integer(ctyp),
            as.double(lx),as.double(cx),as.double(hsx),as.integer(rx),
            as.double(ls),as.double(csu),as.double(hss),as.integer(rs),
            as.double(mu),as.double(sigma),as.integer(qm),as.integer(vice_versa),
            ans=double(length=1),PACKAGE="spc")$ans 
  names(pms) <- "pms"
  return (pms)
}

imr.Ru_Mgiven <- function(M, L0, N=30, qm=30) {
  M0 <- qnorm( 1-1/(2*L0) )
  if ( M > M0 ) {
    zero <- function(x) imr.arl(M, x, 0, 1, N=N, qm=qm) - L0
    Ru <- uniroot(zero, c(M0, 10), tol=1e-9)$root
  } else {
    Ru <- Inf
  }
  Ru
}

# Computation of CUSUM ARLs (drift monitoring)
xDcusum.arl <- function(k, h, delta, hs=0, sided="one", mode="Gan", m=NULL, q=1, r=30, with0=FALSE) {
  if (k<0)
    stop("k has to be non-negative")
  if (h<=0)
    stop("h has to be positive")
  if ( hs<0 | (sided=="two" & hs>h/2+k) | (sided=="one" & hs>h/2+k) )
    stop("wrong headstart")
  if (r<4)
    stop("r is too small")
  if ( is.null(m) ) {
    m <- 0
  } else {
    if ( m<1 ) stop("m is too small") 
  }
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if (is.na(ctyp))
    stop("invalid cusum type")
  cmode <- pmatch(mode, c("Gan", "Knoth")) - 1
  if (is.na(cmode))
    stop("invalid algorithm mode")
  q <- round(q)
  if (q<1)
    stop("wrong change point position (q)")
  arl <- .C("xDcusum_arl",as.integer(ctyp),as.double(k),
            as.double(h),as.double(hs),as.double(delta),as.integer(m),
            as.integer(r),as.integer(with0),as.integer(cmode),as.integer(q),
            ans=double(length=1),PACKAGE="spc")$ans
  names(arl) <- "arl"
  return (arl)
}

# Computation of GRSR (Girshick, Rubin, Shiryaev, Roberts) ARLs (mean monitoring)
xgrsr.arl <- function(k, g, mu, zr=0, hs=NULL, sided="one", q=1, MPT=FALSE, r=30) {
  if ( k < 0 )			stop("k has to be non-negative")
  if ( g < 0 )			stop("g has to be positive")
  if ( !is.null(hs) ) {
    #if ( hs > g )		stop("wrong headstart")
    hs <- hs
  } else {
    hs <- 11*g # mimics -infinity
    #hs <- inf
  }
  q <- round(q)
  if ( q < 1 )			stop("wrong change point position (q)")
  if ( r < 4 )			stop("r is too small")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if ( is.na(ctyp) )		stop("invalid grsr type")
  arl <- .C("xgrsr_arl",
            as.integer(ctyp), as.double(k), as.double(g),
            as.double(zr), as.double(hs), as.double(mu), as.integer(q), as.integer(r), as.integer(MPT),
            ans=double(length=q), PACKAGE="spc")$ans
  names(arl) <- NULL
  return (arl)
}

# Computation of EWMA quantiles (variance monitoring) with pre-run uncertainty
sewma.q.prerun <- function(l, cl, cu, sigma, df1, df2, alpha, hs=1, sided="upper", r=40, qm=30, qm.sigma=30, truncate=1e-10) {
  if ( l <= 0 | l > 1 )			stop("l (lambda) has to be between 0 and 1")
  if ( cu<=0 )				stop("cu has to be positive")
  if ( cl<0 )				stop("cl has to be non-negative")
  if ( sided!="upper" & cl<1e-6 )	stop("cl is too small")  
  if ( sigma<=0 )			stop("sigma must be positive")
  if ( df1<1 )				stop("df1 must be larger than or equal to 1")
  if ( df2<1 )				stop("df2 must be larger than or equal to 1")  
  if ( alpha <= 0 | alpha >= 1)		stop("quantile level alpha must be in (0,1)")
  if ( hs<cl | hs>cu )			stop("wrong headstart hs")  
  ctyp <- pmatch(sided, c("upper","Rupper","two","Rlower")) - 1
  if (is.na(ctyp))			stop("invalid ewma type") 
  if ( r<10 )				stop("r is too small")
  if ( qm<5 )				stop("qm is too small") 
  if ( qm.sigma<4 )              stop("qm.sigma is too small")
  if ( truncate < 0 | truncate >= 0.5 ) stop("wrong value for truncate (should follow 0 < truncate < 0.5)")
  quant <- .C("sewma_q_prerun",
              as.integer(ctyp), as.double(l), as.double(cl), as.double(cu),
              as.double(alpha), as.double(hs), as.double(sigma),
              as.integer(df1), as.integer(r), as.integer(qm),
              as.integer(df2), as.integer(qm.sigma), as.double(truncate),
              ans=double(length=1),PACKAGE="spc")$ans
  names(quant) <- "q"
  quant
}

# Computation of EWMA critical values for given ARL (variance monitoring)
sewma.crit <- function(l, L0, df, sigma0=1, cl=NULL, cu=NULL, hs=NULL, s2.on=TRUE, sided="upper", mode="fixed", ur=4, r=40, qm=30) {
  
  mitte <- sqrt( 2/df ) * gamma( (df+1)/2 )/ gamma( df/2 )
  if ( is.null(hs) ) {
    if ( s2.on ) { hs <- 1 } else { hs <- mitte }
  }
  
  cu0 <- cl0 <- 0
  if ( l<=0 | l>1 )
    stop("l has to be between 0 and 1")
  if ( L0<1 )
    stop("L0 is too small")
  if ( df<1 )
    stop("df must be positive")
  if ( sigma0<=0 )
    stop("sigma0 must be positive")
  if ( sided=="Rupper" ) {
    if ( is.null(cl) )
      stop("set cl")
    if ( cl<=0 )
      stop("cl must be positive")
    cl0 <- cl
    if ( hs<cl0 )
      stop("hs must not be smaller than cl")
  }
  if ( sided=="Rlower" ) {
    if ( is.null(cu) )
      stop("set cu")
    if ( cu<sigma0 )
      stop(paste("cu must be larger than sigma0 =", sigma0))
    cu0 <- cu
    if ( hs>cu0 )
      stop("hs must not be larger than cu")
  }
  if ( sided=="two" & mode=="fixed" ) {
    if ( is.null(cu) )
      stop("set cu")
    if ( cu<sigma0 )
      stop(paste("cu must be larger than sigma0 =", sigma0))
    cu0 <- cu
    if ( hs>cu0 )
      stop("hs must not be larger than cu")
  }
  s_squared <- as.numeric(s2.on)
  if ( !(s_squared %in% c(0,1)) )
    stop("wrong value for s2.on")
  
  ctyp <- pmatch(sided, c("upper", "Rupper", "two", "Rlower")) - 1
  if (is.na(ctyp))
    stop("invalid ewma type")
  ltyp <- pmatch(mode, c("fixed", "unbiased", "eq.tails", "vanilla")) - 1
  if ( is.na(ltyp) )
    stop("invalid limits type")
  if ( r<10 )
    stop("r is too small")
  if ( qm<10 )
    stop("qm is too small")

  if ( isTRUE(all.equal(1, l)) ) {
    if ( sided=="upper" ) {
      cv <- c(0, qchisq( 1-1/L0, df)/df)
    }
    if ( sided=="Rupper" ) {
      cv <- c(0, Inf)
      warning("not useful for Shewhart type chart")
    }
    if ( sided=="Rlower" ) {
      cv <- c(qchisq( 1/L0, df)/df, Inf)
    }
    if ( sided=="two") {
      if ( mode=="fixed" ) {
        a2 <- 1 - pchisq(df*cu, df)
        if ( a2 > 1/L0 ) {
          cv <- c(0, Inf)
          warning("upper limit too small") 
        } else {
          a1 <- 1/L0 - a2
          cv <- c(qchisq(a1, df)/df, cu)
        }        
      }
      if ( mode=="unbiased") {
        a1 <- 1/(2*L0)
        step <- 1/(2*L0)/10
        one <- 1
        for ( j in 1:8 ) {
          for ( i in 1:11 ) {
            a1 <- a1 + step*one
            k1 <- qchisq(a1, df)
            a2 <- 1/L0 - a1
            k2 <- qchisq(1-a2, df)
            condition <- (k2 - k1)/(log(k2) - log(k1))
            if ( (one*condition) > (one*df) ) break
          }
          step <- step/10
          one <- -one
          if ( abs(condition - df) < 1e-10 ) break
        }
        cv <- c(k1, k2)/df
      }
      if ( mode=="eq.tails" ) {
        a <- 1/(2*L0)
        cv <- qchisq(c(a, 1-a), df)/df
      }
      if ( mode=="vanilla" ) {
        a <- 1/L0
        k2 <- qchisq(1-a, df)
        k1 <- 2*df - k2
        if ( k1 > 0 ) {
          zero <- function(x) ( pchisq( 2*df - qchisq(1-x, df), df) + x ) - 1/L0
          a <- uniroot(zero, c(1/(2*L0), 1/L0), tol=1e-12)$root
          k2 <- qchisq(1-a, df)
          k1 <- 2*df - k2
          cv <- c(k1, k2)/df
        } else {
          cv <- c(0, Inf)
          warning("symmetric design not possible") 
        }
      }      
    }
  } else {
    cv <- .C("sewma_crit",as.integer(ctyp),as.integer(ltyp),as.double(l),
             as.double(L0),as.double(cl0),as.double(cu0),as.double(hs),
             as.double(sigma0),as.integer(df),as.integer(r),as.integer(qm),
             as.double(ur),as.integer(s_squared),
             ans=double(length=2),PACKAGE="spc")$ans
  }
  names(cv) <- c("cl", "cu")
  cv
}

# Computation of CUSUM ARLs (mean monitoring)
xcusum.arl <- function(k, h, mu, hs=0, sided="one", method="igl", q=1, r=30) {
  if (k<0) 
    stop("k has to be non-negative")
  if (h<=0) 
    stop("h has to be positive")
  if ( hs<0 | (sided=="two" & hs>h/2+k) | (sided=="one" & hs>h) ) 
    stop("wrong headstart")
  if (r<4) 
    stop("r is too small")
  ctyp <- pmatch(sided, c("one", "two", "Crosier")) - 1
  if (is.na(ctyp)) 
    stop("invalid cusum type")
  mtyp <- pmatch(method, c("igl", "mc", "mct", "mcl")) - 1
  if (is.na(mtyp)) 
    stop("invalid method")
  q <- round(q)
  if (q<1)
    stop("wrong change point position (q)")
  arl <- .C("xcusum_arl",
            as.integer(ctyp), as.double(k), as.double(h), as.double(hs), as.double(mu),
            as.integer(q), as.integer(r), as.integer(mtyp),
            ans=double(length=q), PACKAGE="spc")$ans
  names(arl) <- NULL
  return (arl)
}

# Computation of Poisson CUSUM alarm threshold, randomization constant and target ooc mean (Ewan/Kemp approach)
pois.cusum.crit.L0L1 <- function(mu0, L0, L1, sided="upper", OUTPUT=FALSE) {
  if ( mu0 < 0 )        stop("mu0 has to be positive")
  if ( L1 <= 1 )        stop("L1 has to be > 1") 
  if ( L0 <= L1 )       stop("L0 has to be > L1")
  if ( !(sided %in% c("upper", "lower")) )      stop("sided has to be either 'upper' or 'lower'")
  
  # helper functions to give the out-of-control mean mu1 for given in-control mean mu0 and reference value k
  k_m01 <- function(mu0, mu1) (mu1 - mu0) / (log(mu1) - log(mu0))
  m1_km0 <- function(mu0, k) {
    zero <- function(x) k - k_m01(mu0,x)
    upper <- mu0 + .5
    while ( zero(upper) > 0 ) upper <- upper + 0.5
    mu1 <- uniroot(zero, c(mu0*1.00000001, upper), tol=1e-9)$root
    mu1
  }
  
  L1_1 <- L1 + 1
  
  m1 <- 10
  k1 <- mu0 * m1
  while ( L1_1 > L1 ) {
    k1 <- k1 + 1
    mu1 <- m1_km0(mu0, k1/m1)
    cv1 <- pois.cusum.crit(mu0, k1, L0, m1, sided=sided, rando=TRUE)
    L1_1 <- pois.cusum.arl(mu1, k1, cv1[1], m1, sided=sided, rando=TRUE, gamma=cv1[2])
    if ( OUTPUT ) cat(paste("k1 =", k1, ",\tmu1 = ", mu1, ",\tL1 =", L1_1, "\n"))
  } 
  
  m1 <- 100
  k1 <- 10 * k1
  while ( L1_1 < L1 ) {
    k1 <- k1 - 1
    mu1 <- m1_km0(mu0, k1/m1)
    cv1 <- pois.cusum.crit(mu0, k1, L0, m1, sided=sided, rando=TRUE)
    L1_1 <- pois.cusum.arl(mu1, k1, cv1[1], m1, sided=sided, rando=TRUE, gamma=cv1[2])
    if ( OUTPUT ) cat(paste("k1 =", k1, ",\tmu1 = ", mu1, ",\tL1 =", L1_1, "\n"))
  }
  
  ff <- max(2, ceiling( 3000 / ( cv1[1]/m1 ) ) / m1)
  m1 <- ff * m1
  k1 <- ff * k1

  while ( L1_1 > L1 ) {
    k1 <- k1 + 1
    mu1 <- m1_km0(mu0, k1/m1)
    cv1 <- pois.cusum.crit(mu0, k1, L0, m1, sided=sided, rando=TRUE)
    L1_1 <- pois.cusum.arl(mu1, k1, cv1[1], m1, sided=sided, rando=TRUE, gamma=cv1[2])
    if ( OUTPUT ) cat(paste("k1 =", k1, ",\tmu1 = ", mu1, ",\tL1 =", L1_1, "\n"))
  }
  
  result <- data.frame(m=m1, km=k1, mu1, k=k1/m1, hm=cv1[1], h=cv1[1]/m1, gamma=cv1[2])
  result
}

# Computation of res-EWMA ARLs (mean monitoring)
x.res.ewma.arl <- function(l, c, mu, alpha=0, n=5, hs=0, r=40) {
  if ( l <= 0 || l > 1 )
    stop("l has to be between 0 and 1")
  if ( c <= 0 )
    warning("usually, c has to be positive")
  if ( abs(alpha) > 1 )
    warning("nonstationary AR(1) process")
  if ( n < 1 )
    warning("n is too small")
  n <- round(n)
  if ( abs(hs) > c ) 
    warning("unusual headstart")
  if ( r < 4 )
    stop("r is too small")
  ctyp <- 1 # later more
  arl <- .C("x_res_ewma_arl",as.double(alpha),as.integer(n),
            as.integer(ctyp),as.double(l),
            as.double(c),as.double(hs),
            as.double(mu),as.integer(r),
            ans=double(length=1),PACKAGE="spc")$ans 
  names(arl) <- "arl"
  return (arl)
}

# Computation of EWMA survival function (variance monitoring) with pre-run uncertainty
sewma.sf.prerun <- function(n, l, cl, cu, sigma, df1, df2, hs=1, sided="upper", qm=30, qm.sigma=30, truncate=1e-10, tail_approx=TRUE) {
  if ( n < 1 )			stop("n has to be a natural number")
  if ( l <= 0 | l > 1 )		stop("l (lambda) has to be between 0 and 1")
  if ( cu<=0 )			stop("cu has to be positive")
  if ( cl<0 )			stop("cl has to be non-negative")
  if ( sided!="upper" & cl<1e-6 ) stop("cl is too small")
  if ( sigma<=0 )		stop("sigma must be positive")
  if ( df1<1 )			stop("df1 must be larger than or equal to 1")
  if ( df2<1 )			stop("df2 must be larger than or equal to 1")  
  if ( hs<cl | hs>cu )		stop("wrong headstart hs")
  ctyp <- pmatch(sided, c("upper","Rupper","two","Rlower")) - 1
  if (is.na(ctyp))		stop("invalid ewma type") 
  if ( qm<5 )			stop("qm is too small")
  if ( qm.sigma<4 )              stop("qm.sigma is too small")
  if ( truncate < 0 | truncate >= 0.5 ) stop("wrong value for truncate (should follow 0 < truncate < 0.5)")
  sf <- .C("sewma_sf_prerun",
           as.integer(ctyp), as.double(l), as.double(cl), as.double(cu), as.double(hs),
           as.double(sigma), as.integer(df1), as.integer(qm), as.integer(n),
           as.integer(df2), as.integer(qm.sigma), as.double(truncate), as.integer(tail_approx),
           ans=double(length=n),PACKAGE="spc")$ans
  names(sf) <- NULL
  sf
}

xDshewhartrunsrules.arl <- function(delta, c=1, m=NULL, type="12") {
  eps <- 1e-6
  if ( is.null(m) ) {
    m <- 4
    arl1 <- xDshewhartrunsrulesFixedm.arl(delta, c=c, m=m, type=type)
    arl2 <- arl1 + 2*eps
    while ( abs(arl2-arl1)>eps & m<1e4 ) {
      m <- round(1.5 * m)
      arl1 <- xDshewhartrunsrulesFixedm.arl(delta, c=c, m=m, type=type)
      arl2 <- xDshewhartrunsrulesFixedm.arl(delta, c=c, m=m+1, type=type)
    }
    arl <- arl1
  } else {
    arl <- xDshewhartrunsrulesFixedm.arl(delta, c=c, m=m, type=type)
  }
  arl
}

# Computation of 2-sided tolerance limits factors
tol.lim.fac <- function(n,p,a,mode="WW",m=30) { 
  if (n<2) 
    stop("n has to be larger than 1")
  if (p<=0 | p>=1) 
    stop("p has to be in (0,1)")
  if (a<=0 | a>=1)
    stop("a has to be in (0,1)")
  mtype <- pmatch(mode, c("WW", "exact")) - 1
  if (is.na(mtype))
    stop("invalid mode type")
  if (m<10)
    stop("m has to be at least 10")
  tlf <- .C("tol_lim_fac",as.integer(n),as.double(p),
             as.double(a),as.integer(mtype),as.integer(m),
             ans=double(length=1),PACKAGE="spc")$ans 
  names(tlf) <- "k"
  return (tlf)
}

# Computation of CUSUM quantiles (mean monitoring)
xcusum.q <- function(k, h, mu, alpha, hs=0, sided="one", r=40) {
  if ( k < 0 )  stop("k has to be non-negative")
  if ( h <= 0 ) stop("h has to be positive")
  if ( hs<0 | (sided=="two" & hs>h/2+k) | (sided=="one" & hs>h/2+k) ) stop("wrong headstart")
  if ( alpha <= 0 | alpha >= 1) stop("quantile level alpha must be in (0,1)")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if ( is.na(ctyp) ) stop("invalid cusum type")
  if ( r < 4 ) stop("r (dimension of Markov chain) is too small")
  quant <- .C("xcusum_q",
              as.integer(ctyp), as.double(k),as.double(h), as.double(alpha), as.double(hs), as.double(mu), as.integer(r),
              ans=double(length=1),PACKAGE="spc")$ans
  names(quant) <- "q"
  quant
}

xshewhartrunsrules.crit <- function(L0, mu=0, type="12") {	
  if (type=="14" & L0>255) {
    stop("L0 too large for type=\"14\"")
  } else {
    c1 <- 1
    c2 <- 1.5
    arl1 <- xshewhartrunsrules.arl(mu,c=c1,type=type)
    arl2 <- xshewhartrunsrules.arl(mu,c=c2,type=type)
    a.error <- 1; c.error <- 1
    while ( a.error>1e-6 && c.error>1e-8 ) {
      c3 <- c1 + (L0-arl1)/(arl2-arl1)*(c2-c1)
      arl3 <- xshewhartrunsrules.arl(mu,c=c3,type=type)
      c1 <- c2; c2 <- c3
      arl1 <- arl2; arl2 <- arl3
      a.error <- abs(arl2-L0); c.error <- abs(c2-c1)
    }
  }
  c3
}

# Computation of EWMA critical values for given ARL (variance monitoring) with pre-run uncertainty
sewma.crit.prerun <- function(l, L0, df1, df2, sigma0=1, cl=NULL, cu=NULL, hs=1, sided="upper", mode="fixed", r=40, qm=30, qm.sigma=30, truncate=1e-10, tail_approx=TRUE, c.error=1e-10, a.error=1e-9) {
  cu0 <- cl0 <- 0
  if ( l<=0 | l>1 )		stop("l has to be between 0 and 1")
  if ( L0<1 )			stop("L0 is too small")
  if ( df1<1 )			stop("df1 must be positive")
  if ( df2<1 )			stop("df2 must be positive")
  if ( sigma0<=0 )		stop("sigma0 must be positive")
  if ( sided=="Rupper" ) {
    if ( is.null(cl) )		stop("set cl")
    if ( cl<=0 )		stop("cl must be positive")
    cl0 <- cl
    if ( hs<cl0 )		stop("hs must not be smaller than cl")
  }
  if ( sided=="Rlower" ) {
    if ( is.null(cu) )		stop("set cu")
    if ( cu<sigma0 )		stop(paste("cu must be larger than sigma0 =", sigma0))
    cu0 <- cu
    if ( hs>cu0 )		stop("hs must not be larger than cu")
  }
  if ( sided=="two" & mode=="fixed" ) {
    if ( is.null(cu) )		stop("set cu")
    if ( cu<sigma0 )		stop(paste("cu must be larger than sigma0 =", sigma0))
    cu0 <- cu
    if ( hs>cu0 )		stop("hs must not be larger than cu")
  }
  ctyp <- pmatch(sided, c("upper", "Rupper", "two", "Rlower")) - 1
  if ( is.na(ctyp) )		stop("invalid ewma type")
  ltyp <- pmatch(mode, c("fixed", "unbiased")) - 1
  if ( is.na(ltyp) )		stop("invalid limits type")
  if ( r<10 )			stop("r is too small")
  if ( qm<10 )			stop("qm is too small")
  if ( qm.sigma<4 )             stop("qm.sigma is too small")
  if ( truncate < 0 | truncate >= 0.5 ) stop("wrong value for truncate (should follow 0 < truncate < 0.5)")
  c <- .C("sewma_crit_prerun",
          as.integer(ctyp), as.integer(ltyp), as.double(l), as.integer(L0),
          as.double(cl0), as.double(cu0), as.double(hs), as.double(sigma0),
          as.integer(df1), as.integer(r), as.integer(qm),
          as.integer(df2), as.integer(qm.sigma), as.double(truncate), as.integer(tail_approx),
          as.double(c.error), as.double(a.error),
          ans=double(length=2),PACKAGE="spc")$ans
  names(c) <- c("cl", "cu")
  return (c)
}

# Computation of I-MR ARLs
imr.arl <- function(M, Ru, mu, sigma, vsided="upper", Rl=0, cmode="coll", N=30, qm=30) {
  if ( M <= 0 )         stop("M has to be positive")
  if ( Ru <= 0 )        stop("Ru has to be positive") 
  if ( sigma <= 0 )     stop("sigma has to be positive")
  if ( Rl < 0 )         stop("Rl has to be non-negative") 
  if ( N < 2 )          stop("N has to be >= 2")
  if ( qm < 10 )        stop("qm has to be >= 10")
  vsided <- tolower(vsided)
  cmode <- tolower(cmode)  
  vtyp <- -1 + pmatch(vsided, c("upper", "two"))
  
  if ( Ru >= 2*M & vsided == "upper" ) {
    # initial solution, which works only in the in-control case
    #Lu <- 1 / ( 2*pnorm(-M, mean=mu, sd=sigma) )
    #arl <- 1 + Lu * ( 2*pnorm(M, mean=mu, sd=sigma) - 1 )
    # the complicated solution
    #Lu <- 1 / ( pnorm(-M, mean=mu, sd=sigma) + pnorm(-M, mean=-mu, sd=sigma) )
    #arl <- 1 + Lu * ( pnorm(M, mean=mu, sd=sigma) - pnorm(-M, mean=mu, sd=sigma) )
    # the 'elegant' approach
    arl <- 1 / ( pnorm(-M, mean=mu, sd=sigma) + pnorm(-M, mean=-mu, sd=sigma) )
  } else {
    arl <- -1
    if ( cmode == "coll" | vsided=="two" ) {
      arl <- .C("imr_arl", as.double(M), as.double(Rl), as.double(Ru), as.double(mu), as.double(sigma),
                           as.integer(vtyp), as.integer(N), as.integer(qm), ans=double(length=1), PACKAGE="spc")$ans 
    }
    if ( cmode == "crowder" ) {
      if ( vsided == "two" ) warning("confirmed only for upper MR")
      arl <- imr.arl.C1987b(M, Ru, mu, sigma, N=N)
    }
    if ( cmode %in% c("gl", "rectangular", "trapezoid", "simpson", "simpson3_8") ) {
      if ( vsided == "two" ) warning("confirmed only for upper MR")
      arl <- imr.arl.Ny(M, Ru, mu/sigma, N, kind=cmode)
    }
  }  
  names(arl) <- "arl"
  arl
}

xshewhartrunsrules.ad <- function(mu1, mu0=0, c=1, type="12") {

# Shewhart chart
  if (type=="1") {
    p0 <- pnorm(  3*c, mean=mu1 ) - pnorm( -3*c, mean=mu1)
    ad <- 1/(1-p0)
  }

# ditto with runs rules
  if (type!="1") {
    Q1 <- xshewhartrunsrules.matrix(mu1, c=c, type=type)
    dimQ <- nrow(Q1)
    one <- rep(1, dimQ)
    I   <- diag(1, dimQ)
    arls <- solve(I-Q1, one)

    Q0 <- xshewhartrunsrules.matrix(mu0, c=c, type=type)
    psi <- Re(eigen(t(Q0))$vectors[,1])

    ad <- sum(psi * arls)/sum(psi)
  }

  ad
}

# Computation of GRSR (Girshick, Rubin, Shiryaev, Roberts) steady-state ARLs (mean monitoring)
xgrsr.ad <- function(k, g, mu1, mu0=0, zr=0, sided="one", MPT=FALSE, r=30) {
  if (k<0) 
    stop("k has to be non-negative")
  if (g<0)
    stop("g has to be positive")
  if (r<4) 
    stop("r is too small")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if (is.na(ctyp)) 
    stop("invalid grsr type")
  ad <- .C("xgrsr_ad",as.integer(ctyp),as.double(k),
           as.double(g),as.double(mu0),as.double(mu1),as.double(zr),as.integer(r),as.integer(MPT),
           ans=double(length=1),PACKAGE="spc")$ans 
  names(ad) <- "ad"
  return(ad)
}

# Computation of EWMA critical values for given ARL 
# (simultaneous mean and variance monitoring)
xsewma.crit <- function(lx, ls, L0, df, mu0=0, sigma0=1, cu=NULL, hsx=0, hss=1,
                        s2.on=TRUE, sided="upper", mode="fixed",
                        Nx=30, Ns=40, qm=30) 
{
  if (lx<=0 || lx>1) 
    stop("lx has to be between 0 and 1")
  if (ls<=0 || ls>1) 
    stop("ls has to be between 0 and 1")
  if (L0<1) 
    stop("L0 is too small")
  if (sigma0<=0)
    stop("sigma0 must be positive")
  if (mode=="fixed" & sided=="two") {
    if (is.null(cu)) stop("set cu")
    if (cu<sigma0) stop("cu is too small")
    if (cu<=0) stop("cu must be positive")
    if (hss>cu) stop("hs must be smaller than cu")
    cu0 <- cu
  } else {
    cu0 <- 0
  }
  if (df<1)
    stop("df must be positive")
  s_squared <- as.numeric(s2.on)
  if ( !(s_squared %in% c(0,1)) )
    stop("wrong value for s2.on")
  ctyp <- pmatch(sided, c("upper","Rupper","two","lower")) - 1
  if (is.na(ctyp)) 
    stop("invalid ewma type")
  ltyp <- pmatch(mode, c("fixed","unbiased")) - 1
  if (is.na(ltyp)) 
    stop("invalid limits type")
  if (Nx<5) 
    stop("r.x is too small")
  if (Ns<10) 
    stop("r.s is too small")
  if (qm<10) 
    stop("qm is too small")
  c <- .C("xsewma_crit",as.integer(ctyp),as.integer(ltyp),
          as.double(lx),as.double(ls),
          as.double(L0),as.double(cu0),as.double(hsx),as.double(hss),
          as.double(mu0),as.double(sigma0),
          as.integer(df),as.integer(Nx),as.integer(Ns),
          as.integer(qm),
          ans=double(length=3),PACKAGE="spc")$ans
  names(c) <- c("cx","cl","cu")
  return (c)
}

# Computation of TEWMA ARLs for iid Poisson
tewma.arl <- function(lambda, k, lk, uk, mu, z0, rando=FALSE, gl=0, gu=0) {
  if ( lambda <= 0 || lambda > 1 )      stop("lambda has to be between 0 and 1")
  if ( k < 1 )		                    stop("k must be >= 1")
  if ( mu < 0 )		                    stop("mu must be positive")  
  if ( lk > z0 | z0 > uk )              stop("wrong headstart")  
  if ( 0 > gl | gl > 1 )                stop("wrong value for gl")
  if ( 0 > gu | gu > 1 )                stop("wrong value for gu")
  irando <- as.numeric(rando)
  
  arl <- .C("tewma_arl_wowR",
            as.integer(irando), as.double(lambda),
            as.integer(k), as.integer(lk), as.integer(uk), as.double(gl), as.double(gu),
            as.double(z0), as.double(mu), ans=double(length=1), PACKAGE="spc")$ans 
  names(arl) <- "arl"
  arl
}

xewma.crit.prerun <- function(l, L0, mu, zr=0, hs=0, sided="two", limits="fix", size=100, df=NULL, estimated="mu", qm.mu=30, qm.sigma=30,
                              truncate=1e-10, c.error=1e-12, L.error=1e-9, OUTPUT=FALSE) {
                              
  if ( OUTPUT ) cat("\nc\t\tL\n")
  
  c2 <- xewma.crit(l, L0, mu0=mu, zr=zr, hs=hs, sided=sided, limits=limits)
  L2 <- xewma.arl.prerun(l, c2, mu, zr=zr, hs=hs, sided=sided, limits=limits, q=1, size=size, df=df, estimated=estimated, qm.mu=qm.mu, qm.sigma=qm.sigma, truncate=truncate)
  if ( OUTPUT ) cat(paste(c2,"\t",L2,"\n"))
  
  if ( L2 < L0 ) {
    while ( L2 < L0 ) {
      L1 <- L2
      c2 <- c2 + .5
      L2 <- xewma.arl.prerun(l, c2, mu, zr=zr, hs=hs, sided=sided, limits=limits, q=1, size=size, df=df, estimated=estimated, qm.mu=qm.mu, qm.sigma=qm.sigma, truncate=truncate)
      if ( OUTPUT ) cat(paste(c2,"\t",L2,"\n"))
    }
    c1 <- c2 - .5
  } else {
    while ( L2 >= L0 ) {
      L1 <- L2
      c2 <- c2 - .5
      L2 <- xewma.arl.prerun(l, c2, mu, zr=zr, hs=hs, sided=sided, limits=limits, q=1, size=size, df=df, estimated=estimated, qm.mu=qm.mu, qm.sigma=qm.sigma, truncate=truncate)
      if ( OUTPUT ) cat(paste(c2,"\t",L2,"\n"))
    }
    c1 <- c2 + .5
  }
  
  if ( size < 51 ) {
    if ( qm.mu < 70 ) qm.mu <- 70 
    if ( qm.mu < 70 ) qm.mu <- 70
    if ( size < 31 ) {
      if ( qm.mu < 90 ) qm.mu <- 90 
      if ( qm.mu < 90 ) qm.mu <- 90
    }
    if ( L2 < L0 ) {
      while ( L2 < L0 ) {
        L1 <- L2
        c2 <- c2 + .1
        L2 <- xewma.arl.prerun(l, c2, mu, zr=zr, hs=hs, sided=sided, limits=limits, q=1, size=size, df=df, estimated=estimated, qm.mu=qm.mu, qm.sigma=qm.sigma, truncate=truncate)
        if ( OUTPUT ) cat(paste(c2,"\t",L2,"\n"))
      }
      c1 <- c2 - .1
    } else {
      while ( L2 >= L0 ) {
        L1 <- L2
        c2 <- c2 - .1
        L2 <- xewma.arl.prerun(l, c2, mu, zr=zr, hs=hs, sided=sided, limits=limits, q=1, size=size, df=df, estimated=estimated, qm.mu=qm.mu, qm.sigma=qm.sigma, truncate=truncate)
        if ( OUTPUT ) cat(paste(c2,"\t",L2,"\n"))
      }
      c1 <- c2 + .1
    }
  }

  L.error_ <- 1; c.error_ <- 1
  while ( L.error_ > L.error & c.error_ > c.error ) {
    c3 <- c1 + (L0 - L1)/(L2 - L1)*(c2 - c1)
    L3 <- xewma.arl.prerun(l, c3, mu, zr=zr, hs=hs, sided=sided, limits=limits, q=1, size=size, df=df, estimated=estimated, qm.mu=qm.mu, qm.sigma=qm.sigma, truncate=truncate)
    if ( OUTPUT ) cat(paste(c3,"\t",L3,"\n"))
    c1 <- c2; c2 <- c3
    L1 <- L2; L2 <- L3
    L.error_ <- abs(L2 - L0); c.error_ <- abs(c2 - c1)
  }
  
  names(c3) <- "c"
  c3
}

# Computation of EWMA ARLs (variance monitoring) with pre-run uncertainty
sewma.arl.prerun <- function(l, cl, cu, sigma, df1, df2, hs=1, sided="upper", r=40, qm=30, qm.sigma=30, truncate=1e-10) {
  if ( l<=0 || l>1 ) 
    stop("l has to be between 0 and 1")
  if ( cl<0 )
    stop("cl has to be non-negative")
  if ( cu<=0 ) 
    stop("cu has to be positive")
  if ( sigma<=0 )
    stop("sigma must be positive")
  if ( df1<1 )
    stop("df1 must be larger than or equal to 1")
  if ( df2<1 )
    stop("df2 must be larger than or equal to 1")
  if ( hs<cl | hs>cu ) 
    stop("wrong headstart")
  ctyp <- pmatch(sided, c("upper", "Rupper", "two", "Rlower")) - 1
  if (is.na(ctyp))
    stop("invalid ewma type")
  if ( r<10 ) 
    stop("r is too small")
  if ( qm<10 ) 
    stop("qm is too small")
  if ( qm.sigma<4 )
    stop("qm.sigma is too small")
  if ( truncate < 0 | truncate >= 0.5 )
    stop("wrong value for truncate (should follow 0 < truncate < 0.5)")  
  arl <- .C("sewma_arl_prerun", as.integer(ctyp), as.double(l),
            as.double(cl), as.double(cu), as.double(hs),
            as.double(sigma), as.integer(df1), as.integer(r), as.integer(qm), 
            as.integer(df2), as.integer(qm.sigma), as.double(truncate),
            ans=double(length=1),PACKAGE="spc")$ans 
  names(arl) <- "arl"
  return (arl)
}

# Computation of Poisson CUSUM ARLs
pois.cusum.arl <- function(mu, km, hm, m, i0=0, sided="upper", rando=FALSE, gamma=0, km2=0, hm2=0, m2=0, i02=0, gamma2=0) {
  if ( mu < 0 )         stop("mu has to be positive")
  if ( km < 1 )         stop("km has to be >= 1") 
  if ( hm < 1 )         stop("hm has to be >= 1")
  if (  m < 1 )         stop("m has to be >= 1")
  if ( !(i0 %in% 0:hm) )     stop("head start i0 has to be an integer in 0, 1, ..., hm")
  ctyp <- pmatch(sided, c("upper", "lower", "two")) - 1
  if ( is.na(ctyp) )    stop("invalid cusum type")
  if ( rando ) {
    if ( gamma < 0 | gamma > 1)   stop("gamma has to be a probability value (0<=.<=1)")
  }
  if ( ctyp==2 ) {
    if ( km2 < 1 )         stop("km2 has to be >= 1") 
    if ( hm2 < 1 )         stop("hm2 has to be >= 1")
    if (  m2 < 1 )         stop("m2 has to be >= 1")
    if ( !(i02 %in% 0:hm) )     stop("head start i02 has to be an integer in 0, 1, ..., hm")
    if ( rando ) {
      if ( gamma2 < 0 | gamma2 > 1)   stop("gamma2 has to be a probability value (0<=.<=1)")
    }
  }
  arl <- .C("ccusum_arl_be",
            as.integer(ctyp), as.integer(rando), as.double(mu),
            as.integer(km), as.integer(hm), as.integer(m), as.integer(i0), as.double(gamma),
            as.integer(km2), as.integer(hm2), as.integer(m2), as.integer(i02), as.double(gamma2),
            ans=double(length=1), PACKAGE="spc")$ans 
  names(arl) <- "arl"
  arl
}

# Computation of EWMA phat upper control limits
phat.ewma.crit <- function(lambda, L0, mu, n, z0, sigma=1, type="known", LSL=-3, USL=3, N=15, qm=25) {
  if ( lambda <= 0 || lambda > 1 )
    stop("lambda has to be between 0 and 1")
  p.star <- pnorm( LSL/sigma ) + pnorm( -USL/sigma )
  if ( type == "estimated" )
    p.star <- 0
  if ( L0 < 1 )
    stop("L0 is too small")
  if ( n < 1 )
    stop("n must be >= 1")
  if ( z0 < p.star & z0 >= 1 )
    stop("wrong headstart")    
  if ( sigma<1e-10 )
    stop("sigma much too small")
  ctyp <- -1 + pmatch(type, c("known", "estimated"))
  if ( is.na(ctyp) )
    stop("invalid sigma mode")
  if ( LSL >= USL )
    stop("wrong relationship between lower and upper specification limits (LSL must be smaller than USL)")
  if ( N < 3 )
    stop("N too small")
  if ( qm < 5 )
    stop("qm too small")
  ucl <- .C("ewma_phat_crit_coll",
            as.double(lambda), as.double(L0), as.double(mu), as.double(sigma), as.integer(n),
            as.double(z0), as.integer(ctyp), as.double(LSL), as.double(USL), as.integer(N), as.integer(qm),
            ans=double(length=1), PACKAGE="spc")$ans 
  names(ucl) <- "ucl"
  ucl
}

# Computation of attribute p EWMA ARLs
p.ewma.arl <- function(lambda, ucl, n, p, z0, sided="upper", lcl=NULL, d.res=1, r.mode="ieee.round", i.mode="integer") {
  i.r.mode <- -2 + pmatch(r.mode, c("gan.floor", "floor", "ceil", "ieee.round", "round", "mix"))
  i.i.mode <- -1 + pmatch(i.mode, c("integer", "half"))
  if ( lambda <= 0 || lambda > 1 )      stop("lambda has to be between 0 and 1")
  if ( ucl < 0 )                        stop("ucl must be larger than 0")
  if ( sided == "upper" ) lcl <- 0
  if ( sided == "lower" ) ucl <- n
  if ( sided == "two" ) {
    if ( is.null(lcl) )                 stop("lcl must be set")
    if ( lcl < 0 )                      stop("lcl must be larger than 0")
  }
  if ( n < 1 )		                    stop("n must be >= 1")
  if ( 0 > p | p > 1 )	                stop("wrong value for p")
  if ( is.null(lcl) ) lcl <- 0
  if ( z0 < lcl | z0 > ucl )            stop("wrong headstart")
  ctyp <- pmatch(sided, c("upper", "lower", "two")) - 1
  if ( is.na(ctyp) )                    stop("invalid ewma type")
  if ( d.res < 1 )                      stop("d.res too small")
  if ( is.na(i.r.mode) )                stop("invalid round mode")
  if ( is.na(i.i.mode) )                stop("invalid interval mode")
  arl <- .C("ewma_p_arl_be",
            as.integer(ctyp), as.double(lambda), as.double(lcl), as.double(ucl), as.integer(n),
            as.double(p), as.double(z0), as.integer(d.res), as.integer(i.r.mode), as.integer(i.i.mode),
            ans=double(length=1), PACKAGE="spc")$ans 
  names(arl) <- "arl"
  arl
}

# Computation of the ARL for modified Shewhart charts, AR(1) data
xtshewhart.ar1.arl <- function(alpha, cS, df, delta=0, N1=50, N2=30, N3=2*N2, INFI=10, mode="tan") {
  if ( abs(alpha) >= 1 )    stop("alpha has to be between -1 and 1")
  if ( cS <= 0 )            stop("cS has to be positive")
  ntyp <- -1 + pmatch(mode, c("identity", "sin", "sinh", "tan"))
  if ( is.na(ntyp) )
    stop("substitution type not provided (yet) or simply wrong") 
  arl <- .C("tshewhart_ar1_arl",
              as.double(alpha), as.double(cS), as.double(delta), as.integer(df),
              as.integer(N1), as.integer(N2), as.integer(N3), as.double(INFI), as.integer(ntyp), ans=double(length=1), PACKAGE="spc")$ans
  names(arl) <- NULL
  arl
}

# Computation of EWMA ARLs (variance monitoring)
sewma.arl <- function(l, cl, cu, sigma, df, s2.on=TRUE, hs=NULL, sided="upper", r=40, qm=30) {
  mitte <- sqrt( 2/df ) * gamma( (df+1)/2 )/ gamma( df/2 )
  if ( is.null(hs) ) {
    if ( s2.on ) { hs <- 1 } else { hs <- mitte }
  }
  
  if ( l<=0 || l>1 ) 
    stop("l has to be between 0 and 1")
  if ( cu<=0 ) 
    stop("cu has to be positive")
  if ( cl<0 )
    stop("cl has to be non-negative")
  if ( sigma<=0 )
    stop("sigma must be positive")
  if ( df<1 )
    stop("df must be larger than or equal to 1")
  s_squared <- as.numeric(s2.on)
  if ( !(s_squared %in% c(0,1)) )
    stop("wrong value for s2.on")
  
  if ( hs<cl | hs>cu ) 
    stop("wrong headstart")
  ctyp <- pmatch(sided, c("upper", "Rupper", "two", "Rlower")) - 1
  if (is.na(ctyp))
    stop("invalid ewma type")
  if (r<10) 
    stop("r is too small")
  if (qm<10) 
    stop("qm is too small")
  arl <- .C("sewma_arl",as.integer(ctyp),as.double(l),
            as.double(cl),as.double(cu),as.double(hs),
            as.double(sigma),as.integer(df),as.integer(r),as.integer(qm),
            as.integer(s_squared),
            ans=double(length=1),PACKAGE="spc")$ans 
  names(arl) <- "arl"
  return (arl)
}

xewma.q.crit.prerun <- function(l, L0, mu, p, zr=0, hs=0, sided="two", limits="fix", size=100, df=NULL, estimated="mu", qm.mu=30, qm.sigma=30,
                                truncate=1e-10, bound=1e-10, c.error=1e-10, p.error=1e-9, OUTPUT=FALSE) {                                
                                
  if ( OUTPUT ) cat("\nc\t\tp\n")
  
  c2 <- xewma.q.crit(l, L0, mu, p, zr=zr, hs=hs, sided=sided, limits=limits, OUTPUT=FALSE)  
  p2 <- 1 - xewma.sf.prerun(l, c2, mu, L0, zr=zr, hs=hs, sided=sided, limits=limits, q=1,
                            size=size, df=df, estimated=estimated, qm.mu=qm.mu, qm.sigma=qm.sigma, truncate=truncate, bound=bound)[L0]                            
  if ( OUTPUT ) cat(paste(c2,"\t",p2,"\n"))
  
  if ( p2 > p ) {
    while ( p2 > p ) {
      p1 <- p2
      c2 <- c2 + .5
      p2 <- 1 - xewma.sf.prerun(l, c2, mu, L0, zr=zr, hs=hs, sided=sided, limits=limits, q=1,
                                size=size, df=df, estimated=estimated, qm.mu=qm.mu, qm.sigma=qm.sigma, truncate=truncate, bound=bound)[L0]
      if ( OUTPUT ) cat(paste(c2,"\t",p2,"\n"))
    }
    c1 <- c2 - .5
  } else {
    while ( p2 <= p ) {
      p1 <- p2
      c2 <- c2 - .5
      p2 <- 1 - xewma.sf.prerun(l, c2, mu, L0, zr=zr, hs=hs, sided=sided, limits=limits, q=1,
                                size=size, df=df, estimated=estimated, qm.mu=qm.mu, qm.sigma=qm.sigma, truncate=truncate, bound=bound)[L0]
      if ( OUTPUT ) cat(paste(c2,"\t",p2,"\n"))
    }
    c1 <- c2 + .5
  }
  
  if ( size < 41 ) {
    if ( qm.mu < 70 ) qm.mu <- 70 
    if ( qm.mu < 70 ) qm.mu <- 70
    if ( size < 21 ) {
      if ( qm.mu < 90 ) qm.mu <- 90 
      if ( qm.mu < 90 ) qm.mu <- 90
    }
    if ( p2 > p ) {
      while ( p2 > p ) {
        p1 <- p2
        c2 <- c2 + .1
        p2 <- 1 - xewma.sf.prerun(l, c2, mu, L0, zr=zr, hs=hs, sided=sided, limits=limits, q=1,
                                  size=size, df=df, estimated=estimated, qm.mu=qm.mu, qm.sigma=qm.sigma, truncate=truncate, bound=bound)[L0]
        if ( OUTPUT ) cat(paste(c2,"\t",p2,"\n"))
      }
      c1 <- c2 - .1
    } else {
      while ( p2 <= p ) {
        p1 <- p2
        c2 <- c2 - .1
        p2 <- 1 - xewma.sf.prerun(l, c2, mu, L0, zr=zr, hs=hs, sided=sided, limits=limits, q=1,
                                  size=size, df=df, estimated=estimated, qm.mu=qm.mu, qm.sigma=qm.sigma, truncate=truncate, bound=bound)[L0]
        if ( OUTPUT ) cat(paste(c2,"\t",p2,"\n"))
      }
      c1 <- c2 + .1
    }
  }
  
  p.error_ <- 1; c.error_ <- 1
  while ( p.error_ > p.error & c.error_ > c.error ) {
    c3 <- c1 + (p - p1)/(p2 - p1)*(c2 - c1)
    p3 <- 1 - xewma.sf.prerun(l, c3, mu, L0, zr=zr, hs=hs, sided=sided, limits=limits, q=1,
                              size=size, df=df, estimated=estimated, qm.mu=qm.mu, qm.sigma=qm.sigma, truncate=truncate, bound=bound)[L0]
    if ( OUTPUT ) cat(paste(c3,"\t",p3,"\n"))
    c1 <- c2; c2 <- c3
    p1 <- p2; p2 <- p3
    p.error_ <- abs(p2 - p); c.error_ <- abs(c2 - c1)
  }
  
  names(c3) <- "c"
  c3
}

imr.Rl_Rugiven <- function(Ru, L0, N=30, qm=30, M0=12) {
  zero <- function(x) imr.arl(M0, Ru, 0, 1, vsided="two", Rl=x, N=N, qm=qm) - L0
  Rl1 <- sqrt(2) * qnorm(0.5+1/(4*L0))
  z1 <- zero(Rl1)
  if ( z1 < 0 ) {
    while ( z1 < 0 ) {
      Rl2 <- Rl1
      Rl1 <- Rl1 / 1.1
      z1 <- zero(Rl1)
    }
    z1 <- Rl1
    Rl1 <- Rl2
    Rl2 <- z1
  } else {
    while ( z1 > 0 ) {
      Rl2 <- Rl1
      Rl1 <- Rl1 * 1.1
      z1 <- zero(Rl1)
    }
  }
  Rl <- uniroot(zero, c(Rl1, Rl2), tol=1e-9)$root
  Rl
}

# Computation of Poisson CUSUM alarm threshold and randomization constant
pois.cusum.crit <- function(mu0, km, A, m, i0=0, sided="upper", rando=FALSE) {
  if ( mu0 < 0 )        stop("mu0 has to be positive")
  if ( km < 1 )         stop("km has to be >= 1") 
  if ( A < 1 )          stop("A has to be >= 1")
  if ( m < 1 )          stop("m has to be >= 1")
  i0 <- round(i0)
  if ( i0 < 0 | i0 > 100 )     stop("head start i0 has to be an integer in 0, 1, ..., 100")
  ctyp <- pmatch(sided, c("upper", "lower", "two")) - 1
  if ( is.na(ctyp) )    stop("invalid cusum type")
  cv <- .C("ccusum_crit_be",
           as.integer(ctyp), as.integer(rando),
           as.double(mu0), as.integer(km), as.double(A), as.integer(m), as.integer(i0),
           ans=double(length=2), PACKAGE="spc")$ans 
  names(cv) <- c("hm", "gamma")
  cv[1] <- as.integer(cv[1])
  cv
}

# Computation of EWMA phat lambda minimizing certain out-of-control ARL
phat.ewma.lambda <- function(L0, mu, n, z0, sigma=1, type="known", max_l=1, min_l=.001, LSL=-3, USL=3, qm=25) {
  p.star <- pnorm( LSL/sigma ) + pnorm( -USL/sigma )
  if ( type == "estimated" )
    p.star <- 0
  if ( L0 < 1 )
    stop("L0 is too small")
  if ( n < 1 )
    stop("n must be >= 1")
  if ( z0 < p.star & z0 >= 1 )
    stop("wrong headstart")
  if ( sigma<1e-12 )
    stop("sigma much too small")
  ctyp <- -1 + pmatch(type, c("known", "estimated"))
  if ( is.na(ctyp) )
    stop("invalid sigma mode")
  if ( max_l < min_l | max_l > 1 )
    stop("wrong value for max_l (or min_l)")
  if ( min_l < 1e-4 )
    stop("min_l too small")
  if ( LSL >= USL )
    stop("wrong relationship between lower and upper specification limits (LSL must be smaller than USL)")
  if ( qm < 5 )
    stop("qm too small")
  lambda <- .C("ewma_phat_lambda_coll",
               as.double(L0), as.double(mu), as.double(sigma), as.integer(ctyp),
               as.double(max_l), as.double(min_l), as.integer(n), as.double(z0),
               as.double(LSL), as.double(USL), as.integer(qm),
               ans=double(length=1), PACKAGE="spc")$ans 
  names(lambda) <- "lambda"
  lambda
}

# Computation of EWMA critical values for given QRL (variance monitoring)
sewma.q.crit <- function(l, L0, alpha, df, sigma0=1, cl=NULL, cu=NULL, hs=1, sided="upper", mode="fixed", ur=4, r=40, qm=30, c.error=1e-12, a.error=1e-9) {
  cu0 <- cl0 <- 0
  if ( l<=0 | l>1 )		stop("l has to be between 0 and 1")
  if ( L0<1 )			stop("L0 is too small")
  if ( alpha<=0 | alpha>=1 )	stop("quantile level alpha must be in (0,1)")
  if ( df<1 )			stop("df must be positive")
  if ( sigma0<=0 )		stop("sigma0 must be positive")
  if ( sided=="Rupper" ) {
    if ( is.null(cl) )		stop("set cl")
    if ( cl<=0 )		stop("cl must be positive")
    cl0 <- cl
    if ( hs<cl0 )		stop("hs must not be smaller than cl")
  }
  if ( sided=="Rlower" ) {
    if ( is.null(cu) )		stop("set cu")
    if ( cu<sigma0 )		stop(paste("cu must be larger than sigma0 =", sigma0))
    cu0 <- cu
    if ( hs>cu0 )		stop("hs must not be larger than cu")
  }
  if ( sided=="two" & mode=="fixed" ) {
    if ( is.null(cu) )		stop("set cu")
    if ( cu<sigma0 )		stop(paste("cu must be larger than sigma0 =", sigma0))
    cu0 <- cu
    if ( hs>cu0 )		stop("hs must not be larger than cu")
  }
  ctyp <- pmatch(sided, c("upper", "Rupper", "two", "Rlower")) - 1
  if ( is.na(ctyp) )		stop("invalid ewma type")
  ltyp <- pmatch(mode, c("fixed", "unbiased", "classic")) - 1
  if ( is.na(ltyp) )		stop("invalid limits type")
  if ( r<10 )			stop("r is too small")
  if ( qm<10 )			stop("qm is too small")
  c <- .C("sewma_q_crit",
          as.integer(ctyp), as.integer(ltyp), as.double(l),
          as.integer(L0), as.double(alpha),
          as.double(cl0), as.double(cu0), as.double(hs),
          as.double(sigma0), as.integer(df), as.integer(r), as.integer(qm), as.double(ur),
          as.double(c.error), as.double(a.error),
          ans=double(length=2),PACKAGE="spc")$ans
  names(c) <- c("cl", "cu")
  return (c)
}

# Computation of 'Euklid'-EWMA ARLs for iid Poisson (Rakitzis/Castagliola/Maravelakis 2015)
euklid.ewma.arl <- function(gX, gY, kL, kU, mu, y0, r0=0) {
  if ( gX <= 0 )            stop("gX must be positive")
  if ( gY <= 0 )            stop("gY must be positive")
  if ( kL <= 0 )            stop("kL must be positive")
  if ( kU <= 0 )            stop("kU must be positive")
  if ( kU < kL )            stop("kU must be larger than or equal to kL")
  if ( mu <= 0 )            stop("mu must be positive")  
  if ( y0 <= 0 )            stop("y0 must be positive")
  if ( r0 < 0 )             stop("r0 must be non-negative")

  arl <- .C("euklid_ewma_arl",
            as.integer(gX), as.integer(gY), as.integer(kL), as.integer(kU),
            as.double(mu), as.double(y0), as.integer(r0),
            ans=double(length=1), PACKAGE="spc")$ans 
  names(arl) <- "arl"
  arl
}

# Computation of EWMA quantiles (mean monitoring)
xewma.q <- function(l, c, mu, alpha, zr=0, hs=0, sided="two", limits="fix", q=1, r=40) {
  if ( l <= 0 | l > 1 )          stop("l (lambda) has to be between 0 and 1")
  if ( c<=0 )                    warning("usually, c has to be positive")
  if ( alpha <= 0 | alpha >= 1)  stop("quantile level alpha must be in (0,1)")
  if ( zr > c & sided == "one")  stop("wrong reflexion border")
  if ( (sided == "two" & abs(hs) > c) | (sided == "one" & ( hs < zr | hs > c )) )
                                 warning("unusual headstart")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if ( is.na(ctyp) )             stop("invalid ewma type")
  ltyp <- -1 + pmatch(limits, c("fix", "vacl", "fir", "both", "Steiner", "stat", "test"))
  if (is.na(ltyp))               stop("invalid limits type")
  if ( (sided=="one") & !( limits %in% c("fix", "vacl", "stat") ) )
                                 stop("not supported for one-sided EWMA (not reasonable or not implemented yet")
  if ( r < 4 )                   stop("r is too small")
  q <- round(q)
  if ( q<1 )                     stop("wrong change point position (q)")
  quant <- .C("xewma_q",
              as.integer(ctyp), as.double(l), as.double(c), as.double(alpha), as.double(zr), as.double(hs), as.double(mu),
              as.integer(ltyp), as.integer(r), as.integer(q),
              ans=double(length=1),PACKAGE="spc")$ans
  names(quant) <- "q"
  quant
}

# Computation of EWMA quantiles (mean monitoring) under specified pr-run scenarios
xewma.q.prerun <- function(l, c, mu, p, zr=0, hs=0, sided="two", limits="fix", q=1, size=100, df=NULL, estimated="mu", qm.mu=30, qm.sigma=30, truncate=1e-10, bound=1e-10) {
  if ( l <= 0 | l > 1 )          stop("l (lambda) has to be between 0 and 1")
  if ( c<=0 )                    warning("usually, c has to be positive")
  if ( p <= 0 | p >= 1)          stop("quantile level p must be in (0,1)")
  if ( zr > c & sided == "one")  stop("wrong reflexion border")
  if ( (sided == "two" & abs(hs) > c) | (sided == "one" & ( hs < zr | hs > c )) )
                                 stop("wrong headstart")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if ( is.na(ctyp) )             stop("invalid ewma type")
  ltyp <- -1 + pmatch(limits, c("fix", "vacl", "fir", "both", "Steiner", "stat"))
  if (is.na(ltyp))               stop("invalid limits type")
  if ( (sided=="one") & !( limits %in% c("fix", "vacl", "stat") ) )
                                 stop("not supported for one-sided EWMA (not reasonable or not implemented yet")
  q <- round(q)
  if ( q<1 )                     stop("wrong change point position (q)")
  if ( size<2 )                  stop("pre run size too small")
  if ( is.null(df) ) df = size - 1
  if ( df<1 )                    stop("degrees of freedom (df) too small")
  emode <- -1 + pmatch(estimated, c("mu", "sigma", "both"))
  if (is.na(emode))              stop("invalid to be estimated type")
  if ( qm.mu<4 )                 stop("qm.mu is too small")
  if ( qm.sigma<4 )              stop("qm.sigma is too small")
  if ( truncate < 0 | truncate >= 0.5 ) stop("wrong value for truncate (should follow 0 < truncate < 0.5)")
  if ( bound < 0 | bound >= 0.001 ) stop("wrong value for bound (should follow 0 < truncate < 0.001)")
  quant <- .C("xewma_q_prerun",
              as.integer(ctyp), as.double(l), as.double(c),
              as.double(p), as.double(zr), as.double(hs),
              as.double(mu), as.integer(ltyp),
              as.integer(q), as.integer(size), as.integer(df), as.integer(emode),
              as.integer(qm.mu), as.integer(qm.sigma), as.double(truncate), as.double(bound),
              ans=double(length=1),PACKAGE="spc")$ans
  names(quant) <- "q"
  quant
}

# Computation of EWMA survival function (mean monitoring)
xtewma.sf <- function(l, c, df, mu, n, zr=0, hs=0, sided="two", limits="fix", mode="tan", q=1, r=40) {
  if ( l<=0 || l>1 )             warning("l is typically between 0 and 1 -- you should really know what you do")
  if ( c <= 0 )                  warning("usually, c has to be positive")
  if ( df < 1 )                  stop("df must be greater or equal to 1")
  if ( n < 1 )                   stop("n has to be a natural number")
  if ( zr > c & sided == "one")  stop("wrong reflexion border")
  if ( (sided == "two" & abs(hs) > c) | (sided == "one" & ( hs < zr | hs > c )) )
                                 warning("unusual headstart")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if ( is.na(ctyp) )             stop("invalid ewma type")
  ltyp <- -1 + pmatch(limits, c("fix", "vacl"))
  if (is.na(ltyp))               stop("invalid limits type")
  ntyp <- -1 + pmatch(mode, c("identity", "sin", "sinh", "tan"))
  if ( is.na(ntyp) )             stop("substitution type not provided (yet)") 
  if ( r < 4 )                   stop("r is too small")
  q <- round(q)
  if ( q<1 )                     stop("wrong change point position (q)")
  sf <- .C("xtewma_sf",
           as.integer(ctyp), as.double(l), as.double(c), as.double(zr), as.double(hs), as.integer(df), as.double(mu),
           as.integer(ltyp), as.integer(r), as.integer(ntyp), as.integer(q), as.integer(n),
           ans=double(length=n),PACKAGE="spc")$ans
  names(sf) <- NULL
  sf
}

# Computation of EWMA survival function (variance monitoring)
sewma.sf <- function(n, l, cl, cu, sigma, df, hs=1, sided="upper", r=40, qm=30) {
  if ( n < 1 )			stop("n has to be a natural number")
  if ( l <= 0 | l > 1 )		stop("l (lambda) has to be between 0 and 1")
  if ( cu<=0 )			stop("cu has to be positive")
  if ( cl<0 )			stop("cl has to be non-negative")
  if ( sided!="upper" & cl<1e-6 ) stop("cl is too small")
  if ( sigma<=0 )		stop("sigma must be positive")
  if ( df<1 )			stop("df must be larger than or equal to 1")  
  if ( hs<cl | hs>cu )		stop("wrong headstart hs")
  if ( r<10 )			stop("r is too small")
  ctyp <- pmatch(sided, c("upper","Rupper","two","Rlower")) - 1
  if (is.na(ctyp))		stop("invalid ewma type") 
  if ( qm<5 )			stop("qm is too small") 
  sf <- .C("sewma_sf",
           as.integer(ctyp), as.double(l), as.double(cl), as.double(cu), as.double(hs), as.integer(r),
           as.double(sigma), as.integer(df), as.integer(qm), as.integer(n),
           ans=double(length=n),PACKAGE="spc")$ans
  names(sf) <- NULL
  sf
}

pphat <- function(q, n, mu=0, sigma=1, type="known", LSL=-3, USL=3, nodes=30) {    
  if ( n < 1 )
    stop("n must be >= 1")
  if ( sigma<1e-10 )
    stop("sigma much too small")
  ctyp <- -1 + pmatch(type, c("known", "estimated"))
  if ( is.na(ctyp) )
    stop("invalid sigma mode")
  if ( LSL >= USL )
    stop("wrong relationship between lower and upper specification limits (LSL must be smaller than USL)")
  if ( nodes<2 )
    stop("far too less nodes")
    
  p.star <- pnorm( LSL/sigma ) + pnorm( -USL/sigma )
  if ( type == "estimated" ) p.star <- 0

  cdf <- rep(NA, length(q))
  for ( i in 1:length(q) ) {
    cdf[i] <- 0
    if ( q[i] >= 1 ) cdf[i] <- 1
    if ( p.star<q[i] && q[i]<1 )
      cdf[i] <- .C("phat_cdf",
                    as.double(q[i]), as.integer(n), as.double(mu), as.double(sigma), as.integer(ctyp),
                    as.double(LSL), as.double(USL), as.integer(nodes),
                    ans=double(length=1), PACKAGE="spc")$ans
  }           
  names(cdf) <- NULL
  cdf
}

xDshewhartrunsrulesFixedm.arl <- function(delta, c=1, m=100, type="12") {
  mus <- (1:m)*delta

# Shewhart chart
  if (type=="1") {
    p0  <- pnorm(  3*c, mean=mus ) - pnorm( -3*c, mean=mus)
    arls <- 1/(1-p0[m])
    for ( i in (m-1):1 ) arls <- 1 + p0[i]*arls
  }

# ditto with runs rules
  if (type!="1") {
    Q <- xshewhartrunsrules.matrix(mus[m], c=c, type=type)
    dimQ <- nrow(Q)
    one <- rep(1, dimQ)
    I   <- diag(1, dimQ)
    arls <- solve(I-Q, one)

    for ( i in (m-1):1 ) {
      Q <- xshewhartrunsrules.matrix(mus[i], c=c, type=type)
      arls <- 1 + (Q %*% arls)[,1]
    }
  }

  arl <- arls[1]
  arl
}

xshewhartrunsrules.arl <- function(mu, c=1, type="12") {

# Shewhart chart
  if (type=="1") {
    p0  <- pnorm(  3*c, mean=mu ) - pnorm( -3*c, mean=mu)
    arls <- 1/(1-p0)
  }

# ditto with runs rules
  if (type!="1") {
    Q <- xshewhartrunsrules.matrix(mu, c=c, type=type)
    dimQ <- nrow(Q)
    one <- rep(1, dimQ)
    I   <- diag(1, dimQ)
    arls <- solve(I-Q, one)
  }

  arl <- arls[1]
  arl
}

# Computation of CUSUM k (reference value) and threshold h for given in-control ARL L0 and out-of-control ARL L1 (mean monitoring)
# Ewan & Kemp 1960 or Kemp 1962

xcusum.crit.L0L1 <- function(L0, L1, hs = 0, sided="one", r = 30, L1.eps=1e-6, k.eps=1e-8) {
  k1 <- 0
  L1_1 <- L1 + 1
  while ( L1_1 > L1 ) {
    k1 <- k1 + .1
    h1   <- xcusum.crit(k1, L0, hs=hs, sided=sided, r=r)
    L1_1 <- xcusum.arl(k1, h1, 2*k1, hs=hs, sided=sided, r=r)
  } 
  while ( L1_1 < L1 & k1 > 0.01 ) {
    k1 <- k1 - .01
    h1   <- xcusum.crit(k1, L0, hs=hs, sided=sided, r=r)
    L1_1 <- xcusum.arl(k1, h1, 2*k1, hs=hs, sided=sided, r=r)
  }
  k2 <- k1 + .01
  h2   <- xcusum.crit(k2, L0, hs=hs, sided=sided, r=r)
  L1_2 <- xcusum.arl(k2, h2, 2*k2, hs=hs, sided=sided, r=r)
  dk <- 1
  while ( abs(L1-L1_2) > L1.eps  &  abs(dk) > k.eps  ) {
    k3 <- k1 + ( L1 - L1_1 ) / ( L1_2 - L1_1 ) * ( k2 - k1 )
    h3   <- xcusum.crit(k3, L0, hs=hs, sided=sided, r=r)
    L1_3 <- xcusum.arl(k3, h3, 2*k3, hs=hs, sided=sided, r=r)
    # secant rule
    dk <- k3-k2
    k1 <- k2
    L1_1 <- L1_2
    k2 <- k3
    L1_2 <- L1_3
  }
  result <- c(k3, h3)
  names(result) <- c("k", "h")
  result
}

# Computation of EWMA RL quantiles (simultaneous mean & variance monitoring)
xsewma.q <- function(lx, cx, ls, csu, df, alpha, mu, sigma, hsx=0, Nx=40, csl=0, hss=1, Ns=40, sided="upper", qm=30) {
  if ( lx<=0 | lx>1 ) 
    stop("lx has to be between 0 and 1")
  if ( ls<=0 | ls>1 )
    stop("ls has to be between 0 and 1")
  if ( cx<=0 )
    stop("cx has to be positive")
  if ( csu<=0 ) 
    stop("csu has to be positive")
  if ( df<1 )
    stop("df must be larger than or equal to 1")
  if ( alpha <= 0 | alpha >= 1)
    stop("quantile level alpha must be in (0,1)") 
  if ( sigma<=0 )
    stop("sigma must be positive")
  if ( abs(hsx)>cx )
    stop("wrong headstart hsx")
  if ( Nx<5 )
    stop("Nx is too small")    
  if ( csl<0 )
    stop("clu has to be non-negative")  
  if ( hss<csl | hss>csu ) 
    stop("wrong headstart hss")
  if ( Ns<10 ) 
    stop("Ns is too small")
  ctyp <- pmatch(sided, c("upper","two")) - 1
  if (is.na(ctyp)) 
    stop("invalid ewma type")
  if ( qm<5 ) 
    stop("qm is too small")
  quant <- .C("xsewma_q",as.integer(ctyp),as.double(alpha),
            as.double(lx),as.double(cx),as.double(hsx),as.integer(Nx),
            as.double(ls),as.double(csl),as.double(csu),as.double(hss),as.integer(Ns),
            as.double(mu),as.double(sigma),
            as.integer(df),as.integer(qm),
            ans=double(length=1),PACKAGE="spc")$ans 
  names(quant) <- "q"
  quant
}

imr.RuRl_alone_tail <- function(L0, N=30, qm=30, M0=12) {
  zero <- Vectorize(function(x) {
    alpha <- 4 * (1 - pnorm(x/sqrt(2)) )
    Rl <- sqrt(2) * qnorm(0.5+alpha/4)
    imr.arl(M0, x, 0, 1, vsided="two", Rl=Rl, N=N, qm=qm) - L0
  })  
  Ru1 <- sqrt(2) * qnorm( 1-1/(4*L0) )
  D1 <- zero(Ru1)  
  if ( D1 < 0 ) {
    while ( D1 < 0 ) {
      Ru2 <- Ru1
      Ru1 <- Ru1 * 1.1
      D1 <- zero(Ru1)
    }
    D1 <- Ru1
    Ru1 <- Ru2
    Ru2 <- D1
  } else {
    while ( D1 > 0 ) {
      Ru2 <- Ru1
      Ru1 <- Ru1 / 1.1
      D1 <- zero(Ru1)
    }
  }
  Ru <- uniroot(zero, c(Ru1, Ru2), tol=1e-9)$root
  alpha <- 4 * (1 - pnorm(Ru/sqrt(2)) )
  Rl <- sqrt(2) * qnorm(0.5+alpha/4)
  c(Rl, Ru)
}

# Computation of EWMA ARLs (mean monitoring, t distributed data)
xtewma.arl <- function(l, c, df, mu, zr=0, hs=0, sided="two", limits="fix", mode="tan", q=1, r=40) {
  if ( l<=0 || l>1 )
    warning("l is typically between 0 and 1 -- you should really know what you do")
  if ( c<=0 )
    warning("usually, c has to be positive")
  if ( df < 1 )
    stop("df must be greater or equal to 1")
  if ( zr>c & sided=="one" )
    stop("wrong reflexion border")
  if ( (sided=="two" & abs(hs)>c) | (sided=="one" & (hs<zr | hs>c)) ) 
    warning("unusual headstart")
  if ( r<4 )
    stop("r is too small")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if ( is.na(ctyp) )
    stop("invalid ewma type")
  ltyp <- -1 + pmatch(limits, c("fix", "vacl"))
  if ( is.na(ltyp) )
    stop("invalid limits type")
  ntyp <- -1 + pmatch(mode, c("identity", "sin", "sinh", "tan"))
  if ( is.na(ntyp) )
    stop("substitution type not provided (yet)")    
  q <- round(q)
  if ( q<1 )
    stop("wrong change point position (q)")
  if ( limits=="fix" & q>1 ) {
    arl <- .C("xtewma_arl",as.integer(ctyp),as.double(l),
              as.double(c),as.double(zr),as.double(hs),as.integer(df),
              as.double(mu),as.integer(ltyp),as.integer(r),as.integer(ntyp),as.integer(q),
              ans=double(length=q), PACKAGE="spc")$ans 
  } else {
    arl <- .C("xtewma_arl",as.integer(ctyp),as.double(l),
              as.double(c),as.double(zr),as.double(hs),as.integer(df),
              as.double(mu),as.integer(ltyp),as.integer(r),as.integer(ntyp),as.integer(q),
              ans=double(length=1), PACKAGE="spc")$ans
  }
  names(arl) <- NULL
  return (arl)
}

# Computation of EWMA ARLs (mean monitoring) under specified pr-run scenarios
xewma.arl.prerun <- function(l, c, mu, zr=0, hs=0, sided="two", limits="fix", q=1, size=100, df=NULL, estimated="mu", qm.mu=30, qm.sigma=30, truncate=1e-10) {
  if ( l<=0 | l>1 ) stop("l has to be between 0 and 1")
  if ( c<=0 )     warning("usually, c has to be positive")
  if ( zr>c & sided=="one" ) stop("wrong reflexion border")
  if ( (sided=="two" & abs(hs)>c) | (sided=="one" & (hs<zr | hs>c)) ) warning("unusual headstart")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if (is.na(ctyp)) stop("invalid ewma type")
  ltyp <- -1 + pmatch(limits, c("fix", "vacl", "fir", "both", "Steiner", "stat"))
  if (is.na(ltyp)) stop("invalid limits type")
  if ( (sided=="one") & !(limits %in% c("fix", "vacl", "stat", "limit", "fixW")) )
    stop("not supported for one-sided EWMA (not reasonable or not implemented yet")
  q <- round(q)
  if ( q<1 ) stop("wrong change point position (q)")
  if ( size<2 ) stop("pre run size too small")
  if ( is.null(df) ) df = size - 1
  if ( df<1 ) stop("degrees of freedom (df) too small")
  emode <- -1 + pmatch(estimated, c("mu", "sigma", "both"))
  if (is.na(emode)) stop("invalid to be estimated type")
  if ( qm.mu<4 ) stop("qm.mu is too small")
  if ( qm.sigma<4 ) stop("qm.sigma is too small")
  if ( truncate < 0 | truncate >= 0.5 ) stop("wrong value for truncate (should follow 0 < truncate < 0.5)")
  arl <- .C("xewma_arl_prerun",
            as.integer(ctyp), as.double(l), as.double(c),
            as.double(zr), as.double(hs), as.double(mu),
            as.integer(ltyp), as.integer(q),
            as.integer(size), as.integer(df),
            as.integer(emode), as.integer(qm.mu), as.integer(qm.sigma), as.double(truncate),
            ans=double(length=1), PACKAGE="spc")$ans 
  names(arl) <- "arl"
  return (arl)
}

qphat <- function(p, n, mu=0, sigma=1, type="known", LSL=-3, USL=3, nodes=30) {    
  if ( n < 1 )
    stop("n must be >= 1")
  if ( sigma<1e-10 )
    stop("sigma much too small")
  ctyp <- -1 + pmatch(type, c("known", "estimated"))
  if ( is.na(ctyp) )
    stop("invalid sigma mode")
  if ( LSL >= USL )
    stop("wrong relationship between lower and upper specification limits (LSL must be smaller than USL)")
  if ( nodes<2 )
    stop("far too less nodes")

  qf <- rep(NA, length(p))
  for ( i in 1:length(p) ) {
    qf[i] <- NA
    if ( 0<p[i] && p[i]<1 )
      qf[i] <- .C("phat_qf",
                    as.double(p[i]), as.integer(n), as.double(mu), as.double(sigma), as.integer(ctyp),
                    as.double(LSL), as.double(USL), as.integer(nodes),
                    ans=double(length=1), PACKAGE="spc")$ans
  }           
  names(qf) <- "qf"
  qf
}

# Computation of CUSUM ARLs (mean monitoring, t distributed data)
xtcusum.arl <- function(k, h, df, mu, hs=0, sided="one", mode="tan", r=30) {
  if ( k < 0 ) 
    stop("k has to be non-negative")
  if ( h <= 0 ) 
    stop("h has to be positive")
  if ( df < 1 )
    stop("df must be greater or equal to 1")
  if ( hs < 0 | ( sided=="two" & hs>h/2+k ) | ( sided=="one" & hs>h ) ) 
    stop("wrong headstart")
    ntyp <- -1 + pmatch(mode, c("identity", "sin", "sinh", "tan"))
  if ( is.na(ntyp) )
    stop("substitution type not provided (yet)")    
  if ( r < 4 ) 
    stop("r is too small")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if (is.na(ctyp)) 
    stop("invalid cusum type")
  arl <- .C("xtcusum_arl",
            as.integer(ctyp), as.double(k), as.double(h), as.double(hs), as.integer(df), double(mu), as.integer(r), as.integer(ntyp),
            ans=double(length=1), PACKAGE="spc")$ans
  names(arl) <- NULL
  return (arl)
}

# Computation of MEWMA ARLs (multivariate mean monitoring)
mewma.arl <- function(l, cE, p, delta=0, hs=0, r=20, ntype=NULL, qm0=20, qm1=qm0) {
  if ( l<=0 | l>1 )		stop("l has to be between 0 and 1")
  if ( cE<=0 )			stop("threshold c has to be positive")
  if ( p<1 )			stop("wrong dimension parameter")
  if ( delta<0 )		stop("wrong magnitude value")
  if ( hs<0 )			stop("wrong head start value")
  if ( r<4 )			stop("resolution too small")
  if ( qm0<5 )			stop("more quadrature nodes needed")
  if ( qm1<5 )			stop("more quadrature nodes needed")
  
  if ( is.null(ntype) ) {
    if ( delta <1e-10 ) {
      ntype <- "gl2"
    } else {
      #if ( p %in% c(2,4) ) {
      if ( p==2 ) {
        ntype <- "gl3"
      } else {
        ntype <- "gl5"
      }
    }
  }
  
  qtyp <- pmatch(tolower(ntype), c("gl", "co", "ra", "cc", "mc", "sr", "co2", "gl2", "gl3", "gl4", "gl5", "co3", "co4", "ngl1", "ngl2", "ngl3", "ngl4", "ngl5")) - 1
  if ( is.na(qtyp) )		stop("invalid type of numerical algorithm")

  arl <- .C("mewma_arl", as.double(l), as.double(cE),
                         as.integer(p), as.double(delta),
                         as.double(hs), as.integer(r),
                         as.integer(qtyp), as.integer(qm0), as.integer(qm1),
                         ans=double(length=1), PACKAGE="spc")$ans

  names(arl) <- NULL
  arl
}

# Computation of EWMA ARLs (mean monitoring)
xewma.arl <- function(l, cE, mu, zr=0, hs=0, sided="one", limits="fix", q=1, steady.state.mode="conditional", r=40) {
  if ( l<=0 | l>2 )
    stop("l has to be between 0 and 2")
  if ( any(cE<=0) )
    warning("usually, cE has to be positive")
  if ( limits!="cfar" ) {  
    if ( zr>cE & sided=="one" )
      stop("wrong reflexion border")
    if ( (sided=="two" & abs(hs)>cE) | (sided=="one" & (hs<zr | hs>cE)) ) 
      warning("unusual headstart")
  }
  if ( r<4 )
    stop("r is too small")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if ( is.na(ctyp) )
    stop("invalid ewma type")
  ltyp <- -1 + pmatch(limits,
          c("fix", "vacl", "fir", "both", "Steiner", "stat", "cfar", "limit", "fixW", "fixC"))            
  if ( is.na(ltyp) )
    stop("invalid limits type")
  if ( (sided=="one") & !(limits %in% c("fix", "vacl", "stat", "limit", "fixW")) )
    stop("not supported for one-sided EWMA (not reasonable or not implemented yet")
  q <- round(q)
  if ( q<1 )
    stop("wrong change point position (q)")
  styp <- pmatch(steady.state.mode, c("conditional", "cyclical")) - 1
  if (is.na(styp))
    stop("invalid steady.state.mode")
  if ( limits=="cfar" ) {
    ctyp <- length(cE) # nc
    hs <- cE[length(cE)] # cinf
  }
  if ( limits=="fix" & q>1 & styp==0 ) {
    arl <- .C("xewma_arl",as.integer(ctyp),as.double(l),
              as.double(cE),as.double(zr),as.double(hs),
              as.double(mu),as.integer(ltyp),as.integer(r),as.integer(q),as.integer(styp),
              ans=double(length=q), PACKAGE="spc")$ans 
  } else {
    arl <- .C("xewma_arl",as.integer(ctyp),as.double(l),
              as.double(cE),as.double(zr),as.double(hs),
              as.double(mu),as.integer(ltyp),as.integer(r),as.integer(q),as.integer(styp),
              ans=double(length=1), PACKAGE="spc")$ans
  }
  names(arl) <- NULL
  return (arl)
}

# Computation of MEWMA steady-state pdf (multivariate mean monitoring)
mewma.psi <- function(l, cE, p, type="cond", hs=0, r=20) {
  if ( l<=0 | l>1 )		stop("l has to be between 0 and 1")
  if ( cE<=0 )			stop("threshold c has to be positive")
  if ( p<1 )			stop("wrong dimension parameter")
  if ( hs<0 )			stop("wrong head start value")
  if ( r<4 )			stop("resolution too small")

  itype <- pmatch(tolower(type), c("cond", "cycl")) - 1
  if ( is.na(itype) )		stop("wrong type of steady-state density")
  
  zeug <- .C("mewma_psi", as.double(l), as.double(cE), as.integer(p), as.integer(itype), as.double(hs), as.integer(r),
                          ans=double(length=3*r+1), PACKAGE="spc")$ans
        
  zahl <- zeug[1]      
  PSI  <- zeug[1:r + 1]                          
  w    <- zeug[1:r + r+1]
  z    <- zeug[1:r + 2*r+1]
  
  l2 <- ( (1-l)/l )^2 
  fchi <- function(u, a) 2*a * dchisq( u^2/l^2, p, ncp=l2*a^2)/l^2
  
  if ( itype == 0 ) psi <- Vectorize(function(x) sum( w * PSI * fchi(sqrt(x), z))/zahl, "x")
  
  if ( itype == 1 ) {
    if ( hs < 1e-9 )  psi <- Vectorize(function(x) dchisq( x/l^2, p)/l^2 / zahl + sum( w * PSI * fchi(sqrt(x), z) ), "x")
    if ( hs >= 1e-9 ) psi <- Vectorize(function(x) fchi(sqrt(x), hs) / zahl + sum( w * PSI * fchi(sqrt(x), z) ), "x")
  }  
  
  psi
}

# Computation of EWMA survival function (simultaneous mean & variance monitoring)
xsewma.sf <- function(n, lx, cx, ls, csu, df, mu, sigma,
                       hsx=0, Nx=40,
                       csl=0, hss=1, Ns=40,
                       sided="upper", qm=30) {
                       
  if ( n < 1 )				stop("n has to be a natural number")
  if ( lx<=0 | lx>1 )			stop("lx has to be between 0 and 1")
  if ( ls<=0 | ls>1 )			stop("ls has to be between 0 and 1")
  if ( cx<=0 )				stop("cx has to be positive")
  if ( csu<=0 )				stop("csu has to be positive")
  if ( csl<0 )				stop("csl has to be non-negative")
  if ( sigma<=0 )			stop("sigma must be positive")
  if ( df<1 )				stop("df must be larger than or equal to 1")
  if ( abs(hsx)>cx )			stop("wrong headstart hsx")
  if ( hss<csl | hss>csu )		stop("wrong headstart hss")
  if ( Nx<5 )				stop("Nx is too small")
  if ( Ns<10 )				stop("Ns is too small")
  if ( qm<5 )				stop("qm is too small")
  ctyp <- pmatch(sided, c("upper","Rupper","two","lower")) - 1
  if (is.na(ctyp))			stop("invalid ewma type") 

  sf <- .C("xsewma_sf", as.integer(ctyp),
           as.double(lx),as.double(cx),as.double(hsx),as.integer(Nx),
           as.double(ls),as.double(csl),as.double(csu),as.double(hss),as.integer(Ns),
           as.double(mu),as.double(sigma),as.integer(df),as.integer(qm),
           as.integer(n),
           ans=double(length=n),PACKAGE="spc")$ans

  names(sf) <- NULL
  sf
}

# Computation of res-EWMA ARLs (simultaneous mean & variance monitoring)
xs.res.ewma.arl <- function(lx, cx, ls, csu, mu, sigma, alpha=0, n=5,
                       hsx=0, rx=40, hss=1, rs=40, qm=30) {
  if ( lx<=0 || lx>1 ) 
    stop("lx has to be between 0 and 1")
  if ( ls<=0 || ls>1 )
    stop("ls has to be between 0 and 1")
  if ( cx <= 0 )
    stop("cx has to be positive")
  if ( csu <= 0 ) 
    stop("csu has to be positive")
  if ( sigma <= 0 )
    stop("sigma must be positive")
  if ( abs(alpha)>1 )
    warning("nonstationary AR(1) process")
  if ( n < 2 )
    warning("n is too small")
  n <- round(n)
  if ( abs(hsx) > cx )
    stop("wrong headstart hsx")
  if ( hss < 0 | hss > csu ) 
    stop("wrong headstart hss")
  if ( rx < 5 )
    stop("rx is too small")
  if ( rs < 10 ) 
    stop("rs is too small")
  if ( qm < 5 ) 
    stop("qm is too small")
  ctyp <- 1 # later more
  arl <- .C("xsewma_res_arl",as.double(alpha),as.integer(n-1),as.integer(ctyp),
            as.double(lx),as.double(cx),as.double(hsx),as.integer(rx),
            as.double(ls),as.double(csu),as.double(hss),as.integer(rs),
            as.double(mu),as.double(sigma),as.integer(qm),
            ans=double(length=1),PACKAGE="spc")$ans 
  names(arl) <- "arl"
  return (arl)
}

imr.Rl_Mgiven <- function(M, L0, N=30, qm=30) {
  M0 <- qnorm( 1-1/(2*L0) )
  if ( M > M0 ) {
    zero <- function(x) imr.arl(M, 3*M, 0, 1, vsided="two", Rl=x, N=N, qm=qm) - L0
    Rl <- uniroot(zero, c(1e-6, 1), tol=1e-9)$root
  } else {
    Rl <- 0
  }
  Rl
}

imr.arl.Ny <- function(M, R, mu, N, kind="gl") {
  phij <- function(x,y) ifelse(abs(y-x)<=R, dnorm(y, mean=mu), 0)
  one <- rep(1, N)
  I <- diag(1, N)
  if ( kind == "gl" ) {
    GQ <- quadrature.nodes.weights(N, x1=-M, x2=M)
    z <- GQ$nodes
    w <- GQ$weights
  }
  if ( kind == "rectangular" ) {
    b <- 2*M/N
    z <- -M + (0:(N-1) + .5)*b
    w <- rep(b, N)
  }
  if ( kind %in% c("trapezoid", "simpson", "simpson3_8") ) {
    b <- 2*M/(N-1)
    z <- -M + (0:(N-1))*b
    w <- 1:N
  }
  if ( kind == "trapezoid" ) {
    w <- rep(2, N)
    w[c(1, N)] <- 1
    w <- w * b/2    
  }
  if ( kind == "simpson" ) {
    w <- 4*((w %% 2) == 0) + 2*((w %% 2) == 1)
    w[c(1, N)] <- 1
    w <- w * b/3    
  }
  if ( kind == "simpson3_8" ) {
    w <- 3*((w %% 3) != 1) + 2*((w %% 3) == 1)
    w[c(1, N)] <- 1
    w <- w * 3*b/8    
  }
  Q <- outer(z,z,phij)*t(array(w,c(N,N)))
  vLu <- solve(I-Q, one)
  Lu <- Vectorize(function(x) { 1 + ( phij(x,z) * w ) %*% vLu })
  integrand <- function(x) Lu(x) * dnorm(x, mean=mu)
  GQ2 <- quadrature.nodes.weights(200, x1=-M, x2=M)
  z2 <- GQ2$nodes
  w2 <- GQ2$weights
  ET <- 1 + sum( w2 * integrand(z2) )
  ET
}

imr.MandRuRl <- function(L0, N=30, qm=30) {
  Xarl <- function(M) 1/(2*pnorm(-M))
  MRarl <- function(Ru, Rl, N=N, qm=qm) imr.arl(12, Ru, 0, 1, vsided="two", Rl=Rl, N=N, qm=qm)
  M0 <- qnorm(1-1/(2*L0))
  M1 <- M0 + .1
  delta <- Vectorize(function(x) {
    LM <- Xarl(x)    
    RRR <- imr.RuRl_alone(L0, N=N, qm=qm, M0=x)    
    LR <- MRarl(RRR[2], RRR[1], N=N, qm=qm)
    LM - LR
  })
  D1 <- delta(M1)
  if ( D1 > 0 ) {
    while ( D1 > 0 ) {
      M2 <- M1
      M1 <- mean(c(M0,M1))
      D1 <- delta(M1)
    }
  } else {
    while ( D1 < 0 ) {
      M2 <- M1
      M1 <- M1 + 0.1
      D1 <- delta(M1)
    }
    while ( D1 > 0 ) {
      M2 <- M1
      M1 <- M1 - 0.02
      D1 <- delta(M1)
    }
  }
  M <- uniroot(delta, c(M1, M2), tol=1e-9)$root
  RRR <- imr.RuRl_alone(L0, N=N, qm=qm, M0=M)
  Rl <- RRR[1]
  Ru <- RRR[2]
  c(M, Rl, Ru)
}

# Computation of EWMA steady-state ARLs (mean monitoring)
xewma.ad <- function(l, c, mu1, mu0=0, zr=0, z0=0, sided="one", limits="fix", steady.state.mode="conditional", r=40) { 
  if ( l<=0 || l>1 ) 		stop("l has to be between 0 and 1")
  
  if ( c<=0 )			warning("usually, c has to be positive")
  
  if ( zr>c & sided=="one" )    stop("wrong reflexion border")
  
  if ( r<4 )			stop("r is too small")
  
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if (is.na(ctyp))		stop("invalid ewma type")
  
  ltyp <- pmatch(limits, c("fix","vacl","fir","both","Steiner","stat")) - 1
  if ( is.na(ltyp) )		stop("invalid limits type")
  
  if ( (sided=="one") & !(limits %in% c("fix", "vacl", "stat")) )
				stop("not supported for one-sided EWMA (not reasonable or not implemented yet")

  styp <- pmatch(steady.state.mode, c("conditional", "cyclical")) - 1
  if (is.na(styp))		stop("invalid steady.state.mode")
  
  if ( abs(z0) > abs(c) ) 	stop("wrong restarting value")
  
  ad <- .C("xewma_ad", as.integer(ctyp), as.double(l),
           as.double(c), as.double(zr), as.double(mu0), as.double(mu1), as.double(z0),
           as.integer(ltyp), as.integer(styp), as.integer(r),
           ans=double(length=1), PACKAGE="spc")$ans 
  names(ad) <- "ad"
  return (ad)
}

# Computation of CUSUM decision intervals -- alarm thresholds --  (variance monitoring)
scusum.crit <- function(k, L0, sigma, df, hs=0, sided="upper", mode="eq.tails", k2=NULL, hs2=0, r=40, qm=30) {
  if ( k<0 ) 
    stop("k has to be non-negative")
  if ( L0<1 )
    stop("L0 is too small")
  if ( hs<0 ) 
    stop("wrong headstart")
  if ( sided=="two" ) {
    if ( is.null(k2) )
      stop("in case of a two-sided CUSUM scheme one has to define two reference values")
    if ( k2<0 ) 
      stop("k2 has to be non-negative")
    if ( hs2<0 ) 
      stop("wrong headstart")
  }
    
  if ( sigma<=0 )
    stop("sigma must be positive")
  if ( df<1 )
    stop("df must be larger than or equal to 1")

  ctyp <- pmatch(sided, c("upper", "lower", "two")) - 1
  if ( is.na(ctyp) )
    stop("invalid cusum type")
  ltyp <- pmatch(mode, c("eq.tails", "unbiased")) - 1
  if ( is.na(ltyp) )
    stop("invalid limits cusum type")
  if ( r<10 ) 
    stop("r is too small")
  if ( qm<10 ) 
    stop("qm is too small")
   
  a.length <- 1
  if ( sided=="two" ) a.length <- 2
    
  h <- .C("scusum_crit", as.integer(ctyp),
          as.double(k), as.double(L0), as.double(hs),
          as.double(sigma), as.integer(df), as.integer(ltyp),
          as.double(k2), as.double(hs2),
          as.integer(r), as.integer(qm),
          ans=double(length=a.length), PACKAGE="spc")$ans
          
  if ( sided=="two" )  {
    names(h) <- c("hl","hu")
  } else {
    names(h) <- "h"
  }
  
  h
}

# Computation of GRSR (Girshick, Rubin, Shiryaev, Roberts) alarm threshold for given ARL (mean monitoring)
xgrsr.crit <- function(k, L0, mu0=0, zr=0, hs=NULL, sided="one", MPT=FALSE, r=30) {
  if ( k<0 )	stop("k has to be non-negative")
  if ( L0<1 )	stop("L0 is too small")
  if ( !is.null(hs) ) {
    if ( hs>2*log(L0) )	stop("wrong headstart")
  } else {
    hs <- 2*L0
  }
  if ( r<4 )	stop("r is too small")
  
  g <- .C("xgrsr_crit",as.double(k),
          as.double(L0),as.double(zr),as.double(hs),as.double(mu0),as.integer(r),as.integer(MPT),
          ans=double(length=1),PACKAGE="spc")$ans 
          
  names(g) <- "g"
  return (g)
}

# Computation of CUSUM survival function (mean monitoring)
xcusum.sf <- function(k, h, mu, n, hs=0, sided="one", r=40) {
  if ( k < 0 )  stop("k has to be non-negative")
  if ( h <= 0 ) stop("h has to be positive")
  if ( hs < 0 | (sided=="two" & hs>h/2+k) | (sided=="one" & hs>h/2+k) ) stop("wrong headstart")
  if ( n < 1 ) stop("n has to be a natural number")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if ( is.na(ctyp) ) stop("invalid cusum type")
  if ( r < 4 ) stop("r is too small")
  sf <- .C("xcusum_sf",
   as.integer(ctyp), as.double(k), as.double(h), as.double(hs), as.double(mu), as.integer(r), as.integer(n),
              ans=double(length=n),PACKAGE="spc")$ans
  names(sf) <- NULL
  sf
}

# Computation of Poisson EWMA control limits
pois.ewma.crit <- function(lambda, L0, mu0, z0, AU=3, sided="two", design="sym", rando=FALSE, mcdesign="transfer", N=101, jmax=4, OLD=FALSE) {
  if ( lambda <= 0 | lambda > 1 )       stop("lambda has to be between 0 and 1")
  if ( L0 < 1 )                         stop("L0 has to be larger")
  if ( mu0 < 0 )     	                stop("wrong value for mu0")
  ctyp <- pmatch(sided, c("upper", "lower", "two")) - 1
  if ( is.na(ctyp) )                    stop("invalid ewma type")
  dtyp <- pmatch(design, c("sym", "unb")) - 1
  if ( is.na(dtyp) )                    stop("invalid design type")
  mcd <- pmatch(mcdesign, c("classic", "transfer")) - 1
  if ( is.na(mcd) )                    stop("invalid mcdesign value")
  LL <- ifelse(dtyp==0, 1, 2)
  if (dtyp==1 & rando) LL <- 4
  crit <- .C("cewma_crit_be",
             as.integer(ctyp), as.integer(dtyp), as.integer(mcd), as.integer(rando),
             as.double(lambda), as.double(L0), as.double(AU), as.double(mu0),
             as.double(z0), as.integer(N), as.integer(jmax), as.integer(OLD),
             ans=double(length=LL), PACKAGE="spc")$ans
  if (dtyp==1 & rando) {
    names(crit) <- c("AL", "AU", "gL", "gU")
  } else {
    if ( ctyp==0 ) names(crit) <- "AU"
    if ( ctyp==1 ) names(crit) <- "AL"
    if ( dtyp==0 ) names(crit) <- "A"
    if ( dtyp==1 ) names(crit) <- c("AL", "AU")
  }
  crit
}

xewma.q.crit <- function(l, L0, mu, alpha, zr=0, hs=0, sided="two", limits="fix", r=40, c.error=1e-12, a.error=1e-9, OUTPUT=FALSE) {

  c2 <- 0
  p2 <- 1
  if ( OUTPUT ) cat("\nc\t\tp\n")
  while ( p2 > alpha ) {
    p1 <- p2
    c2 <- c2 + .5
    p2 <- 1 - xewma.sf(l, c2, mu, L0, zr=zr, hs=hs, sided=sided, limits=limits, q=1, r=r)[L0]
    if ( OUTPUT ) cat(paste(c2,"\t",p2,"\n"))
  }
  while ( p2 <= alpha & c2 > .02 ) {
    p1 <- p2
    c2 <- c2 - .02
    p2 <- 1 - xewma.sf(l, c2, mu, L0, zr=zr, hs=hs, sided=sided, limits=limits, q=1, r=r)[L0]
    if ( OUTPUT ) cat(paste(c2,"\t",p2,"\n"))
  }
  c1 <- c2 + .02
  
  a.error_ <- 1; c.error_ <- 1
  while ( a.error_ > a.error & c.error_ > c.error ) {
    c3 <- c1 + (alpha - p1)/(p2 - p1)*(c2 - c1)
    p3 <- 1 - xewma.sf(l, c3, mu, L0, zr=zr, hs=hs, sided=sided, limits=limits, q=1, r=r)[L0]
    if ( OUTPUT ) cat(paste(c3,"\t",p3,"\n"))
    c1 <- c2; c2 <- c3
    p1 <- p2; p2 <- p3
    a.error_ <- abs(p2 - alpha); c.error_ <- abs(c2 - c1)
  }
  
  names(c3) <- "c"
  c3
}

# Computation of EWMA ARLs (drift monitoring)
xDewma.arl <- function(l, c, delta, zr=0, hs=0, sided="one", limits="fix", mode="Gan", m=NULL, q=1, r=40, with0=FALSE) {
  if (l<=0 || l>1) 
    stop("l has to be between 0 and 1")
  if (c<=0) 
    stop("c has to be positive")
  if (zr>c & sided=="one") 
    stop("wrong reflexion border")
  if ( (sided=="two" & abs(hs)>c) | (sided=="one" & (hs<zr | hs>c)) ) 
    stop("wrong headstart")
  if (r<4)
    stop("r is too small")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if (is.na(ctyp)) 
    stop("invalid ewma type")
  ltyp <- -1 + pmatch(limits, 
          c("fix","vacl","fir","both","Steiner","Knoth","fink","fixW","fixC"))
  if (is.na(ltyp))
    stop("invalid limits type")
  cmode <- pmatch(mode, c("Gan", "Knoth", "Waldmann")) - 1
  if (is.na(cmode))
    stop("invalid algorithm mode")
  if ( is.null(m) ) {
    m <- 0
  } else { if ( m<1 ) stop("m is too small") }
  q <- round(q)
  if (q<1)
    stop("wrong change point position (q)")
  arl <- .C("xDewma_arl",as.integer(ctyp),as.double(l),
            as.double(c),as.double(zr),as.double(hs),
            as.double(delta),as.integer(ltyp),as.integer(m),as.integer(r),
            as.integer(with0),as.integer(cmode),as.integer(q),
            ans=double(length=1),PACKAGE="spc")$ans 
  names(arl) <- "arl"
  return (arl)
}

# Computation of MEWMA threshold (multivariate mean monitoring)
mewma.crit <- function(l, L0, p, hs=0, r=20) {
  if ( l<=0 | l>1 )		stop("l has to be between 0 and 1")
  if ( L0<1 )			stop("L0 is too small")
  if ( p<1 )			stop("wrong dimension parameter")
  if ( hs<0 )			stop("wrong head start value")
  if ( r<4 )			stop("resolution too small")

  h <- .C("mewma_crit", as.double(l), as.double(L0),
                        as.integer(p), as.double(hs),
                        as.integer(r),
                        ans=double(length=1), PACKAGE="spc")$ans

  names(h) <- NULL
  h
}

# Computation of GRSR (Girshick, Rubin, Shiryaev, Roberts) ARLs (drift monitoring)
xDgrsr.arl <- function(k, g, delta, zr=0, hs=NULL, sided="one", m=NULL, mode="Gan", q=1, r=30, with0=FALSE) {
  if (k<0)
    stop("k has to be non-negative")
  if (g<=0)
    stop("g has to be positive")
  if (zr>g)
    stop("zr has to be smaller than g")
  if ( !is.null(hs) ) {
    if ( hs>g )
      stop("wrong headstart")
  } else {
    hs <- 2*g # mimics hs = -inf
  }
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if (is.na(ctyp))
    stop("invalid grsr type")
  if (r<4)
    stop("r is too small")
  if ( is.null(m) ) {
    m <- 0
  } else {
    if ( m<1 ) stop("m is too small") 
  }
  cmode <- pmatch(mode, c("Gan", "Knoth")) - 1
  if (is.na(cmode))
    stop("invalid algorithm mode")
  q <- round(q)
  if (q<1)
    stop("wrong change point position (q)")
  arl <- .C("xDgrsr_arl",as.double(k),
            as.double(g),as.double(zr),as.double(hs),as.double(delta),as.integer(m),
            as.integer(r),as.integer(with0),as.integer(cmode),as.integer(q),
            ans=double(length=1),PACKAGE="spc")$ans
  names(arl) <- "arl"
  return (arl)
}

# Computation of EWMA critical values for given ARL (mean monitoring)
xewma.crit <- function(l,L0,mu0=0,zr=0,hs=0,sided="one",limits="fix",r=40,c0=NULL,nmax=1e4) {
  if ( l<=0 | l>2 ) 
    stop("l has to be between 0 and 2")
  if ( L0<1 ) 
    stop("L0 is too small")
  if ( r<4 )
    stop("r is too small")
  if ( sided=="one" & hs<zr )
    warning("unusual headstart")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if ( is.na(ctyp) ) 
    stop("invalid ewma type")
  ltyp <- pmatch(limits, c("fix","vacl","fir","both","Steiner","stat","cfar")) - 1
  if ( is.na(ltyp) ) 
    stop("invalid limits type")
  if ( (sided=="one") & !(limits %in% c("fix", "vacl", "stat")) )
    stop("not supported for one-sided EWMA (not reasonable or not implemented yet")
  if ( is.null(c0) ) {
    if ( sided=="one" ) c0 <- zr - 1
    if ( sided=="two" ) c0 <- -1
  }
  if (limits=="cfar") {
    na <- nmax+1
    ctyp <- na
    zero <- function(x) L0 - xewma.ad(l, x, 0, sided="two", r=r)
    c1 <- 1
    if ( zero(c1) > 0 ) {
      while ( zero(c1) > 0 ) c1 <- c1 + 1
      c0 <- c1 - 1
    } else {
      while ( zero(c1) < 0 ) c1 <- c1 / 1.2
      c0 <- c1
      c1 <- c1 * 1.2
    }
    hs <- uniroot(zero, c(c0,c1), tol=1e-9)$root  
    hs <- sqrt( l/(2-l) ) * hs
  } else {
    na <- 1
  }
  cE <- .C("xewma_crit",as.integer(ctyp),as.double(l),
           as.double(L0),as.double(zr),as.double(hs),
           as.double(mu0),as.integer(ltyp),as.integer(r),
           as.double(c0),
           ans=double(length=na),PACKAGE="spc")$ans 
  if (limits=="cfar") {
    nm <- as.integer(cE[1])
    cE <- cE[2:(nm+1)]
    names(cE) <- NULL
  } else {
    names(cE) <- "cE"
  }
  return (cE)
}

# Computation of CUSUM k (reference value) for given in-control ARL and threshold h (mean monitoring)

xcusum.crit.L0h <- function(L0, h, hs=0, sided="one", r=30, L0.eps=1e-6, k.eps=1e-8) {
  h.max <- xcusum.crit(0, L0, 0)
  if ( h.max < h ) stop("h too large  or  L0 far too small")
  k1 <- 0
  L0_1 <- 0
  while ( L0_1 < L0 ) {
    k1 <- k1 + .1
    L0_1 <- xcusum.arl(k1, h, 0, hs=hs, sided=sided, r=r)
  } 
  while ( L0_1 > L0 & k1 > 0.01) {
    k1 <- k1 - .01
    L0_1 <- xcusum.arl(k1, h, 0, hs=hs, sided=sided, r=r)
  }
  k2 <- k1 + .01
  L0_2 <- xcusum.arl(k2, h, 0, hs=hs, r=r)
  dk <- 1
  while ( abs(L0-L0_2) > L0.eps  &  abs(dk) > k.eps  ) {
    k3 <- k1 + ( L0 - L0_1 ) / ( L0_2 - L0_1 ) * ( k2 - k1 )
    L0_3 <- xcusum.arl(k3, h, 0, hs=hs, sided=sided, r=r) 
    # secant rule
    dk <- k3-k2
    k1 <- k2
    L0_1 <- L0_2
    k2 <- k3
    L0_2 <- L0_3
  }
  k3
}

quadrature.nodes.weights <- function(n, type="GL", x1=-1, x2=1) {
  if ( n < 1 )			stop("n has to be a natural number")
  qtyp <- pmatch(type, c("GL", "Ra")) - 1
  if ( is.na(qtyp) )		stop("invalid quadrature type")
  if ( x1 >= x2 )		stop("x1 must be smaller than x2")
  
  nw <- .C("quadrature_nodes_weights",
           as.integer(n),
           as.double(x1), as.double(x2),
           as.integer(qtyp),
           ans=double(length=2*n), PACKAGE="spc")$ans
           
  qnw <- data.frame(nodes=nw[1:n], weights=nw[-(1:n)])         
  qnw
}

# Computation of CUSUM-Shewhart ARLs (variance monitoring)
scusums.arl <- function(k, h, cS, sigma, df, hs=0, sided="upper", k2=NULL, h2=NULL, hs2=0, r=40, qm=30, version=2) {
  if ( k<0 ) 
    stop("k has to be non-negative")
  if ( h<=0 ) 
    stop("h has to be positive")
  if ( cS<=0 ) 
    stop("cS has to be positive")
  if ( hs<0 | hs>h ) 
    stop("wrong headstart")
  if ( sided=="two" ) {
    if ( is.null(k2) | is.null(h2) )
      stop("in case of a two-sided CUSUM scheme one has to define two sets of (k,h,hs)")
    if ( k2<0 ) 
      stop("k2 has to be non-negative")
    if ( h2<=0 ) 
      stop("h2 has to be positive")
    if ( hs2<0 | hs2>h2 ) 
      stop("wrong headstart")
  }
    
  if ( sigma<=0 )
    stop("sigma must be positive")
  if ( df<1 )
    stop("df must be larger than or equal to 1")

  ctyp <- pmatch(sided, c("upper", "lower", "two")) - 1
  if ( is.na(ctyp) )
    stop("invalid cusum type")
  if ( r<10 ) 
    stop("r is too small")
  if ( qm<10 ) 
    stop("qm is too small")

  arl <- .C("scusum_s_arl", as.integer(ctyp),
            as.double(k), as.double(h), as.double(hs), as.double(cS),
            as.double(sigma), as.integer(df),
            as.double(k2), as.double(h2), as.double(hs2),
            as.integer(r), as.integer(qm), as.integer(version),
            ans=double(length=1), PACKAGE="spc")$ans 
  names(arl) <- "arl"
  
  arl
}

imr.RuRl_alone <- function(L0, N=30, qm=30, M0=12, eps=1e-3) {
  zero <- Vectorize(function(x) {
    Ru1 <- imr.Ru_Rlgiven(x, L0, N=N, qm=qm, M0=M0)
    Lp <- imr.arl(M0, Ru1, 0, 1+eps, vsided="two", Rl=x, N=N, qm=qm)
    Lm <- imr.arl(M0, Ru1, 0, 1-eps, vsided="two", Rl=x, N=N, qm=qm)
    DELTA <- (Lp-Lm)/(2*eps)
  })
  pivot <- imr.Rl_Mgiven(M0, L0, N=N, qm=qm)
  Rl1 <- pivot / 2
  D1 <- zero(Rl1)
  Rl <- 1
  if ( D1 < 0 ) {
    while ( D1 < 0 & Rl1 < pivot/1.1 ) {
      Rl2 <- Rl1
      Rl1 <- Rl1 * 1.1
      D1 <- zero(Rl1)
      D1
    }
    if ( D1 > 0 ) {
      D1 <- Rl1
      Rl1 <- Rl2
      Rl2 <- D1
    } else {
      Rl <- 0
    }
  } else {
    while ( D1 > 0 ) {
      Rl2 <- Rl1
      Rl1 <- Rl1 / 1.1
      D1 <- zero(Rl1)
    }
  }
  if ( Rl > 0 ) {
    Rl <- uniroot(zero, c(Rl1, Rl2), tol=1e-9)$root
    Ru <- imr.Ru_Rlgiven(Rl, L0, N=N, qm=qm, M0=M0) 
  } else {
    Ru <- 3*M0 # like infinity
    Rl <- pivot
  }
  c(Rl, Ru)
}

# Computation of the ARL for modified Shewhart charts, AR(1) data
xshewhart.ar1.arl <- function(alpha, cS, delta=0, N1=50, N2=30) {
  if ( abs(alpha) >= 1 )    stop("alpha has to be between -1 and 1")
  if ( cS <= 0 )            stop("cS has to be positive")
  arl <- .C("xshewhart_ar1_arl",
              as.double(alpha), as.double(cS), as.double(delta),
              as.integer(N1), as.integer(N2), ans=double(length=1), PACKAGE="spc")$ans
  names(arl) <- NULL
  arl
}

# Computation of Poisson EWMA steady-state ARLs
pois.ewma.ad <- function(lambda, AL, AU, mu0, mu, sided="two", rando=FALSE, gL=0, gU=0, mcdesign="classic", N=101, OLD=FALSE) {
  if ( lambda <= 0 | lambda > 1 )       stop("lambda has to be between 0 and 1")
  if ( AL < 0 | AU < 0 )                stop("control limit factors must be positive")
  if ( mu0 < 0 )     	                stop("wrong value for mu0")
  if ( mu < 0 )     	                stop("wrong value for mu")
  hL  <- mu0 - AL*sqrt(lambda*mu0/(2-lambda))
  hU  <- mu0 + AU*sqrt(lambda*mu0/(2-lambda))
  ctyp <- pmatch(sided, c("upper", "lower", "two")) - 1
  if ( is.na(ctyp) )                    stop("invalid ewma type")
  mcd <- pmatch(mcdesign, c("classic", "transfer")) - 1
  if ( is.na(mcd) )                    stop("invalid mcdesign value")
  if ( rando ) {
    if ( gL < 0 | gL > 1 )              stop("wrong value for gL") 
    if ( gU < 0 | gU > 1 )              stop("wrong value for gU")
  }
  ad <- .C("cewma_ad_be",
           as.integer(ctyp), as.integer(mcd), as.integer(rando), as.double(lambda),
           as.double(AL), as.double(AU), as.double(gL), as.double(gU),
           as.double(mu0), as.double(mu), as.integer(N), as.integer(OLD),
           ans=double(length=1), PACKAGE="spc")$ans 
  names(ad) <- "ad"
  ad
}

# Computation of res-EWMA ARLs (scale monitoring)
s.res.ewma.arl <- function(l,cu,sigma,mu=0,alpha=0,n=5,hs=1,r=40,qm=30) {
  if ( l <= 0 || l > 1 )
    stop("l has to be between 0 and 1")
  if ( cu <= 0 )
    warning("usually, cu has to be positive")
  if ( sigma <= 0 )
    stop("sigma must be positive")
  if ( abs(alpha) > 1 )
    warning("nonstationary AR(1) process")
  if ( n < 2 )
    warning("n is too small")
  n <- round(n)
  if ( abs(hs) > cu ) 
    warning("unusual headstart")
  if ( r < 4 )
    stop("r is too small")
  if ( qm < 10 ) 
    stop("qm is too small")
  ctyp <- 1 # later more
  arl <- .C("s_res_ewma_arl",as.double(alpha),as.integer(n-1),
            as.integer(ctyp),as.double(l),
            as.double(cu),as.double(hs),
            as.double(sigma),as.double(mu),as.integer(r),as.integer(qm),
            ans=double(length=1),PACKAGE="spc")$ans 
  names(arl) <- "arl"
  return (arl)
}

# Computation of EWMA ARLs (mean monitoring), returns function
xewma.arl.f <- function(l, c, mu, zr=0, sided="one", limits="fix", r=40) {
  if ( l<=0 | l>1 )             stop("l has to be between 0 and 2")
  if ( c<=0 )                   warning("usually, c has to be positive")
  if ( zr>c & sided=="one" )    stop("wrong reflexion border")
  if ( r<4 )                    stop("r is too small")
  ctyp <- pmatch(sided, c("one", "two")) - 1
  if ( is.na(ctyp) )            stop("invalid ewma type")
  ltyp <- -1 + pmatch(limits, c("fix", "vacl", "fir", "both", "Steiner", "stat", "fink", "limit", "fixW", "fixC"))
  if ( is.na(ltyp) )            stop("invalid limits type")
  if ( (sided=="one") & !(limits %in% c("fix", "vacl", "stat", "limit", "fixW")) )
                                stop("not supported for one-sided EWMA (not reasonable or not implemented yet")

  LENGTH <- 3*r
  zeug <- .C("xewma_arl_f", as.integer(ctyp), as.double(l), as.double(c), as.double(zr),
                            as.double(mu), as.integer(ltyp), as.integer(r),
                            ans=double(length=LENGTH), PACKAGE="spc")$ans
                            
  g <- zeug[1:r]                          
  w <- zeug[1:r + r]
  z <- zeug[1:r + 2*r]
   
  arl <- Vectorize( function(x) 1 + sum( w * dnorm( ( z - (1-l)*x ) / l - mu)/l * g )  )
  
  arl   
}

imr.Ru_Rlgiven <- function(Rl, L0, N=30, qm=30, M0=12) {
  zero <- function(x) imr.arl(M0, x, 0, 1, vsided="two", Rl=Rl, N=N, qm=qm) - L0
  Ru1 <- sqrt(2) * qnorm(1-1/(4*L0))
  z1 <- zero(Ru1)
  Ru <- 0
  if ( z1 > 0 ) {
    while ( z1 > 0 ) {
      Ru2 <- Ru1
      Ru1 <- Ru1 / 1.1
      z1 <- zero(Ru1)
    }
  } else {
    if ( zero(2*M0-1e-10) > 0 ) {
      while ( z1 < 0 ) {
        Ru2 <- Ru1
        Ru1 <- Ru1 * 1.1
        z1 <- zero(Ru1)
      }
      z1 <- Ru1
      Ru1 <- Ru2
      Ru2 <- z1
    } else {
      Ru <- Inf
    }
  }
  if ( is.finite(Ru) ) Ru <- uniroot(zero, c(Ru1, Ru2), tol=1e-9)$root
  Ru
}

imr.MandRu <- function(L0, N=30, qm=30) {
  Xarl <- function(M) 1/(2*pnorm(-M))
  MRarl <- function(Ru, N=20, qm=30) ifelse(Ru<100, imr.arl(10, Ru, 0, 1, N=N, qm=qm), Inf)
  M0 <- qnorm(1-1/(2*L0))
  M1 <- M0 + .1
  delta <- Vectorize(function(x) {
    LM <- Xarl(x)
    Ru <- imr.Ru_Mgiven(x, L0, N=N, qm=qm)
    LR <- MRarl(Ru, N=N, qm=qm)
    LM - LR
  })
  D1 <- delta(M1)
  if ( D1 > 0 ) {
    while ( D1 > 0 ) {
      M2 <- M1
      D2 <- D1
      M1 <- mean(c(M0,M1))
      D1 <- delta(M1)
    }
    M <- uniroot(delta, c(M1, M2), tol=1e-9)$root
  } else {
    while ( D1 < 0 ) {
      M2 <- M1
      D2 <- D1
      M1 <- M1 + .1
      D1 <- delta(M1)
    }
    M <- uniroot(delta, c(M2, M1), tol=1e-9)$root
  }
  Ru <- imr.Ru_Mgiven(M, L0, N=N, qm=qm)
  c(M, Ru)
}

# Computation of CUSUM steady-state ARLs (mean monitoring)
xcusum.ad <- function(k, h, mu1, mu0 = 0, sided = "one", r = 30) {
  if (k<0) 
    stop("k has to be non-negative")
  if (h<=0) 
    stop("h has to be positive")
  if (r<4) 
    stop("r is too small")
  if (r>30 & r<=50 & sided=="two") 
    warning("computation needs some time")
  if (r>50 & sided=="two") 
    warning("ought to be restricted to very fast CPUs")
  ctyp <- pmatch(sided, c("one", "two", "Crosier")) - 1
  if (is.na(ctyp)) 
    stop("invalid cusum type")
  ad <- .C("xcusum_ad",as.integer(ctyp),as.double(k),
           as.double(h),as.double(mu0),as.double(mu1),as.integer(r),
           ans=double(length=1),PACKAGE="spc")$ans 
  names(ad) <- "ad"
  return(ad)
}

# Computation of EWMA phat ARLs
phat.ewma.arl <- function(lambda, ucl, mu, n, z0, sigma=1, type="known", LSL=-3, USL=3, N=15, qm=25, ntype="coll") {
  if ( lambda <= 0 || lambda > 1 )
    stop("lambda has to be between 0 and 1")
  p.star <- pnorm( LSL ) + pnorm( -USL )
  if ( type == "known" ) {
    if ( ucl <= p.star )
      stop("ucl must be larger than p.star")
  }
  if ( type == "estimated" ) {
    p.star <- 0
    if ( ucl <= 0 )
      stop("ucl must be positive")
  }
  if ( ucl >= 1 )
    stop("ucl must be smaller than 1")
  if ( n < 1 )
    stop("n must be >= 1")
  if ( z0 < p.star | z0 > ucl )
    stop("wrong headstart")
  if ( sigma<1e-12 )
    stop("sigma much too small")
  ctyp <- -1 + pmatch(tolower(type), c("known", "estimated"))
  if ( is.na(ctyp) )
    stop("invalid sigma mode")
  if ( LSL >= USL )
    stop("wrong relationship between lower and upper specification limits (LSL must be smaller than USL)")
  if ( N < 2 )
    stop("N too small")
  if ( qm < 5 )
    stop("qm too small")
  ntyp <- -1 + pmatch(tolower(ntype), c("coll", "markov"))
  if ( is.na(ntyp) )
    stop("wrong label for numerical algorithm")    
  arl <- .C("ewma_phat_arl_coll",
            as.double(lambda), as.double(ucl), as.double(mu), as.double(sigma), as.integer(n),
            as.double(z0), as.integer(ctyp), as.double(LSL), as.double(USL),
            as.integer(N), as.integer(qm), as.integer(ntyp),
            ans=double(length=1), PACKAGE="spc")$ans 
  names(arl) <- "arl"
  arl
}

# Computation of EWMA ARLs (simultaneous mean & variance monitoring)
xsewma.arl <- function(lx, cx, ls, csu, df, mu, sigma,
                       hsx=0, Nx=40,
                       csl=0, hss=1, Ns=40,
                       s2.on=TRUE, sided="upper", qm=30) {
  if (lx<=0 | lx>1) 
    stop("lx has to be between 0 and 1")
  if (ls<=0 | ls>1)
    stop("ls has to be between 0 and 1")
  if (cx<=0)
    stop("cx has to be positive")
  if (csu<=0) 
    stop("csu has to be positive")
  if (csl<0)
    stop("clu has to be non-negative")
  if ( sigma<=0 )
    stop("sigma must be positive")
  if ( df<1 )
    stop("df must be larger than or equal to 1")
  s_squared <- as.numeric(s2.on)
  if ( !(s_squared %in% c(0,1)) )
    stop("wrong value for s2.on")
  if ( abs(hsx)>cx )
    stop("wrong headstart hsx")
  if ( hss<csl | hss>csu ) 
    stop("wrong headstart hss")
  if (Nx<5)
    stop("Nx is too small")
  if (Ns<10) 
    stop("Ns is too small")
  if (qm<5) 
    stop("qm is too small")
  ctyp <- pmatch(sided, c("upper","Rupper","two","lower")) - 1
  if (is.na(ctyp)) 
    stop("invalid ewma type")
  arl <- .C("xsewma_arl",as.integer(ctyp),
            as.double(lx),as.double(cx),as.double(hsx),as.integer(Nx),
            as.double(ls),as.double(csl),as.double(csu),as.double(hss),
            as.integer(Ns),
            as.double(mu),as.double(sigma),
            as.integer(df),as.integer(qm),
            as.integer(s_squared),
            ans=double(length=1),PACKAGE="spc")$ans 
  names(arl) <- "arl"
  return (arl)
}

# Computation of CUSUM decision limits for given ARL (mean monitoring)
xcusum.crit <- function(k, L0, mu0 = 0, hs = 0, sided = "one", r = 30) {
  if (k<0)
    stop("k has to be non-negative")
  if (L0<1) 
    stop("L0 is too small")
  if (hs<0) 
    stop("wrong headstart")
  if (r<4) 
    stop("r is too small")
  ctyp <- pmatch(sided, c("one", "two", "Crosier")) - 1
  if (is.na(ctyp)) 
    stop("invalid cusum type")
  h <- .C("xcusum_crit",as.integer(ctyp),as.double(k),
          as.double(L0),as.double(hs),as.double(mu0),as.integer(r),
          ans=double(length=1),PACKAGE="spc")$ans 
  names(h) <- "h"
  return (h)
}

imr.arl.C1987b <- function(M, R, mu, sig, N=80) {
  # original FORTRAN code re-coded in R
  H <- 2*M/N
  NN <- N + 1
  B <- rep(-1, NN)
  A <- matrix(NA, nrow=NN, ncol=NN)
  for ( i in 1:NN ) {
    U <- -M + H*(i-1)
    Bi <- min( M, U+R )
    Ai <- max(-M, U-R )
    N1 <- floor( (Ai+M)/H + 1e-9 ) + 1
    N2 <- floor( (Bi+M)/H + 1e-9 ) + 1
    for ( j in 1:NN ) {
      ARG <- Ai + H*(j-N1)
      A[i,j] <- H * dnorm(ARG, mean=mu, sd=sig)
      if ( j  < N1 ) A[i,j] <- 0
      if ( j == N1 ) A[i,j] <- H/2 * dnorm(Ai, mean=mu, sd=sig)
      if ( j == N2 ) A[i,j] <- H/2 * dnorm(Bi, mean=mu, sd=sig)
      if ( j  > N2 ) A[i,j] <- 0
      if ( j == i  ) A[i,j] <- A[i,j] - 1
    }
  }
  X <- solve(A, B)
  ET <- H/2 * ( dnorm(-M, mean=mu, sd=sig)*X[1] + dnorm(M, mean=mu, sd=sig)*X[NN] ) + 1
  for ( i in 2:N ) {
    ARG <- -M + H*(i-1)
    ET <- ET + H*X[i]*dnorm(ARG, mean=mu, sd=sig)
  }
  ET
}

# Computation of EWMA ARLs (variance monitoring) based on ln S^2
lns2ewma.arl <- function(l, cl, cu, sigma, df, hs=NULL, sided="upper", r=40) {

  #mitte <- -1/df - 1/3/df^2 + 2/15/df^4 # approx following Crowder/Hamilton
  mitte <- log(2/df) + digamma(df/2)
  
  if ( is.null(cl) ) cl <- mitte
  
  if ( is.null(cu) ) cu <- mitte
  
  if ( is.null(hs) ) hs <- mitte
  
  if ( l<=0 || l>1 ) stop("l has to be between 0 and 1")
  
  #if ( cu < mitte )  stop(paste("cu has to be larger than", mitte))
  #if ( cl > mitte )  stop(paste("cl has to be smaller than", mitte))
  
  if ( sigma<=0 ) stop("sigma must be positive")
  
  if ( df<1 ) stop("df must be larger than or equal to 1")
  
  if ( hs<cl-1e-9 | hs>cu+1e-9 ) stop("wrong headstart")
  
  ctyp <- pmatch(sided, c("upper", "lower", "two")) - 1
  if ( is.na(ctyp) ) stop("invalid ewma type")
  
  if ( r<10 ) stop("r is too small")
  
  arl <- .C("lns2ewma_arl", as.integer(ctyp), as.double(l),
            as.double(cl), as.double(cu), as.double(hs),
            as.double(sigma), as.integer(df), as.integer(r),
            ans=double(length=1),PACKAGE="spc")$ans 
            
  names(arl) <- "arl"
  return (arl)
}

# Computation of EWMA critical values for given ARL (variance monitoring) based on ln S^2
lns2ewma.crit <- function(l, L0, df, sigma0=1, cl=NULL, cu=NULL, hs=NULL, sided="upper", mode="fixed", r=40) {

  #mitte <- -1/df - 1/3/df^2 + 2/15/df^4 # approx following Crowder/Hamilton
  mitte <- log(2/df) + digamma(df/2)
  
  if ( is.null(hs) ) hs <- mitte
  cu0 <- cl0 <- 0
  
  if ( l<=0 || l>1 ) stop("l has to be between 0 and 1")
  
  if ( L0<1 ) stop("L0 is too small")
  
  if ( df<1 ) stop("df must be positive")
  
  if ( sigma0<=0 ) stop("sigma0 must be positive")
    
  if ( sided=="upper" ) {
    if ( is.null(cl) ) cl <- mitte
    #if ( cl > mitte + 1e-9 ) stop(paste("cl has to be smaller than", mitte))
    cl0 <- cl
    if ( hs<cl0-1e-9 ) stop("hs must not be smaller than cl")
  }
  if ( sided=="lower" ) {
    if ( is.null(cu) ) cu <- mitte
    #if ( cu < mitte - 1e-9 ) stop(paste("cu has to be larger than", mitte))
    cu0 <- cu
    if ( hs>cu0+1e-9 ) stop("hs must not be larger than cu")
  }
  if (sided=="two" & mode=="fixed") {
    if ( is.null(cu) ) stop("set cu")
    #if ( cu < mitte - 1e-9 ) stop(paste("cu has to be larger than", mitte))
    cu0 <- cu
    if ( hs>cu0+1e-9 ) stop("hs must not be larger than cu")
  }
  
  ctyp <- pmatch(sided, c("upper", "lower", "two")) - 1
  if (is.na(ctyp)) stop("invalid ewma type")
  
  ltyp <- pmatch(mode, c("fixed", "unbiased", "eq.tails", "vanilla")) - 1
  if (is.na(ltyp)) stop("invalid limits type")
  
  if ( r<10 ) stop("r is too small")
  
  c <- .C("lns2ewma_crit", as.integer(ctyp), as.integer(ltyp), as.double(l),
          as.double(L0), as.double(cl0), as.double(cu0), as.double(hs),
          as.double(sigma0), as.integer(df), as.integer(r),
          ans=double(length=2),PACKAGE="spc")$ans
          
  names(c) <- c("cl", "cu")
  return (c)
}

# Computation of MEWMA ARLs (multivariate mean monitoring), returns function
mewma.arl.f <- function(l, cE, p, delta=0, r=20, ntype=NULL, qm0=20, qm1=qm0) {
  if ( l<=0 | l>1 )		stop("l has to be between 0 and 1")
  if ( cE<=0 )			stop("threshold c has to be positive")
  if ( p<1 )			stop("wrong dimension parameter")
  if ( delta<0 )		stop("wrong magnitude value")
  
  if ( r<4 )			stop("resolution too small")
  if ( qm0<5 )			stop("more quadrature nodes needed")
  if ( qm1<5 )			stop("more quadrature nodes needed")
  
  if ( is.null(ntype) ) {
    if ( delta <1e-10 ) {
      ntype <- "gl2"
    } else {
      #if ( p %in% c(2,4) ) {
      if ( p==2 ) {
        ntype <- "gl3"
      } else {
        ntype <- "gl5"
      }
    }
  }
  
  # collocation basis of Chebshev polynomials
  Tn <- Vectorize(function(z, n) {
    if ( n==0 ) result <- 1
    if ( n==1 ) result <- z
    if ( n==2 ) result <- 2*z^2 - 1
    if ( n==3 ) result <- 4*z^3 - 3*z
    if ( n==4 ) result <- 8*z^4 - 8*z^2 + 1
    if ( n==5 ) result <- 16*z^5 - 20*z^3 + 5*z
    if ( n>5 )  result <- cos( n*acos(z) ) 
    result
  })
  
  qtyp <- pmatch(tolower(ntype), c("gl", "co", "ra", "cc", "mc", "sr", "co2", "gl2", "gl3", "gl4", "gl5", "co3", "co4", "ngl1", "ngl2", "ngl3", "ngl4", "ngl5")) - 1
  if ( is.na(qtyp) )		stop("invalid type of numerical algorithm")

  if ( abs(delta) < 1e-10 ) { # in-control    
    LENGTH <- 3*r
    zeug <- .C("mewma_arl_f", as.double(l), as.double(cE), as.integer(p), as.double(delta),
                              as.integer(r), as.integer(qtyp), as.integer(qm0), as.integer(qm1),
                              ans=double(length=LENGTH), PACKAGE="spc")$ans
                              
    g <- zeug[1:r]                          
    w <- zeug[1:r + r]
    z <- zeug[1:r + 2*r]
    
    # helper functions
    cE <- cE * l/(2-l)
    l2 <- ( (1-l)/l )^2 
    fchi <- function(a, u) dchisq( u/l^2, p, ncp=l2*a ) / l^2
    FCHI <- function(a, u) pchisq( u/l^2, p, ncp=l2*a )
    
    if ( qtyp %in% c(0,2,5) ) arl <- Vectorize(function(a) 1 + sum( w * fchi(a, z) * g ), "a") # ordinary GL or Radau or Simpson rule Nystroem 
    if ( qtyp==7 ) arl <- Vectorize(function(a) 1 + sum( w * fchi(a, z^2) * g * 2*z ), "a") # GL Nystroem with ()^2 substitution
    if ( qtyp==1 ) arl <- Vectorize(function(a) sum( Tn( (2*a-cE)/cE, 0:(r-1) ) * g ), "a") # collocation
    if ( qtyp==3 ) arl <- Vectorize(function(a) 1 + sum( w * fchi(a, z^2) * g ) * cE/2 , "a") # Clenshaw-Curtis
    if ( qtyp==4 ) arl <- Vectorize(function(a) 1 + sum( ( FCHI(a, z^2) - FCHI(a, c(0, z[-length(z)]^2)) ) * g), "a") # Markov chain (Runger/Prabhu)
    
  } else { # out-of-control
    if ( qtyp==4 ) {
      cE_ <- sqrt( cE * l/(2-l) )
      w  <- 2*cE_/( 2*r + 1 )
      ii <- function(ix, iy) (ix-r)^2 + iy^2 < cE_^2/w^2
      CIRC <- as.vector( t(outer( 0:(2*r), 0:r, ii)) )
      dQ <- sum(CIRC)
      LENGTH <- dQ
    } else {
      r2 <- r^2
      LENGTH <- r2 + 4*r
    }
    
    zeug <- .C("mewma_arl_f", as.double(l), as.double(cE), as.integer(p), as.double(delta),
                              as.integer(r), as.integer(qtyp), as.integer(qm0), as.integer(qm1),
                              ans=double(length=LENGTH), PACKAGE="spc")$ans                              
                              
    if ( qtyp!=4 ) {
      g  <- zeug[1:r2]
      w0 <- zeug[1:r + r2]
      z0 <- zeug[1:r + r2 + r]
      w1 <- zeug[1:r + r2 + 2*r]
      z1 <- zeug[1:r + r2 + 3*r]
    } else {
      g <- zeug[1:dQ]
    }
    
    # helpers
    l2 <- ( (1-l)/l )^2
    h <- cE * l/(2-l)
    rdc <- l * sqrt( delta/h )
    sig <- l / sqrt( h )
    
    lsd  <- l * sqrt( delta )    
    
    if ( qtyp %in% c(0, 2, 3, 5) ) arl <- Vectorize(function(a, b) { # ordinary GL or Radau or CC or Simpson rule Nystroem 
      if ( abs(h-a) < 1e-10 ) a_ <- 1 else a_ <- ( a - b^2/delta ) / ( h - b^2/delta )
      b_ <- b / sqrt( delta * h )
      m <- rdc + (1-l)*b_
      eta <- l2 * h * (1-b_^2) * a_
      if ( eta < 1e-10 ) eta <- 0
      result <- 1
      for ( i in 1:r ) {
        korr <- h * (1-z1[i]^2) / l^2 
        outer <- korr * w1[i] * dnorm( z1[i], mean=m, sd=sig)
        inner <- sum( w0 * dchisq( korr*z0, p-1, eta) * g[ (i-1)*r + 1:r ] )
        result <- result + inner * outer
      }
      result
    })
    
    if ( qtyp==7 ) arl <- Vectorize(function(a, b) { # GL Nystroem with ()^2 substitution
      if ( abs(h-a) < 1e-10 ) a_ <- 1 else a_ <- ( a - b^2/delta ) / ( h - b^2/delta )
      b_ <- b / sqrt( delta * h )
      m <- rdc + (1-l)*b_
      eta <- l2 * h * (1-b_^2) * a_
      if ( eta < 1e-10 ) eta <- 0
      result <- 1
      for ( i in 1:r ) {
        korr <- h * (1-z1[i]^2) / l^2 
        outer <- korr * w1[i] * dnorm( z1[i], mean=m, sd=sig)
        inner <- sum( w0 * dchisq( korr*z0^2, p-1, eta) * 2*z0 * g[ (i-1)*r + 1:r ] )
        result <- result + inner * outer
      }
      result
    })
        
    if ( qtyp==8 ) arl <- Vectorize(function(a, b) { # GL Nystroem with ()^2 plus sin() substitution
      if ( abs(h-a) < 1e-10 ) a_ <- 1 else a_ <- ( a - b^2/delta ) / ( h - b^2/delta )
      b_ <- b / sqrt( delta * h )
      m <- rdc + (1-l)*b_
      eta <- l2 * h * (1-b_^2) * a_
      if ( eta < 1e-10 ) eta <- 0
      result <- 1
      for ( i in 1:r ) {
        korr <- h * ( 1 - sin(z1[i])^2 ) / l^2 
        outer <- korr * w1[i] * dnorm( sin(z1[i]), mean=m, sd=sig) * cos(z1[i])
        inner <- sum( w0 * dchisq( korr*z0^2, p-1, eta) * 2*z0 * g[ (i-1)*r + 1:r ] )
        result <- result + inner * outer
      }
      result
    })
    
    if ( qtyp==9 ) arl <- Vectorize(function(a, b) { # GL Nystroem with ()^2 plus tan() substitution
      if ( abs(h-a) < 1e-10 ) a_ <- 1 else a_ <- ( a - b^2/delta ) / ( h - b^2/delta )
      b_ <- b / sqrt( delta * h )
      m <- rdc + (1-l)*b_
      eta <- l2 * h * (1-b_^2) * a_
      if ( eta < 1e-10 ) eta <- 0
      result <- 1
      for ( i in 1:r ) {
        korr <- h * ( 1 - tan(z1[i])^2 ) / l^2 
        outer <- korr * w1[i] * dnorm( tan(z1[i]), mean=m, sd=sig) / cos(z1[i])^2
        inner <- sum( w0 * dchisq( korr*z0^2, p-1, eta) * 2*z0 * g[ (i-1)*r + 1:r ] )
        result <- result + inner * outer
      }
      result
    })   
 
    if ( qtyp==10 ) arl <- Vectorize(function(a, b) { # GL Nystroem with ()^2 plus sinh() substitution
      norm <- sinh(1)
      if ( abs(h-a) < 1e-10 ) a_ <- 1 else a_ <- ( a - b^2/delta ) / ( h - b^2/delta )
      b_ <- b / sqrt( delta * h )
      m <- rdc + (1-l)*b_
      eta <- l2 * h * (1-b_^2) * a_
      if ( eta < 1e-10 ) eta <- 0
      result <- 1
      for ( i in 1:r ) {
        korr <- h * ( 1 - (sinh(z1[i])/norm)^2 ) / l^2         
        outer <- korr * w1[i] * dnorm( sinh(z1[i])/norm, mean=m, sd=sig) * cosh(z1[i])/norm
        inner <- sum( w0 * dchisq( korr*z0^2, p-1, eta) * 2*z0 * g[ (i-1)*r + 1:r ] )
        result <- result + inner * outer
      }
      result
    })
    
    if ( qtyp %in% c(1, 6, 11, 12) ) arl <- Vectorize(function(a, b) { # collocation 
      if ( abs(h-a) < 1e-10 ) a_ <- 1 else a_ <- ( a - b^2/delta ) / ( h - b^2/delta )
      b_ <- b / sqrt( delta * h )
      result <- 0
      for ( i in 1:r ) {
        outer <- Tn( 2*a_-1, i-1 ) 
        inner <- sum( Tn( b_, 0:(r-1)) * g[ (i-1)*r + 1:r ] )
        result <- result + inner * outer
      }
      result
    })
    
    if ( qtyp==4 ) arl <- Vectorize(function(a, b) { # Markov chain (Runger/Prabhu)
      #a   <- sqrt(a)
      cE_ <- sqrt( cE * l/(2-l) )
      w  <- 2*cE_/( 2*r + 1 )
      wl <- w^2 / l^2
      ii <- function(ix, iy) (ix-r)^2 + iy^2 < cE_^2/w^2
      Vf <- function(iy,jy) pchisq( (jy+.5)^2*wl, p-1, ncp=(iy*w)^2*l2) - as.numeric(jy>0)*pchisq( (jy-.5)^2*wl, p-1, ncp=(iy*w)^2*l2)
      Hf <- function(ix,jx) pnorm( (-cE_+(jx+1)*w-(1-l)*(-cE_+(ix+.5)*w) )/l, mean=delta) - pnorm( (-cE_+jx*w-(1-l)*(-cE_+(ix+.5)*w) )/l, mean=delta)
      CIRC <- as.vector( t(outer( 0:(2*r), 0:r, ii)) )
      Vv <- Vf( a/w, 0:r )
      Hv <- Hf( (b+cE_)/w-.5, 0:(2*r) )
      dQ <- sum(CIRC)
      Qv <- as.vector( Vv %o% Hv )
      Qv_ <- Qv[ CIRC ]
      result <- 1 + sum( Qv_ * g )
    })
    
    if ( qtyp %in% c(13, 14, 15, 16, 17) ) arl <- Vectorize(function(a, b) { # new GL designs
      gam <- 0
      #if ( a > 0 ) gam <- b / sqrt( a * delta )
      if ( a > 0 ) gam <- b / sqrt( a )
      mij <- lsd + (1 - l ) * sqrt(a) * gam
      ncpij <- l2 * a * ( 1 - gam^2 )
      norm <- sinh(1)
      result <- 1
      for ( i in 1:r ) {
        if ( qtyp == 13 ) korr <- w0[i] * sqrt( z0[i] ) / l^2
        if ( qtyp > 13 ) korr <- 2 * w0[i] * z0[i]^2 / l^2
        wl <- z1
        korr2 <- rep(1, r)
        if ( qtyp == 15 ) { wl <- sin( z1 ); korr2 <- cos( z1 ) }
        if ( qtyp == 16 ) { wl <- tan( z1 ); korr2 <- 1 / ( cos( z1 )^2 ) }
        if ( qtyp == 17 ) { wl <- sinh( z1 ) / norm;  korr2 <- cosh( z1 ) / norm }
        if ( qtyp == 13 ) term <- korr * sum( w1 * dnorm( (sqrt(z0[i])*wl - mij)/l ) / l * dchisq( z0[i]*( 1 - wl^2 ) / l^2, p-1, ncpij ) * g[ (i-1)*r + 1:r ] * korr2 )
        if ( qtyp > 13 )  term <- korr * sum( w1 * dnorm( (z0[i]*wl - mij)/l ) / l * dchisq( z0[i]^2*( 1 - wl^2 ) / l^2, p-1, ncpij ) * g[ (i-1)*r + 1:r ] * korr2 )
        result <- result + term 
      }
      result
    })
  } 
  
  arl
}

# Computation of EWMA critical values for given QRL 
# (simultaneous mean and variance monitoring)
xsewma.q.crit <- function(lx, ls, L0, alpha, df, mu0=0, sigma0=1, csu=NULL, hsx=0, hss=1, sided="upper", mode="fixed", Nx=20, Ns=40, qm=30, c.error=1e-12, a.error=1e-9)
{
  if (lx<=0 || lx>1)			stop("lx has to be between 0 and 1")
  if (ls<=0 || ls>1)			stop("ls has to be between 0 and 1")
  if (L0<1)				stop("L0 is too small")
  if ( alpha<=0 | alpha>=1 )		stop("quantile level alpha must be in (0,1)")
  if ( df<1 )				stop("df must be positive")
  if ( sigma0<=0 )			stop("sigma0 must be positive")  
  if ( mode=="fixed" & sided=="two" ) {
    if ( is.null(csu) )			stop("set csu")
    if ( csu<sigma0 )			stop("csu is too small")
    if ( csu<=0 )			stop("csu must be positive")
    if ( hss>csu )			stop("hs must be smaller than csu")
    cu0 <- csu
  } else {
    cu0 <- 0
  }
  ctyp <- pmatch(sided, c("upper","two")) - 1
  if ( is.na(ctyp) )			stop("invalid ewma type")
  ltyp <- pmatch(mode, c("fixed","unbiased")) - 1
  if ( is.na(ltyp) )			stop("invalid limits type")
  if ( Nx<5 )				stop("Nx is too small")
  if ( Ns<10 )				stop("Ns is too small")
  if ( qm<10 )				stop("qm is too small")
  c <- .C("xsewma_q_crit", as.integer(ctyp), as.integer(ltyp),
          as.double(lx), as.double(ls),
          as.double(L0), as.double(alpha),
          as.double(cu0), as.double(hsx), as.double(hss),
          as.double(mu0), as.double(sigma0), as.integer(df), as.integer(Nx), as.integer(Ns), as.integer(qm),
          as.double(c.error), as.double(a.error),
          ans=double(length=3),PACKAGE="spc")$ans
  names(c) <- c("cx", "csl","csu")
  return (c)
}

\name{xgrsr.arl}
\alias{xgrsr.arl}
\title{Compute (zero-state) ARLs of Shiryaev-Roberts schemes}
\description{Computation of the (zero-state) Average Run Length (ARL)
for Shiryaev-Roberts schemes monitoring normal mean.}
\usage{xgrsr.arl(k, g, mu, zr = 0, hs=NULL, sided = "one", q = 1, MPT = FALSE, r = 30)}
\arguments{
\item{k}{reference value of the Shiryaev-Roberts scheme.}
\item{g}{control limit (alarm threshold) of Shiryaev-Roberts scheme.}
\item{mu}{true mean.}
\item{zr}{reflection border to enable the numerical algorithms used here.}
\item{hs}{so-called headstart (enables fast initial response). If \code{hs=NULL}, then
the classical headstart -Inf is used (corresponds to 0 for the non-log scheme).}
\item{sided}{distinguishes between one- and two-sided schemes by choosing 
\code{"one"} and\code{"two"}, respectively. Currently only one-sided schemes are
implemented.}
\item{q}{change point position. For \eqn{q=1} and
\eqn{\mu=\mu_0} and \eqn{\mu=\mu_1}, the usual
zero-state ARLs for the in-control and out-of-control case, respectively,
are calculated. For \eqn{q>1} and \eqn{\mu!=0} conditional delays, that is,
\eqn{E_q(L-q+1|L\ge q)}, will be determined.
Note that mu0=0 is implicitely fixed.}
\item{MPT}{switch between the old implementation (\code{FALSE}) and the new one (\code{TRUE}) that considers the complete
likelihood ratio. MPT stands for the initials of G. Moustakides, A. Polunchenko and A. Tartakovsky.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1}.}
}
\details{
\code{xgrsr.arl} determines the Average Run Length (ARL) by numerically
solving the related ARL integral equation by means of the Nystroem method
based on Gauss-Legendre quadrature.
}
\value{Returns a vector of length \code{q} which resembles the ARL and the sequence of conditional expected delays for
\code{q}=1 and \code{q}>1, respectively.}
\references{
S. Knoth (2006),
The art of evaluating monitoring schemes --
how to measure the performance of control charts?
S. Lenz, H. & Wilrich, P. (ed.),
\emph{Frontiers in Statistical Quality Control 8}, Physica Verlag, Heidelberg, Germany, 74-99.

G. Moustakides, A. Polunchenko, A. Tartakovsky (2009),
Numerical comparison of CUSUM and Shiryaev-Roberts procedures for detecting changes in distributions,
\emph{Communications in Statistics: Theory and Methods 38}, 3225-3239.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} and \code{xcusum-arl} for zero-state ARL computation of EWMA and CUSUM control charts,
respectively, and \code{xgrsr.ad} for the steady-state ARL.
}
\examples{
## Small study to identify appropriate reflection border to mimic unreflected schemes
k <- .5
g <- log(390)
zrs <- -(0:10)
ZRxgrsr.arl <- Vectorize(xgrsr.arl, "zr")
arls <- ZRxgrsr.arl(k, g, 0, zr=zrs)
data.frame(zrs, arls)

## Table 2 from Knoth (2006)
## original values are
#  mu   arl
#  0    697
#  0.5  33
#  1    10.4
#  1.5  6.2
#  2    4.4
#  2.5  3.5
#  3    2.9
#
k <- .5
g <- log(390)
zr <- -5 # see first example
mus <- (0:6)/2
Mxgrsr.arl <- Vectorize(xgrsr.arl, "mu")
arls <- round(Mxgrsr.arl(k, g, mus, zr=zr), digits=1)
data.frame(mus, arls)

XGRSR.arl  <- Vectorize("xgrsr.arl", "g")
zr <- -6

## Table 2 from Moustakides et al. (2009)
## original values are
# gamma   A     ARL/E_infty(L) SADD/E_1(L)
#   50   47.17      50.29        41.40
#  100   94.34     100.28        72.32
#  500  471.70     500.28       209.44
# 1000  943.41    1000.28       298.50
# 5000 4717.04    5000.24       557.87
#10000 9434.08   10000.17       684.17

theta <- .1
As2 <- c(47.17, 94.34, 471.7, 943.41, 4717.04, 9434.08)
gs2 <- log(As2)
arls0 <- round(XGRSR.arl(theta/2, gs2, 0, zr=-5, r=300, MPT=TRUE), digits=2)
arls1 <- round(XGRSR.arl(theta/2, gs2, theta, zr=-5, r=300, MPT=TRUE), digits=2)
data.frame(As2, arls0, arls1)

## Table 3 from Moustakides et al. (2009)
## original values are
# gamma   A     ARL/E_infty(L) SADD/E_1(L)
#   50   37.38      49.45        12.30
#  100   74.76      99.45        16.60
#  500  373.81     499.45        28.05
# 1000  747.62     999.45        33.33
# 5000 3738.08    4999.45        45.96
#10000 7476.15    9999.24        51.49

theta <- .5
As3 <- c(37.38, 74.76, 373.81, 747.62, 3738.08, 7476.15)
gs3 <- log(As3)
arls0 <- round(XGRSR.arl(theta/2, gs3, 0, zr=-5, r=70, MPT=TRUE), digits=2)
arls1 <- round(XGRSR.arl(theta/2, gs3, theta, zr=-5, r=70, MPT=TRUE), digits=2)
data.frame(As3, arls0, arls1)

## Table 4 from Moustakides et al. (2009)
## original values are
# gamma   A     ARL/E_infty(L) SADD/E_1(L)
#   50   28.02      49.78         4.98
#  100   56.04      99.79         6.22
#  500  280.19     499.79         9.30
# 1000  560.37     999.79        10.66
# 5000 2801.85    5000.93        13.86
#10000 5603.70    9999.87        15.24

theta <- 1
As4 <- c(28.02, 56.04, 280.19, 560.37, 2801.85, 5603.7)
gs4 <- log(As4)
arls0 <- round(XGRSR.arl(theta/2, gs4, 0, zr=-6, r=40, MPT=TRUE), digits=2)
arls1 <- round(XGRSR.arl(theta/2, gs4, theta, zr=-6, r=40, MPT=TRUE), digits=2)
data.frame(As4, arls0, arls1)
}
\keyword{ts}

\name{imr.arl}
\alias{imr.arl}
\alias{imr.Ru_Mgiven}
\alias{imr.Rl_Mgiven}
\alias{imr.MandRu}
\alias{imr.MandRuRl}
\title{Compute ARLs and control limit factors for I-MR combos in case of normal data}
\description{Computation of the (zero-state) Average Run Length (ARL) at given mean \code{mu} and \code{sigma} etc.}
\usage{imr.arl(M, Ru, mu, sigma, vsided="upper", Rl=0, cmode="coll", N=30, qm=30)

imr.Ru_Mgiven(M, L0, N=30, qm=30)

imr.Rl_Mgiven(M, L0, N=30, qm=30)

imr.MandRu(L0, N=30, qm=30)

imr.MandRuRl(L0, N=30, qm=30)}
\arguments{
\item{M}{control limit multiple for mean chart.}
\item{Ru}{upper control limit multiple for moving range chart.}
\item{mu}{actual mean.}
\item{sigma}{actual standard deviation.}
\item{vsided}{switches between the more common "upper" and the less popular "two"(-sided) case of the MR chart.
Setting \code{vsided} to "two" and \code{Ru} sufficiently large (at least \code{2*M}), creates an I-MR chart
with a lower limit only for the MR part.}
\item{Rl}{lower control limit multiple for moving range chart (not needed in the upper case, i.e. if \code{vsided}="upper").}
\item{cmode}{selects the numerical algorithm. The default \code{"coll"} picks the piecewise collocation, which is
the most accurate method. Selecting \code{"Crowder"}, the algorithm from Crowder (1987b) is chosen (re-implemented in R).
Taking a label from \code{"gl"}, \code{"rectangular"}, \code{"trapezoid"}, \code{"simpson"} or \code{"simpson3_8"},
one decides for the quite common Nystroem procedure to numerically solve the considered integral equation.
It is astonishing that Crowder's modified Nystroem design with the trapezoidal quadrature works so well.
However, it is clearly dominated by the piecewise collocation algorithm.}
\item{N}{Controls the dimension of the linear equation system and consequently the accuracy of the result. See details.}
\item{qm}{Number of quadrature nodes (and weights) to determine the collocation definite integrals.}
\item{L0}{pre-defined in-control ARL, that is, determine \code{Ru}, \code{Rl}, or \code{M} and \code{Ru} or
all of them (essentially ending in a lower limit MR chart) so that the mean
number of observations until a false alarm is \code{L0}.}
}
\details{
Crowder (1987a) provided some math to determine the ARL of the so-called individual moving range (IMR) chart.
The given integral equation was approximated by a linear equation system applying trapezoidal quadratures.
Interestingly, Crowder did not recognize the specific behavior of the solution for \code{Ru} >= \code{M} (which is
the more common case), where the resulting function L() is constant in the central part of the
considered domain. In addition, by performing collocation on two (\code{Ru} > \code{M})
or three (\code{Ru} < \code{M}) subintervals piecewise, one obtains highly accurate
ARL numbers. Note that \code{imr.MandRu} yields \code{M} and \code{Ru} for the upper MR trace, whereas
\code{imr.MandRuRl} provides in addition the lower factor \code{Rl} for IMR consisting of two two-sided control charts.
Note that the underlying ARL unbiased condition suppresses the upper limit \code{Ru} in all considered cases so far.
This is not completely surprising, because the mean chart is already quite sensitive for increases in the variance.
The two functions \code{imr.Ru_Mgiven} and \code{imr.Rl_Mgiven} deliver the single upper and lower limit,
respectively, if a one-sided MR design is utilized and the control lmit factor \code{M} of
the mean chart is set already. Note that for \code{Ru} > \code{2*M}, the upper MR limit is
not operative anymore. If it was initially an upper MR chart, then it reduces to a single mean chart.
If it was originally a two-sided MR design, then it becomes a two-sided mean/lower variance chart combo.
Within the latter scheme, the mean chart signals variance increases (a well-known pattern), whereas
the MR subchart delivers only decreasing variance signals. However, these simple Shewhart charts
exhibit in all configurations week variance decreases detection behavior.
Eventually, we should note that the scientific control chart community mainly recommends to
ignore MR charts, see, for example, Vardeman and Jobe (2016), whereas standards (such as ISO), commercial
SPC software and many training manuals provide the IMR scheme with completely wrong upper limits for the MR chart.}
\value{Returns either the ARL or control limit factors (alias multiples).}
\references{
S. V. Crowder (1987a)
Computation of ARL for Combined Individual Measurement and Moving Range Charts,
\emph{Journal of Quality Technology} 19(2), 98-102.
  
S. V. Crowder (1987b)
A Program for the Computation of ARL for Combined Individual Measurement and Moving Range Charts,
\emph{Journal of Quality Technology} 19(2), 103-106.

K. C. B. Roes, R. J. M. M. Does, Y. Schurink,
Shewhart-Type Control Charts for Individual Observations,
\emph{Journal of Quality Technology} 25(3), 188-198.

S. E. Rigdon, E. N. Cruthis, C. W. Champ (1994)
Design Strategies for Individuals and Moving Range Control Charts,
\emph{Journal of Quality Technology} 26(4), 274-287.

D. Radson, L. C. Alwan (1995)
Detecting Variance Reductions Using the Moving Range,
\emph{Quality Engineering} 8(1), 165-178.

S. R. Adke, X. Hong (1997)
A Supplementary Test Based on the Control Chart for Individuals,
\emph{Journal of Quality Technology} 29(1), 16-20.

R. W. Amin, R. A. Ethridge (1998)
A Note on Individual and Moving Range Control Charts,
\emph{Journal of Quality Technology} 30(1), 70-74.

C. A. Acosta-Mejia, J. J. Pignatiello (2000)
Monitoring process dispersion without subgrouping,
\emph{Journal of Quality Technology} 32(2), 89-102.

N. B. Marks, T. C. Krehbiel (2011)
Design And Application Of Individuals And Moving Range Control Charts,
\emph{Journal of Applied Business Research ({JABR})} 25(5), 31-40.

D. Rahardja (2014)
Comparison of Individual and Moving Range Chart Combinations to Individual
Charts in Terms of ARL after Designing for a Common ``All OK'' ARL,
\emph{Journal of Modern Applied Statistical Methods} 13(2), 364-378.

S. B. Vardeman, J. M. Jobe (2016)
\emph{Statistical Methods for Quality Assurance},
Springer, 2nd edition.
}
\author{Sven Knoth}
\seealso{later.}
\examples{
## Crowder (1987b), Output Listing 1, trapezoidal quadrature (less accurate)

M <- 2
Ru <- 3
mu <- seq(0, 2, by=0.25)
LL <- LL2 <- rep(NA, length(mu))
for ( i in 1:length(mu) ) {
  LL[i] <- round( imr.arl(M, Ru, mu[i], 1), digits=4)
  LL2[i] <- round( imr.arl(M, Ru, mu[i], 1, cmode="Crowder", N=80), digits=4)
}
LL1987b <- c(18.2164, 16.3541, 12.4282, 8.7559, 6.1071, 4.3582, 3.2260, 2.4878, 1.9989)
print( data.frame(mu, LL2, LL1987b, LL) )

## Crowder (1987a), Table 1, trapezoidal quadrature (less accurate)

M <- 4
Ru <- 3
mu <- seq(0, 2, by=0.25)
LL <- rep(NA, length(mu))
for ( i in 1:length(mu) ) LL[i] <- round( imr.arl(M, Ru, mu[i], 1), digits=4)
LL1987a <- c(34.44, 34.28, 34.07, 33.81, 33.45, 32.82, 31.50, 28.85, 24.49)
print( data.frame(mu, LL1987a, LL) )

## Rigdon, Cruthis, Champ (1994), Table 1, Monte Carlo based

M <- 2.992
Ru <- 4.139
icARL <- imr.arl(M, Ru, 0, 1)
icARL1994 <- 200
print( data.frame(icARL1994, icARL) )

M <- 3.268
Ru <- 4.556
icARL <- imr.arl(M, Ru, 0, 1)
icARL1994 <- 500
print( data.frame(icARL1994, icARL) )

## ..., Table 2, Monte Carlo based

M <- 2.992
Ru <- 4.139
tau <- c(seq(1, 1.3, by=0.05), seq(1.4, 2, by=0.1))
LL <- rep(NA, length(tau))
for ( i in 1:length(tau) ) LL[i] <- round( imr.arl(M, Ru, 0, tau[i]), digits=2)
LL1994 <- c(200.54, 132.25, 90.84, 65.66, 49.35, 38.92, 31.11, 21.35, 15.47,
12.04, 9.81, 8.21, 7.03, 6.14)
print( data.frame(tau, LL1994, LL) )

## Radson, Alwan (1995), Table 2 (Monte Carlo based), half-normal, known parameter case
## two-sided (!) MR-alone (!) chart, hence the ARL results has to be decreased by 1
## Here: a large M (=12) is deployed to mimic Inf
alpha <- 0.00915
Ru <- sqrt(2) * qnorm(1-alpha/4)
Rl <- sqrt(2) * qnorm(0.5+alpha/4)
k <- 1.5 - (0:7)/10
LL <- rep(NA, length(k))
for ( i in 1:length(k) )
  LL[i] <- round( imr.arl(12, Ru, 0, k[i], vsided="two", Rl=Rl), digits=2) - 1
RA1995 <- c(18.61, 24.51, 34.21, 49.74, 75.08, 113.14, 150.15, 164.54)
print( data.frame(k, RA1995, LL) )

## Amin, Ethridge (1998), Table 2, column sigma/sigma_0 = 1.00

M <- 3.27
Ru <- 4.56
#M <- 3.268
#Ru <- 4.556
mu <- seq(0, 2, by=0.25)
LL <- rep(NA, length(mu))
for ( i in 1:length(mu) ) LL[i] <- round( imr.arl(M, Ru, mu[i], 1), digits=1)
LL1998 <- c(505.3, 427.6, 276.7, 156.2, 85.0, 46.9, 26.9, 16.1, 10.1)
print( data.frame(mu, LL1998, LL) )

## ..., column sigma/sigma_0 = 1.05

for ( i in 1:length(mu) ) LL[i] <- round( imr.arl(M, Ru, mu[i], 1.05), digits=1)
LL1998 <- c(296.8, 251.6, 169.6, 101.6, 58.9, 34.5, 20.9, 13.2, 8.7)
print( data.frame(mu, LL1998, LL) )

## Acosta-Mejia, Pignatiello (2000), Table 2
## AMP utilized Markov chain approximation
## However, the MR series is not Markovian!
## MR-alone (!) chart, hence the ARL results has to be decreased by 1
## Here: a large M (=8) is deployed to mimic Inf
Ru <- 3.93
sigma <- c(1, 1.05, 1.1, 1.15, 1.2, 1.3, 1.4, 1.5, 1.75)
LL <- rep(NA, length(sigma))
for ( i in 1:length(sigma) ) LL[i] <- round( imr.arl(8, Ru, 0, sigma[i], N=30), digits=1) - 1
AMP2000 <- c(201.0, 136.8, 97.9, 73.0, 56.3, 36.4, 25.6, 19.1, 11.0)
print( data.frame(sigma, AMP2000, LL) )

## Mark, Krehbiel (2011), Table 2, deployment of Crowder (1987b), nominal ic ARL 500

M <- c(3.09, 3.20, 3.30, 3.50, 4.00)
Ru <- c(6.00, 4.67, 4.53, 4.42, 4.36)
LL0 <- rep(NA, length(M))
for ( i in 1:length(M) ) LL0[i] <- round( imr.arl(M[i], Ru[i], 0, 1), digits=1)
print( data.frame(M, Ru, LL0) )
}
\keyword{ts}

\name{scusums.arl}
\alias{scusums.arl}
\title{Compute ARLs of CUSUM-Shewhart control charts (variance charts)}
\description{Computation of the (zero-state) Average Run Length (ARL)
for different types of CUSUM-Shewhart combo control charts (based on the sample variance
\eqn{S^2}) monitoring normal variance.}
\usage{scusums.arl(k, h, cS, sigma, df, hs=0, sided="upper", k2=NULL,
h2=NULL, hs2=0, r=40, qm=30, version=2)}
\arguments{
\item{k}{reference value of the CUSUM control chart.}
\item{h}{decision interval (alarm limit, threshold) of the CUSUM control chart.}
\item{cS}{Shewhart limit.}
\item{sigma}{true standard deviation.}
\item{df}{actual degrees of freedom, corresponds to subgroup size (for known mean it is equal to the subgroup size,
for unknown mean it is equal to subgroup size minus one.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided two-sided CUSUM-\eqn{S^2}{S^2} control charts 
by choosing \code{"upper"} (upper chart), \code{"lower"} (lower chart), and \code{"two"} (two-sided chart), 
respectively. Note that for the two-sided chart the parameters \code{"k2"} and \code{"h2"} have to be set too.}
\item{k2}{In case of a two-sided CUSUM chart for variance the reference value of the lower chart.}
\item{h2}{In case of a two-sided CUSUM chart for variance the decision interval of the lower chart.}
\item{hs2}{In case of a two-sided CUSUM chart for variance the headstart of the lower chart.}
\item{r}{Dimension of the resulting linear equation system (highest order of the collocation
polynomials times number of intervals -- see Knoth 2006).}
\item{qm}{Number of quadrature nodes for calculating the collocation definite integrals.}
\item{version}{Distinguish version numbers (1,2,...). For internal use only.}
}
\details{
\code{scusums.arl} determines the Average Run Length (ARL) by numerically
solving the related ARL integral equation by means of collocation (piecewise Chebyshev polynomials).}
\value{Returns a single value which resembles the ARL.}
\references{
S. Knoth (2006),
Computation of the ARL for CUSUM-\eqn{S^2}{S^2} schemes,
\emph{Computational Statistics & Data Analysis 51}, 499-512.
}
\author{Sven Knoth}
\seealso{
\code{scusum.arl} for zero-state ARL computation of standalone CUSUM control charts for monitoring normal variance.
}
\examples{
## will follow
}
\keyword{ts}

\name{xewma.ad}
\alias{xewma.ad}
\title{Compute steady-state ARLs of EWMA control charts}
\description{Computation of the steady-state Average Run Length (ARL)
for different types of EWMA control charts monitoring normal mean.}
\usage{xewma.ad(l, c, mu1, mu0=0, zr=0, z0=0, sided="one", limits="fix",
steady.state.mode="conditional", r=40)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{c}{critical value (similar to alarm limit) of the EWMA control chart.}
\item{mu1}{out-of-control mean.}
\item{mu0}{in-control mean.}
\item{zr}{reflection border for the one-sided chart.}
\item{z0}{restarting value of the EWMA sequence in case of a false alarm in
\code{steady.state.mode="cyclical"}.}
\item{sided}{distinguishes between one- and two-sided two-sided EWMA control
chart by choosing \code{"one"} and \code{"two"}, respectively.}
\item{limits}{distinguishes between different control limits behavior.}
\item{steady.state.mode}{distinguishes between two steady-state modes -- conditional and cyclical.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-sided) or \code{r}
(two-sided).}
}
\details{
\code{xewma.ad} determines the steady-state Average Run Length (ARL)
by numerically solving the related ARL integral equation by means
of the Nystroem method based on Gauss-Legendre quadrature
and using the power method for deriving the largest in magnitude
eigenvalue and the related left eigenfunction.
}
\value{Returns a single value which resembles the steady-state ARL.}
\references{
R. B. Crosier (1986),
A new two-sided cumulative quality control scheme,
\emph{Technometrics 28}, 187-194.

S. V. Crowder (1987),
A simple method for studying run-length distributions of exponentially weighted
moving average charts,
\emph{Technometrics 29}, 401-407.

J. M. Lucas and M. S. Saccucci (1990),
Exponentially weighted moving average control schemes: Properties and enhancements,
\emph{Technometrics 32}, 1-12.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} for zero-state ARL computation and
\code{xcusum.ad} for the steady-state ARL of CUSUM control charts.
}
\examples{
## comparison of zero-state (= worst case ) and steady-state performance
## for two-sided EWMA control charts

l <- .1
c <- xewma.crit(l,500,sided="two")
mu <- c(0,.5,1,1.5,2)
arl <- sapply(mu,l=l,c=c,sided="two",xewma.arl)
ad <- sapply(mu,l=l,c=c,sided="two",xewma.ad)
round(cbind(mu,arl,ad),digits=2)

## Lucas/Saccucci (1990)
## two-sided EWMA

## with fixed limits
l1 <- .5
l2 <- .03
c1 <- 3.071
c2 <- 2.437
mu <- c(0,.25,.5,.75,1,1.5,2,2.5,3,3.5,4,5)
ad1 <- sapply(mu,l=l1,c=c1,sided="two",xewma.ad)
ad2 <- sapply(mu,l=l2,c=c2,sided="two",xewma.ad)
round(cbind(mu,ad1,ad2),digits=2)

## original results are (in Table 3)
## 0.00 499.   480.  
## 0.25 254.    74.1
## 0.50  88.4   28.6
## 0.75  35.7   17.3
## 1.00  17.3   12.5
## 1.50   6.44   8.00
## 2.00   3.58   5.95
## 2.50   2.47   4.78
## 3.00   1.91   4.02
## 3.50   1.58   3.49
## 4.00   1.36   3.09
## 5.00   1.10   2.55
}
\keyword{ts}

\name{xewma.sf.prerun}
\alias{xewma.sf.prerun}
\title{Compute the survival function of EWMA run length in case of estimated parameters}
\description{Computation of the survival function of the Run Length (RL) for EWMA
control charts monitoring normal mean
if the in-control mean, standard deviation, or both are estimated by a pre run.}
\usage{xewma.sf.prerun(l, c, mu, n, zr=0, hs=0, sided="one", limits="fix", q=1,
size=100, df=NULL, estimated="mu", qm.mu=30, qm.sigma=30,
truncate=1e-10, tail_approx=TRUE, bound=1e-10)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{c}{critical value (similar to alarm limit) of the EWMA control chart.}
\item{mu}{true mean.}
\item{n}{calculate sf up to value \code{n}.}
\item{zr}{reflection border for the one-sided chart.}
\item{hs}{so-called headstart (give fast initial response).}
\item{sided}{distinguish between one- and two-sided EWMA control chart 
by choosing \code{"one"} and \code{"two"}, respectively.}
\item{limits}{distinguish between different control limits behavior.}
\item{q}{change point position. For \eqn{q=1} and
\eqn{\mu=\mu_0} and \eqn{\mu=\mu_1}, the usual
zero-state situation for the in-control and out-of-control case, respectively,
are calculated. Note that mu0=0 is implicitely fixed.}
\item{size}{pre run sample size.}
\item{df}{degrees of freedom of the pre run variance estimator. Typically it is simply \code{size} - 1.
If the pre run is collected in batches, then also other
values are needed.}
\item{estimated}{name the parameter to be estimated within the \code{"mu"},
\code{"sigma"}, \code{"both"}.}
\item{qm.mu}{number of quadrature nodes for convoluting the mean uncertainty.}
\item{qm.sigma}{number of quadrature nodes for convoluting the standard deviation uncertainty.}
\item{truncate}{size of truncated tail.}
\item{tail_approx}{Controls whether the geometric tail approximation is used (is faster) or not.}
\item{bound}{control when the geometric tail kicks in; the larger the quicker and less accurate; \code{bound} should be larger than 0 and less than 0.001.}
}
\details{
The survival function P(L>n) and derived from it also the cdf P(L<=n) and
the pmf P(L=n) illustrate the distribution of the EWMA run length...
}
\value{Returns a vector which resembles the survival function up to a certain point.}
\references{
F. F. Gan (1993),
An optimal design of EWMA control charts based on the median run length,
\emph{J. Stat. Comput. Simulation 45}, 169-184.

S. Knoth (2003),
EWMA schemes with non-homogeneous transition kernels,
\emph{Sequential Analysis 22}, 241-255.

S. Knoth (2004),
Fast initial response features for EWMA Control Charts,
\emph{Statistical Papers 46}, 47-64.

L. A. Jones, C. W. Champ, S. E. Rigdon (2001),
The performance of exponentially weighted moving average charts
with estimated parameters,
\emph{Technometrics 43}, 156-167.

K.-H. Waldmann (1986),
Bounds for the distribution of the run length of geometric moving 
average charts, \emph{Appl. Statist. 35}, 151-158.
}
\author{Sven Knoth}
\seealso{
\code{xewma.sf} for the RL survival function of EWMA control charts
w/o pre run uncertainty.
}
\examples{
## Jones/Champ/Rigdon (2001)

c4m <- function(m, n) sqrt(2)*gamma( (m*(n-1)+1)/2 )/sqrt( m*(n-1) )/gamma( m*(n-1)/2 )

n <- 5 # sample size

# Figure 6, subfigure r=0.1
lambda <- 0.1
L <- 2.454

CDF0 <- 1 - xewma.sf(lambda, L, 0, 600, sided="two")
m <- 10 # pre run size
CDF1 <- 1 - xewma.sf.prerun(lambda, L/c4m(m,n), 0, 600, sided="two",
size=m, df=m*(n-1), estimated="both")
m <- 20
CDF2 <- 1 - xewma.sf.prerun(lambda, L/c4m(m,n), 0, 600, sided="two",
size=m, df=m*(n-1), estimated="both")
m <- 50
CDF3 <- 1 - xewma.sf.prerun(lambda, L/c4m(m,n), 0, 600, sided="two",
size=m, df=m*(n-1), estimated="both")

plot(CDF0, type="l", xlab="t", ylab=expression(P(T<=t)), xlim=c(0,500), ylim=c(0,1))
abline(v=0, h=c(0,1), col="grey", lwd=.7)
points((1:5)*100, CDF0[(1:5)*100], pch=18)
lines(CDF1, col="blue")
points((1:5)*100, CDF1[(1:5)*100], pch=2, col="blue")
lines(CDF2, col="red")
points((1:5)*100, CDF2[(1:5)*100], pch=16, col="red")
lines(CDF3, col="green")
points((1:5)*100, CDF3[(1:5)*100], pch=5, col="green")

legend("bottomright", c("Known", "m=10, n=5", "m=20, n=5", "m=50, n=5"),
       col=c("black", "blue", "red", "green"), pch=c(18, 2, 16, 5), lty=1)
}
\keyword{ts}

\name{pois.ewma.crit}
\alias{pois.ewma.crit}
\title{Compute ARLs of Poisson EWMA control charts}
\description{Computation of the (zero-state) Average Run Length (ARL) at given mean \code{mu}.}
\usage{pois.ewma.crit(lambda, L0, mu0, z0, AU=3, sided="two", design="sym", rando=FALSE,
mcdesign="transfer", N=101, jmax=4, OLD=FALSE)}
\arguments{
\item{lambda}{smoothing parameter of the EWMA p control chart.}
\item{L0}{value of the so-called in-control Average Run Length (ARL) for the Poisson EWMA control chart.}
\item{mu0}{in-control mean.}
\item{z0}{so-called headstart (give fast initial response).}
\item{AU}{in case of the lower chart deployed as reflecting upper barrier -- might be increased step by step until the resulting lower limit
does not change anymore.}
\item{sided}{distinguishes between one- and two-sided EWMA control chart by choosing \code{"upper"}, \code{"lower"}, and \code{"two"}, respectively.}
\item{design}{distinguishes between limits symmetric to the in-control mean \code{mu0} and an ARL-unbiased design (ARL maximum at \code{mu0}); use the shortcuts \code{"sym"} and \code{"unb"}, respectively, please.}
\item{rando}{Switch between the standard limit treatment, \code{FALSE},
and an additional randomisation (to allow `perfect' ARL calibration) by setting \code{TRUE}.
If randomisation is used, then the corresponding probailities, \code{gL} and \code{gU} are determined, appropriately.}
\item{mcdesign}{choose either \code{"classic"} which follows Borror, Champ and Rigdon (1998), or the more
sophisticated \code{"transfer"} which improves the accuracy heavily.}
\item{N}{number of states of the approximating Markov chain; is equal to the dimension of the resulting linear equation system.}
\item{jmax}{number of digits for the to be calculated factors \code{A} (sort of accuracy).}
\item{OLD}{Switch between the original implemenation, used in Morais and Knoth (2020), and a slightly modified new one (more accurate).}
}
\details{
The monitored data follow a Poisson distribution with \code{mu}.
Here we solve the inverse task to the usual ARL calculation. Hence, determine the control limit factors
so that the in-control ARL is (roughly) equal to \code{L0}.
The ARL values underneath the routine are determined by Markov chain approximation.
The algorithm is just a grid search that takes care of the discrete ARL behavior.
}
\value{Return one or two values being he control limit factors.}
\references{
C. M. Borror, C. W. Champ and S. E. Rigdon (1998)
Poisson EWMA control charts,
\emph{Journal of Quality Technonlogy} 30(4), 352-361.

M. C. Morais and S. Knoth (2020)
Improving the ARL profile and the accuracy of its calculation for Poisson EWMA charts,
\emph{Quality and Reliability Engineering International} 36(3), 876-889.
}
\author{Sven Knoth}
\seealso{later.}
\examples{
## Borror, Champ and Rigdon (1998), page 30, original value is A = 2.8275
mu0 <- 4
lambda <- 0.2
L0 <- 351
A <- pois.ewma.crit(lambda, L0, mu0, mu0, mcdesign="classic")
print(round(A, digits=4))

## Morais and Knoth (2020), Table 2, lambda = 0.27 column
lambda <- 0.27
L0 <- 1233.4
ccgg <- pois.ewma.crit(lambda,1233.4,mu0,mu0,design="unb",rando=TRUE,mcdesign="transfer",OLD=TRUE)
print(ccgg, digits=3)
}
\keyword{ts}

\name{xewma.q}
\alias{xewma.q}
\alias{xewma.q.crit}
\title{Compute RL quantiles of EWMA control charts}
\description{Computation of quantiles of the Run Length (RL)
for EWMA control charts monitoring normal mean.}
\usage{xewma.q(l, c, mu, alpha, zr=0, hs=0, sided="two", limits="fix", q=1, r=40)

xewma.q.crit(l, L0, mu, alpha, zr=0, hs=0, sided="two", limits="fix", r=40,
c.error=1e-12, a.error=1e-9, OUTPUT=FALSE)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{c}{critical value (similar to alarm limit) of the EWMA control chart.}
\item{mu}{true mean.}
\item{alpha}{quantile level.}
\item{zr}{reflection border for the one-sided chart.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided EWMA control chart 
by choosing \code{"one"} and \code{"two"}, respectively.}
\item{limits}{distinguishes between different control limits behavior.}
\item{q}{change point position. For \eqn{q=1} and
\eqn{\mu=\mu_0} and \eqn{\mu=\mu_1}, the usual
zero-state ARLs for the in-control and out-of-control case, respectively,
are calculated. For \eqn{q>1} and \eqn{\mu!=0} conditional delays, that is,
\eqn{E_q(L-q+1|L\geq)}, will be determined.
Note that mu0=0 is implicitely fixed.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-sided) or \code{r}
(two-sided).}
\item{L0}{in-control quantile value.}
\item{c.error}{error bound for two succeeding values of the critical value
during applying the secant rule.}
\item{a.error}{error bound for the quantile level \code{alpha} during applying
the secant rule.}
\item{OUTPUT}{activate or deactivate additional output.}
}
\details{
Instead of the popular ARL (Average Run Length) quantiles of the EWMA
stopping time (Run Length) are determined. The algorithm is based on
Waldmann's survival function iteration procedure.
If \code{limits} is not \code{"fix"}, then the method presented
in Knoth (2003) is utilized.
Note that for one-sided EWMA charts (\code{sided}=\code{"one"}), only
\code{"vacl"} and \code{"stat"} are deployed, while for two-sided ones
(\code{sided}=\code{"two"}) also \code{"fir"}, \code{"both"}
(combination of \code{"fir"} and \code{"vacl"}), and \code{"Steiner"} are
implemented. For details see Knoth (2004).
}
\value{Returns a single value which resembles the RL quantile of order \code{q}.}
\references{
F. F. Gan (1993),
An optimal design of EWMA control charts based on the median run length,
\emph{J. Stat. Comput. Simulation 45}, 169-184.

S. Knoth (2003),
EWMA schemes with non-homogeneous transition kernels,
\emph{Sequential Analysis 22}, 241-255.

S. Knoth (2004),
Fast initial response features for EWMA Control Charts,
\emph{Statistical Papers 46}, 47-64.

S. Knoth (2015),
Run length quantiles of EWMA control charts monitoring normal mean or/and variance,
\emph{International Journal of Production Research 53}, 4629-4647.

K.-H. Waldmann (1986),
Bounds for the distribution of the run length of geometric moving 
average charts, \emph{Appl. Statist. 35}, 151-158.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} for zero-state ARL computation of EWMA control charts.
}
\examples{
## Gan (1993), two-sided EWMA with fixed control limits
## some values of his Table 1 -- any median RL should be 500
XEWMA.Q <- Vectorize("xewma.q", c("l", "c"))
G.lambda <- c(.05, .1, .15, .2, .25)
G.h <- c(.441, .675, .863, 1.027, 1.177)
MEDIAN <- ceiling(XEWMA.Q(G.lambda, G.h/sqrt(G.lambda/(2-G.lambda)),
0, .5, sided="two"))
print(cbind(G.lambda, MEDIAN))

## increase accuracy of thresholds

# (i) calculate threshold for given in-control median value by
#     deplyoing secant rule
XEWMA.q.crit <- Vectorize("xewma.q.crit", "l")

# (ii) re-calculate the thresholds and remove the standardization step
L0 <- 500
G.h.new <- XEWMA.q.crit(G.lambda, L0, 0, .5, sided="two")
G.h.new <- round(G.h.new * sqrt(G.lambda/(2-G.lambda)), digits=5)

# (iii) compare Gan's original values and the new ones with 5 digits
print(cbind(G.lambda, G.h.new, G.h))

# (iv) calculate the new medians
MEDIAN <- ceiling(XEWMA.Q(G.lambda, G.h.new/sqrt(G.lambda/(2-G.lambda)),
0, .5, sided="two"))
print(cbind(G.lambda, MEDIAN))
}
\keyword{ts}

\name{xewma.q.prerun}
\alias{xewma.q.prerun}
\alias{xewma.q.crit.prerun}
\title{Compute RL quantiles of EWMA control charts in case of estimated parameters}
\description{Computation of quantiles of the Run Length (RL)
for EWMA control charts monitoring normal mean
if the in-control mean, standard deviation, or both are estimated by a pre run.}
\usage{xewma.q.prerun(l, c, mu, p, zr=0, hs=0, sided="two", limits="fix", q=1, size=100,
df=NULL, estimated="mu", qm.mu=30, qm.sigma=30, truncate=1e-10, bound=1e-10)

xewma.q.crit.prerun(l, L0, mu, p, zr=0, hs=0, sided="two", limits="fix", size=100,
df=NULL, estimated="mu", qm.mu=30, qm.sigma=30, truncate=1e-10, bound=1e-10,
c.error=1e-10, p.error=1e-9, OUTPUT=FALSE)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{c}{critical value (similar to alarm limit) of the EWMA control chart.}
\item{mu}{true mean shift.}
\item{p}{quantile level.}
\item{zr}{reflection border for the one-sided chart.}
\item{hs}{so-called headstart (give fast initial response).}
\item{sided}{distinguish between one- and two-sided EWMA control chart 
by choosing \code{"one"} and \code{"two"}, respectively.}
\item{limits}{distinguish between different control limits behavior.}
\item{q}{change point position. For \eqn{q=1} and
\eqn{\mu=\mu_0} and \eqn{\mu=\mu_1}, the usual
zero-state ARLs for the in-control and out-of-control case, respectively,
are calculated. For \eqn{q>1} and \eqn{\mu!=0} conditional delays, that is,
\eqn{E_q(L-q+1|L\geq)}, will be determined.
Note that mu0=0 is implicitely fixed.}
\item{size}{pre run sample size.}
\item{df}{Degrees of freedom of the pre run variance estimator. Typically it is simply \code{size} - 1.
If the pre run is collected in batches, then also other values are needed.}
\item{estimated}{name the parameter to be estimated within the \code{"mu"}, \code{"sigma"},
\code{"both"}.}
\item{qm.mu}{number of quadrature nodes for convoluting the mean uncertainty.}
\item{qm.sigma}{number of quadrature nodes for convoluting the standard deviation uncertainty.}
\item{truncate}{size of truncated tail.}
\item{bound}{control when the geometric tail kicks in; the larger the quicker and less accurate; \code{bound} should be larger than 0 and less than 0.001.}
\item{L0}{in-control quantile value.}
\item{c.error}{error bound for two succeeding values of the critical value during
applying the secant rule.}
\item{p.error}{error bound for the quantile level \code{p} during applying the secant rule.}
\item{OUTPUT}{activate or deactivate additional output.}
}
\details{
Essentially, the ARL function \code{xewma.q} is convoluted with the
distribution of the sample mean, standard deviation or both.
For details see Jones/Champ/Rigdon (2001) and Knoth (2014?).
}
\value{Returns a single value which resembles the RL quantile of order \code{q}.}
\references{
L. A. Jones, C. W. Champ, S. E. Rigdon (2001),
The performance of exponentially weighted moving average charts
with estimated parameters,
\emph{Technometrics 43}, 156-167.

S. Knoth (2003),
EWMA schemes with non-homogeneous transition kernels,
\emph{Sequential Analysis 22}, 241-255.

S. Knoth (2004),
Fast initial response features for EWMA Control Charts,
\emph{Statistical Papers 46}, 47-64.

S. Knoth (2014?),
tbd,
\emph{tbd}, tbd-tbd.

K.-H. Waldmann (1986),
Bounds for the distribution of the run length of geometric moving 
average charts, \emph{Appl. Statist. 35}, 151-158.
}
\author{Sven Knoth}
\seealso{
\code{xewma.q} for the usual RL quantiles computation of EWMA control charts.
}
\examples{
## Jones/Champ/Rigdon (2001)

c4m <- function(m, n) sqrt(2)*gamma( (m*(n-1)+1)/2 )/sqrt( m*(n-1) )/gamma( m*(n-1)/2 )

n <- 5 # sample size
m <- 20 # pre run with 20 samples of size n = 5
C4m <- c4m(m, n) # needed for bias correction

# Table 1, 3rd column
lambda <- 0.2
L <- 2.636

xewma.Q <- Vectorize("xewma.q", "mu")
xewma.Q.prerun <- Vectorize("xewma.q.prerun", "mu")

mu <- c(0, .25, .5, 1, 1.5, 2)
Q1  <- ceiling(xewma.Q(lambda, L, mu, 0.1, sided="two"))
Q2  <- ceiling(xewma.Q(lambda, L, mu, 0.5, sided="two"))
Q3  <- ceiling(xewma.Q(lambda, L, mu, 0.9, sided="two"))

cbind(mu, Q1, Q2, Q3)

\dontrun{
p.Q1 <- xewma.Q.prerun(lambda, L/C4m, mu, 0.1, sided="two", 
size=m, df=m*(n-1), estimated="both")
p.Q2 <- xewma.Q.prerun(lambda, L/C4m, mu, 0.5, sided="two",
size=m, df=m*(n-1), estimated="both")
p.Q3 <- xewma.Q.prerun(lambda, L/C4m, mu, 0.9, sided="two",
size=m, df=m*(n-1), estimated="both")

cbind(mu, p.Q1, p.Q2, p.Q3)
}

## original values are
#    mu Q1  Q2  Q3 p.Q1 p.Q2 p.Q3
#  0.00 25 140 456   13   73  345
#  0.25 12  56 174    9   46  253
#  0.50  7  20  56    6   20  101
#  1.00  4   7  15    3    7   18
#  1.50  3   4   7    2    4    8
#  2.00  2   3   5    2    3    5
}
\keyword{ts}

\name{xshewhartrunsrules.arl}
\alias{xshewhartrunsrules.arl}
\alias{xshewhartrunsrules.crit}
\alias{xshewhartrunsrules.ad}
\alias{xshewhartrunsrules.matrix}
\title{Compute ARLs of Shewhart control charts with and without runs rules}
\description{Computation of the (zero-state and steady-state) Average Run Length (ARL)
for Shewhart control charts with and without runs rules
monitoring normal mean.}
\usage{xshewhartrunsrules.arl(mu, c = 1, type = "12")

xshewhartrunsrules.crit(L0, mu = 0, type = "12")

xshewhartrunsrules.ad(mu1, mu0 = 0, c = 1, type = "12")

xshewhartrunsrules.matrix(mu, c = 1, type = "12")}
\arguments{
\item{mu}{true mean.}
\item{L0}{pre-defined in-control ARL, that is, determine \code{c} so that the mean
number of observations until a false alarm is \code{L0}.}
\item{mu1, mu0}{for the steady-state ARL two means are specified, mu0 is the in-control one
and usually equal to 0 , and mu1 must be given.}
\item{c}{normalizing constant to ensure specific alarming behavior.}
\item{type}{controls the type of Shewhart chart used, seed details section.}
}
\details{
\code{xshewhartrunsrules.arl} determines the zero-state Average Run Length (ARL)
based on the Markov chain approach given in Champ and Woodall (1987).
\code{xshewhartrunsrules.matrix} provides the corresponding
transition matrix that is also used in \code{xDshewhartrunsrules.arl} (ARL for control charting drift).
\code{xshewhartrunsrules.crit} allows to find the normalization constant \code{c} to
attain a fixed in-control ARL. Typically this is needed to calibrate the chart.
With \code{xshewhartrunsrules.ad} the steady-state ARL is calculated.
With the argument \code{type} certain runs rules could be set. The following list gives an overview.

\describe{
\item{"1"}{ The classical Shewhart chart with \code{+/- 3 c sigma} control limits (\code{c} is typically
equal to 1 and can be changed by the argument \code{c}).}
\item{"12"}{ The classic and the following runs rule: 2 of 3 are beyond \code{+/- 2 c sigma} on the same
side of the chart.}
\item{"13"}{ The classic and the following runs rule: 4 of 5 are beyond \code{+/- 1 c sigma} on the same
side of the chart.}
\item{"14"}{ The classic and the following runs rule: 8 of 8 are on the same side of the chart
(referring to the center line).}}
}
\value{Returns a single value which resembles the zero-state or steady-state ARL.
\code{xshewhartrunsrules.matrix} returns a matrix.}
\references{
C. W. Champ and W. H. Woodall (1987),
Exact results for Shewhart control charts with supplementary runs rules,
\emph{Technometrics 29}, 393-399.
}
\author{Sven Knoth}
\seealso{
\code{xDshewhartrunsrules.arl} for zero-state ARL of Shewhart control charts
with or without runs rules under drift.
}
\examples{
## Champ/Woodall (1987)
## Table 1
mus <- (0:15)/5
Mxshewhartrunsrules.arl <- Vectorize(xshewhartrunsrules.arl, "mu")
# standard (1 of 1 beyond 3 sigma) Shewhart chart without runs rules
C1 <- round(Mxshewhartrunsrules.arl(mus, type="1"), digits=2)
# standard + runs rule: 2 of 3 beyond 2 sigma on the same side
C12 <- round(Mxshewhartrunsrules.arl(mus, type="12"), digits=2)
# standard + runs rule: 4 of 5 beyond 1 sigma on the same side
C13 <- round(Mxshewhartrunsrules.arl(mus, type="13"), digits=2)
# standard + runs rule: 8 of 8 on the same side of the center line
C14 <- round(Mxshewhartrunsrules.arl(mus, type="14"), digits=2)

## original results are
#  mus     C1    C12    C13    C14                                    
#  0.0 370.40 225.44 166.05 152.73                                    
#  0.2 308.43 177.56 120.70 110.52                                    
#  0.4 200.08 104.46  63.88  59.76                                    
#  0.6 119.67  57.92  33.99  33.64                                    
#  0.8  71.55  33.12  19.78  21.07                                    
#  1.0  43.89  20.01  12.66  14.58                                    
#  1.2  27.82  12.81   8.84  10.90                                    
#  1.4  18.25   8.69   6.62   8.60                                    
#  1.6  12.38   6.21   5.24   7.03                                    
#  1.8   8.69   4.66   4.33   5.85                                    
#  2.0   6.30   3.65   3.68   4.89                                    
#  2.2   4.72   2.96   3.18   4.08                                    
#  2.4   3.65   2.48   2.78   3.38                                    
#  2.6   2.90   2.13   2.43   2.81                                    
#  2.8   2.38   1.87   2.14   2.35                                    
#  3.0   2.00   1.68   1.89   1.99

data.frame(mus, C1, C12, C13, C14)

## plus calibration, i. e. L0=250 (the maximal value for "14" is 255!
L0 <- 250
c1  <- xshewhartrunsrules.crit(L0, type = "1")
c12 <- xshewhartrunsrules.crit(L0, type = "12")
c13 <- xshewhartrunsrules.crit(L0, type = "13")
c14 <- xshewhartrunsrules.crit(L0, type = "14")
C1  <- round(Mxshewhartrunsrules.arl(mus, c=c1,  type="1"), digits=2)
C12 <- round(Mxshewhartrunsrules.arl(mus, c=c12, type="12"), digits=2)
C13 <- round(Mxshewhartrunsrules.arl(mus, c=c13, type="13"), digits=2)
C14 <- round(Mxshewhartrunsrules.arl(mus, c=c14, type="14"), digits=2)
data.frame(mus, C1, C12, C13, C14)

## and the steady-state ARL
Mxshewhartrunsrules.ad <- Vectorize(xshewhartrunsrules.ad, "mu1")
C1  <- round(Mxshewhartrunsrules.ad(mus, c=c1,  type="1"), digits=2)
C12 <- round(Mxshewhartrunsrules.ad(mus, c=c12, type="12"), digits=2)
C13 <- round(Mxshewhartrunsrules.ad(mus, c=c13, type="13"), digits=2)
C14 <- round(Mxshewhartrunsrules.ad(mus, c=c14, type="14"), digits=2)
data.frame(mus, C1, C12, C13, C14)
}
\keyword{ts}

\name{tol.lim.fac}
\alias{tol.lim.fac}
\title{Two-sided tolerance limit factors}
\description{For constructing tolerance intervals, which
cover a given proportion \eqn{p}{p} of a normal distribution with
unknown mean and variance with confidence 
\eqn{1-\alpha}{1-a}, one needs to calculate
the so-called tolerance limit factors \eqn{k}{k}. These values
are computed for a given sample size \eqn{n}{n}.}
\usage{tol.lim.fac(n,p,a,mode="WW",m=30)}
\arguments{
\item{n}{sample size.}
\item{p}{coverage.}
\item{a}{error probability \eqn{\alpha}{a}, resulting interval covers at least proportion \code{p}
with confidence of at least \eqn{1-\alpha}{1-a}.}
\item{mode}{distinguish between Wald/Wolfowitz' approximation method (\code{"WW"}) and the more
accurate approach (\code{"exact"})
based on Gauss-Legendre quadrature.}
\item{m}{number of abscissas for the quadrature (needed only for \code{method="exact"}),
of course, the larger the more accurate.}
}
\details{
\code{tol.lim.fac} determines tolerance limits factors 
\eqn{k}{k}
by means of the fast and simple approximation due to
Wald/Wolfowitz (1946) and of Gauss-Legendre quadrature like Odeh/Owen
(1980), respectively, who used in fact the Simpson Rule. Then, by
\eqn{\bar x \pm k \cdot s}{xbar +- k s} 
one can build the tolerance intervals
which cover at least proportion \eqn{p}{p} of a normal distribution for
given confidence level of 
\eqn{1-\alpha}{1-a}. \eqn{\bar x}{xbar} and \eqn{s}{s} stand
for the sample mean and the sample standard deviation, respectively.}
\value{Returns a single value which resembles the tolerance limit factor.}
\references{
A. Wald, J. Wolfowitz (1946), Tolerance limits for a normal distribution,
\emph{Annals of Mathematical Statistics 17}, 208-215.

R. E. Odeh, D. B. Owen (1980), \emph{Tables for Normal Tolerance Limits},
Sampling Plans, and Screening, Marcel Dekker, New York.
}
\author{Sven Knoth}
\seealso{
\code{qnorm} for the ''asymptotic'' case -- cf. second example.
}
\examples{
n <- 2:10
p <- .95
a <- .05
kWW <- sapply(n,p=p,a=a,tol.lim.fac)
kEX <- sapply(n,p=p,a=a,mode="exact",tol.lim.fac)
print(cbind(n,kWW,kEX),digits=4)
## Odeh/Owen (1980), page 98, in Table 3.4.1
##  n  factor k
##  2  36.519
##  3   9.789
##  4   6.341
##  5   5.077
##  6   4.422
##  7   4.020
##  8   3.746
##  9   3.546
## 10   3.393

## n -> infty
n <- 10^{1:7}
p <- .95
a <- .05
kEX <- round(sapply(n,p=p,a=a,mode="exact",tol.lim.fac),digits=4)
kEXinf <- round(qnorm(1-a/2),digits=4)
print(rbind(cbind(n,kEX),c("infinity",kEXinf)),quote=FALSE)
}
\keyword{ts}

\name{mewma.psi}
\alias{mewma.psi}
\title{Compute steady-state density of the MEWMA statistic}
\description{Computation of the (zero-state) steady-state density function of the statistic deployed in
multivariate exponentially weighted moving average (MEWMA) charts monitoring multivariate normal mean.}
\usage{mewma.psi(l, cE, p, type="cond", hs=0, r=20)}
\arguments{
\item{l}{smoothing parameter lambda of the MEWMA control chart.}
\item{cE}{alarm threshold of the MEWMA control chart.}
\item{p}{dimension of multivariate normal distribution.}
\item{type}{switch between \code{"cond"} and \code{"cycl"} for differentiating between the conditional
(no false alarm) and the cyclical (after false alarm re-start in \code{hs}), respectively.}
\item{hs}{the re-starting point for the cyclical steady-state framework.}
\item{r}{number of quadrature nodes.}
}
\details{Basically, ideas from Knoth (2017, MEWMA numerics) and Knoth (2016, steady-state ARL concepts) are merged.
More details will follow.}
\value{Returns a function.}
\references{
Sven Knoth (2016),
The Case Against the Use of Synthetic Control Charts,
\emph{Journal of Quality Technology 48(2)}, 178-195.

Sven Knoth (2017),
ARL Numerics for MEWMA Charts,
\emph{Journal of Quality Technology 49(1)}, 78-89.

Sven Knoth (2018),
The Steady-State Behavior of Multivariate Exponentially Weighted Moving Average Control Charts,
\emph{Sequential Analysis 37(4)}, 511-529.
}
\author{Sven Knoth}
\seealso{
\code{mewma.arl} for calculating the in-control ARL of MEWMA.
}
\examples{
lambda <- 0.1
L0 <- 200
p <- 3
h4 <- mewma.crit(lambda, L0, p)
x_ <- seq(0, h4*lambda/(2-lambda), by=0.002)
psi <- mewma.psi(lambda, h4, p)
psi_ <- psi(x_)
# plot(x_, psi_, type="l", xlab="x", ylab=expression(psi(x)), xlim=c(0,1.2))
# cf. to Figure 1 in Knoth (2018), p. 514, p=3
}
\keyword{ts}

\name{xDshewhartrunsrules.arl}
\alias{xDshewhartrunsrules.arl}
\alias{xDshewhartrunsrulesFixedm.arl}
\title{Compute ARLs of Shewhart control charts with and without runs rules
under drift}
\description{Computation of the zero-state Average Run Length (ARL)
under drift for Shewhart control charts with and without runs rules
monitoring normal mean.}
\usage{xDshewhartrunsrules.arl(delta, c = 1, m = NULL, type = "12")

xDshewhartrunsrulesFixedm.arl(delta, c = 1, m = 100, type = "12")
}
\arguments{
\item{delta}{true drift parameter.}
\item{c}{normalizing constant to ensure specific alarming behavior.}
\item{type}{controls the type of Shewhart chart used, seed details section.}
\item{m}{parameter of Gan's approach. If \code{m=NULL}, then \code{m} will increased until
the resulting ARL does not change anymore.}
}
\details{
Based on Gan (1991), the ARL is calculated for
Shewhart control charts with and without runs rules
under drift. The usual ARL function with mu=m*delta is determined and recursively via
m-1, m-2, ... 1 (or 0) the drift ARL determined.
\code{xDshewhartrunsrulesFixedm.arl} is the actual work horse, while
\code{xDshewhartrunsrules.arl} provides a convenience wrapper.
Note that Aerne et al. (1991) deployed a method that is
quite similar to Gan's algorithm. For \code{type} see
the help page of \code{xshewhartrunsrules.arl}.
}
\value{Returns a single value which resembles the ARL.}
\references{
F. F. Gan (1991),
EWMA control chart under linear drift,
\emph{J. Stat. Comput. Simulation 38}, 181-200.

L. A. Aerne, C. W. Champ and S. E. Rigdon (1991),
Evaluation of control charts under linear trend,
\emph{Commun. Stat., Theory Methods 20}, 3341-3349.
}
\author{Sven Knoth}
\seealso{
\code{xshewhartrunsrules.arl} for zero-state ARL computation of
Shewhart control charts with and without runs rules
for the classical step change model.
}
\examples{
## Aerne et al. (1991)
## Table I (continued)
## original numbers are
#     delta arl1of1 arl2of3 arl4of5  arl10
#  0.005623  136.67  120.90  105.34 107.08
#  0.007499  114.98  101.23   88.09  89.94
#  0.010000   96.03   84.22   73.31  75.23
#  0.013335   79.69   69.68   60.75  62.73
#  0.017783   65.75   57.38   50.18  52.18
#  0.023714   53.99   47.06   41.33  43.35
#  0.031623   44.15   38.47   33.99  36.00
#  0.042170   35.97   31.36   27.91  29.90
#  0.056234   29.21   25.51   22.91  24.86
#  0.074989   23.65   20.71   18.81  20.70
#  0.100000   19.11   16.79   15.45  17.29
#  0.133352   15.41   13.61   12.72  14.47
#  0.177828   12.41   11.03   10.50  12.14
#  0.237137    9.98    8.94    8.71  10.18
#  0.316228    8.02    7.25    7.26   8.45
#  0.421697    6.44    5.89    6.09   6.84
#  0.562341    5.17    4.80    5.15   5.48
#  0.749894    4.16    3.92    4.36   4.39
#  1.000000    3.35    3.22    3.63   3.52
c1of1 <- 3.069/3
c2of3 <- 2.1494/2
c4of5 <- 1.14
c10   <- 3.2425/3
DxDshewhartrunsrules.arl <- Vectorize(xDshewhartrunsrules.arl, "delta")
deltas <- 10^(-(18:0)/8)
arl1of1 <-
round(DxDshewhartrunsrules.arl(deltas, c=c1of1, type="1"), digits=2)
arl2of3 <-
round(DxDshewhartrunsrules.arl(deltas, c=c2of3, type="12"), digits=2)
arl4of5 <-
round(DxDshewhartrunsrules.arl(deltas, c=c4of5, type="13"), digits=2)
arl10 <- 
round(DxDshewhartrunsrules.arl(deltas, c=c10, type="SameSide10"), digits=2)
data.frame(delta=round(deltas, digits=6), arl1of1, arl2of3, arl4of5, arl10)
}
\keyword{ts}

\name{sewma.crit}
\alias{sewma.crit}
\title{Compute critical values of EWMA control charts (variance charts)}
\description{Computation of the critical values (similar to alarm limits)
for different types of EWMA control charts (based on the sample variance
\eqn{S^2}) monitoring normal variance.}
\usage{sewma.crit(l,L0,df,sigma0=1,cl=NULL,cu=NULL,hs=NULL,s2.on=TRUE,
sided="upper",mode="fixed",ur=4,r=40,qm=30)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{L0}{in-control ARL.}
\item{df}{actual degrees of freedom, corresponds to subgroup size (for known mean it is equal to the subgroup size,
for unknown mean it is equal to subgroup size minus one.}
\item{sigma0}{in-control standard deviation.}
\item{cl}{deployed for \code{sided}=\code{"Rupper"}, that is, upper variance control chart
with lower reflecting barrier \code{cl}.}
\item{cu}{for two-sided (\code{sided}=\code{"two"}) and fixed upper
control limit (\code{mode}=\code{"fixed"}) a value larger than \code{sigma0}
has to been given, for all other cases \code{cu} is ignored.}
\item{hs}{so-called headstart (enables fast initial response); the default (\code{NULL})
yields the expected in-control value of \eqn{S^2}{S^2} (1) and \eqn{S}{S} (\eqn{c_4}{c_4}),
respectively.}
\item{s2.on}{distinguishes between \eqn{S^2}{S^2} and \eqn{S}{S} chart.}
\item{sided}{distinguishes between one- and two-sided
two-sided EWMA-\eqn{S^2}{S^2} control charts
by choosing \code{"upper"} (upper chart
without reflection at \code{cl} -- the actual value of
\code{cl} is not used),
\code{"Rupper"} (upper chart with reflection at \code{cl}),
\code{"Rlower"} (lower chart with reflection at \code{cu}),
and \code{"two"} (two-sided chart),
respectively.}
\item{mode}{only deployed for \code{sided}=\code{"two"} -- with \code{"fixed"}
an upper control limit (see \code{cu}) is set and only the lower is
calculated to obtain the in-control ARL \code{L0}, while with \code{"unbiased"}
a certain unbiasedness of the ARL function is guaranteed (here, both the
lower and the upper control limit are calculated). With \code{"vanilla"} limits symmetric
around 1 (the in-control value of the variance)
are determined, while for \code{"eq.tails"} the in-control ARL values of two single EWMA
variance charts (decompose the two-sided
scheme into one lower and one upper scheme) are matched.}
\item{ur}{truncation of lower chart for \code{eq.tails} mode.}
\item{r}{dimension of the resulting linear equation system
(highest order of the collocation polynomials).}
\item{qm}{number of quadrature nodes for calculating the collocation definite integrals.}
}
\details{
\code{sewma.crit} determines the critical values (similar to alarm limits)
for given in-control ARL \code{L0}
by applying secant rule and using \code{sewma.arl()}.
In case of \code{sided}=\code{"two"} and \code{mode}=\code{"unbiased"}
a two-dimensional secant rule is applied that also ensures that the
maximum of the ARL function for given standard deviation is attained
at \code{sigma0}. See Knoth (2010) and the related example.
}
\value{Returns the lower and upper control limit \code{cl} and \code{cu}.}
\references{
H.-J. Mittag and D. Stemann and B. Tewes (1998),
EWMA-Karten zur \"Uberwachung der Streuung von Qualit\"atsmerkmalen,
\emph{Allgemeines Statistisches Archiv 82}, 327-338,

C. A. Acosta-Mej\'ia and J. J. Pignatiello Jr. and B. V. Rao (1999),
A comparison of control charting procedures for monitoring process dispersion,
\emph{IIE Transactions 31}, 569-579.

S. Knoth (2005),
Accurate ARL computation for EWMA-\eqn{S^2}{S^2} control charts,
\emph{Statistics and Computing 15}, 341-352.

S. Knoth (2006a),
Computation of the ARL for CUSUM-\eqn{S^2}{S^2} schemes,
\emph{Computational Statistics & Data Analysis 51}, 499-512.

S. Knoth (2006b), 
The art of evaluating monitoring schemes -- how to measure the performance of control charts?
in \emph{Frontiers in Statistical Quality Control 8},
H.-J. Lenz and P.-T. Wilrich (Eds.),
Physica Verlag, Heidelberg, Germany, 74-99.

S. Knoth (2010),
Control Charting Normal Variance -- Reflections, Curiosities, and Recommendations,
in \emph{Frontiers in Statistical Quality Control 9},
H.-J. Lenz and P.-T. Wilrich (Eds.),
Physica Verlag, Heidelberg, Germany, 3-18.
}
\author{Sven Knoth}
\seealso{\code{sewma.arl} for calculation of ARL of variance charts.}
\examples{
## Mittag et al. (1998)
## compare their upper critical value 2.91 that
## leads to the upper control limit via the formula shown below
## (for the usual upper EWMA \eqn{S^2}{S^2}).
## See Knoth (2006b) for a discussion of this EWMA setup and it's evaluation.

l  <- 0.18
L0 <- 250
df <- 4
limits <- sewma.crit(l, L0, df)
limits["cu"]

limits.cu.mittag_et_al <- 1 + sqrt(l/(2-l))*sqrt(2/df)*2.91
limits.cu.mittag_et_al

## Knoth (2005)
## reproduce the critical value given in Figure 2 (c=1.661865) for
## upper EWMA \eqn{S^2}{S^2} with df=1

l  <- 0.025
L0 <- 250
df <- 1
limits <- sewma.crit(l, L0, df)
cv.Fig2 <- (limits["cu"]-1)/( sqrt(l/(2-l))*sqrt(2/df) )
cv.Fig2

## the small difference (sixth digit after decimal point) stems from
## tighter criterion in the secant rule implemented in the R package.

## demo of unbiased ARL curves
## Deploy, please, not matrix dimensions smaller than 50 -- for the
## sake of accuracy, the value 80 was used.
## Additionally, this example needs between 1 and 2 minutes on a 1.6 Ghz box. 

\dontrun{
l  <- 0.1
L0 <- 500
df <- 4
limits <- sewma.crit(l, L0, df, sided="two", mode="unbiased", r=80)
SEWMA.arl <- Vectorize(sewma.arl, "sigma")
SEWMA.ARL <- function(sigma) 
  SEWMA.arl(l, limits[1], limits[2], sigma, df, sided="two", r=80)
layout(matrix(1:2, nrow=1))
curve(SEWMA.ARL, .75, 1.25, log="y")
curve(SEWMA.ARL, .95, 1.05, log="y")}
# the above stuff needs about 1 minute

## control limits for upper and lower EWMA charts with reflecting barriers
## (reflection at in-control level sigma0 = 1)
## examples from Knoth (2006a), Tables 4 and 5

\dontrun{
## upper chart with reflection at sigma0=1 in Table 4: c = 2.4831
l <- 0.15
L0 <- 100
df <- 4
limits <- sewma.crit(l, L0, df, cl=1, sided="Rupper", r=100)
cv.Tab4 <- (limits["cu"]-1)/( sqrt(l/(2-l))*sqrt(2/df) )
cv.Tab4

## lower chart with reflection at sigma0=1 in Table 5: c = 2.0613
l <- 0.115
L0 <- 200
df <- 5
limits <- sewma.crit(l, L0, df, cu=1, sided="Rlower", r=100)
cv.Tab5 <- -(limits["cl"]-1)/( sqrt(l/(2-l))*sqrt(2/df) )
cv.Tab5}
}
\keyword{ts}

\name{quadrature.nodes.weights}
\alias{quadrature.nodes.weights}
\title{Calculate quadrature nodes and weights}
\description{Computation of the nodes and weights to enable numerical quadrature.}
\usage{quadrature.nodes.weights(n, type="GL", x1=-1, x2=1)}
\arguments{
\item{n}{number of nodes (and weights).}
\item{type}{quadrature type -- currently Gauss-Legendre, \code{"GL"}, and Radau, \code{"Ra"}, are supported.}
\item{x1}{lower limit of the integration interval.}
\item{x2}{upper limit of the integration interval.}
}
\details{
A more detailed description will follow soon. The algorithm for the Gauss-Legendre quadrature was delivered by
Knut Petras to me, while the one for the Radau quadrature was taken from John Burkardt.
}
\value{Returns two vectors which hold the needed quadrature nodes and weights.}
\references{
H. Brass and K. Petras (2011),
\emph{Quadrature Theory. The Theory of Numerical Integration on a Compact Interval,}
Mathematical Surveys and Monographs, American Mathematical Society.

John Burkardt (2015),
\url{https://people.math.sc.edu/Burkardt/c_src/quadrule/quadrule.c}
}
\author{Sven Knoth}
\seealso{
Many of the ARL routines use the Gauss-Legendre nodes.
}
\examples{
# GL
n <- 10
qnw <-quadrature.nodes.weights(n, type="GL")
qnw

# Radau
n <- 10
qnw <-quadrature.nodes.weights(n, type="Ra")
qnw
}
\keyword{ts}

\name{mewma.arl}
\alias{mewma.arl}
\alias{mewma.arl.f}
\alias{mewma.ad}
\title{Compute ARLs of MEWMA control charts}
\description{Computation of the (zero-state) Average Run Length (ARL)
for multivariate exponentially weighted moving average (MEWMA) charts monitoring multivariate normal mean.}
\usage{mewma.arl(l, cE, p, delta=0, hs=0, r=20, ntype=NULL, qm0=20, qm1=qm0)

mewma.arl.f(l, cE, p, delta=0, r=20, ntype=NULL, qm0=20, qm1=qm0)

mewma.ad(l, cE, p, delta=0, r=20, n=20, type="cond", hs=0, ntype=NULL, qm0=20, qm1=qm0)}
\arguments{
\item{l}{smoothing parameter lambda of the MEWMA control chart.}
\item{cE}{alarm threshold of the MEWMA control chart.}
\item{p}{dimension of multivariate normal distribution.}
\item{delta}{magnitude of the potential change, \code{delta=0} refers to the in-control state.}
\item{hs}{so-called headstart (enables fast initial response) -- must be non-negative.}
\item{r}{number of quadrature nodes -- dimension of the resulting linear equation system
for \code{delta} = 0. For non-zero \code{delta} this dimension is mostly r^2 (Markov chain approximation leads
to some larger values). Caution: If \code{ntype} is set to \code{"co"} (collocation), then values of \code{r}
larger than 20 lead to large computing times.
For the other selections this would happen for values larger than 40.}
\item{ntype}{choose the numerical algorithm to solve the ARL integral equation. For \code{delta}=0:
Possible values are
\code{"gl"}, \code{"gl2"} (gauss-legendre, classic and with variables change: square),
\code{"co"} (collocation, for \code{delta} > 0 with sin transformation),
\code{"ra"} (radau),
\code{"cc"} (clenshaw-curtis),
\code{"mc"} (markov chain),
and \code{"sr"} (simpson rule).
For \code{delta} larger than 0, some more values besides the others are possible:
\code{"gl3"}, \code{"gl4"}, \code{"gl5"} (gauss-legendre with a further change in variables: sin, tan, sinh),
\code{"co2"}, \code{"co3"} (collocation with some trimming and tan as quadrature stabilizing transformations, respectively).
If it is set to \code{NULL} (the default), then for \code{delta}=0 then \code{"gl2"} is chosen.
If \code{delta} larger than 0, then for \code{p} equal 2 or 4 \code{"gl3"} and for all other values \code{"gl5"} is taken.
\code{"ra"} denotes the method used in Rigdon (1995a). \code{"mc"} denotes the Markov chain approximation.}
\item{type}{switch between \code{"cond"} and \code{"cycl"} for differentiating between the conditional
(no false alarm) and the cyclical (after false alarm re-start in \code{hs}), respectively.}
\item{n}{number of quadrature nodes for Calculating the steady-state ARL integral(s).}
\item{qm0,qm1}{number of collocation quadrature nodes for the out-of-control case (\code{qm0} for the inner integral,
\code{qm1} for the outer one), that is, for positive \code{delta},
and for the in-control case (now only \code{qm0} is deployed) if via \code{ntype} the collocation procedure is requested.}
}
\details{Basically, this is the implementation of different numerical algorithms for
solving the integral equation for the MEWMA in-control (\code{delta} = 0) ARL introduced in Rigdon (1995a)
and out-of-control (\code{delta} != 0) ARL in Rigdon (1995b).
Most of them are nothing else than the Nystroem approach -- the integral is replaced by a suitable quadrature.
Here, the Gauss-Legendre (more powerful), Radau (used by Rigdon, 1995a), Clenshaw-Curtis, and
Simpson rule (which is really bad) are provided.
Additionally, the collocation approach is offered as well, because it is much better for small odd values for \code{p}.
FORTRAN code for the Radau quadrature based Nystroem of Rigdon (1995a)
was published in Bodden and Rigdon (1999) -- see also \url{https://lib.stat.cmu.edu/jqt/31-1}.
Furthermore, FORTRAN code for the Markov chain approximation (in- and out-ot-control)
could be found at \url{https://lib.stat.cmu.edu/jqt/33-4/}.
The related papers are Runger and Prabhu (1996) and Molnau et al. (2001).
The idea of the Clenshaw-Curtis quadrature was taken from
Capizzi and Masarotto (2010), who successfully deployed a modified Clenshaw-Curtis quadrature
to calculate the ARL of combined (univariate) Shewhart-EWMA charts. It turns out that it works also nicely for the
MEWMA ARL. The version \code{mewma.arl.f()} without the argument \code{hs} provides the ARL as function of one (in-control)
or two (out-of-control) arguments.
}
\value{Returns a single value which is simply the zero-state ARL.}
\references{
Kevin M. Bodden and Steven E. Rigdon (1999),
A program for approximating the in-control ARL for the MEWMA chart,
\emph{Journal of Quality Technology 31(1)}, 120-123.

Giovanna Capizzi and Guido Masarotto (2010),
Evaluation of the run-length distribution for a combined Shewhart-EWMA control chart,
\emph{Statistics and Computing 20(1)}, 23-33.

Sven Knoth (2017),
ARL Numerics for MEWMA Charts,
\emph{Journal of Quality Technology 49(1)}, 78-89.

Wade E. Molnau et al. (2001),
A Program for ARL Calculation for Multivariate EWMA Charts,
\emph{Journal of Quality Technology 33(4)}, 515-521.

Sharad S. Prabhu and George C. Runger (1997),
Designing a multivariate EWMA control chart,
\emph{Journal of Quality Technology 29(1)}, 8-15.

Steven E. Rigdon (1995a), An integral equation for the in-control average run length of a multivariate
exponentially weighted moving average control chart, \emph{J. Stat. Comput. Simulation 52(4)}, 351-365.

Steven E. Rigdon (1995b), A double-integral equation for the average run length of a multivariate
exponentially weighted moving average control chart, \emph{Stat. Probab. Lett. 24(4)}, 365-373.

George C. Runger and Sharad S. Prabhu (1996),
A Markov Chain Model for the Multivariate Exponentially Weighted Moving Averages Control Chart,
\emph{J. Amer. Statist. Assoc. 91(436)}, 1701-1706.
}
\author{Sven Knoth}
\seealso{
\code{mewma.crit} for getting the alarm threshold to attain a certain in-control ARL.
}
\examples{
# Rigdon (1995a), p. 357, Tab. 1
p <- 2
r <- 0.25
h4 <- c(8.37, 9.90, 11.89, 13.36, 14.82, 16.72)
for ( i in 1:length(h4) ) cat(paste(h4[i], "\t", round(mewma.arl(r, h4[i], p, ntype="ra")), "\n"))

r <- 0.1
h4 <- c(6.98, 8.63, 10.77, 12.37, 13.88, 15.88)
for ( i in 1:length(h4) ) cat(paste(h4[i], "\t", round(mewma.arl(r, h4[i], p, ntype="ra")), "\n"))


# Rigdon (1995b), p. 372, Tab. 1
\dontrun{
r <- 0.1
p <- 4
h <- 12.73
for ( sdelta in c(0, 0.125, 0.25, .5, 1, 2, 3) )
  cat(paste(sdelta, "\t",
      round(mewma.arl(r, h, p, delta=sdelta^2, ntype="ra", r=25), digits=2), "\n"))

p <- 5
h <- 14.56
for ( sdelta in c(0, 0.125, 0.25, .5, 1, 2, 3) )
  cat(paste(sdelta, "\t",
      round(mewma.arl(r, h, p, delta=sdelta^2, ntype="ra", r=25), digits=2), "\n"))

p <- 10
h <- 22.67
for ( sdelta in c(0, 0.125, 0.25, .5, 1, 2, 3) )
  cat(paste(sdelta, "\t",
      round(mewma.arl(r, h, p, delta=sdelta^2, ntype="ra", r=25), digits=2), "\n"))
}

# Runger/Prabhu (1996), p. 1704, Tab. 1
\dontrun{
r <- 0.1
p <- 4
H <- 12.73
cat(paste(0, "\t", round(mewma.arl(r, H, p, delta=0, ntype="mc", r=50), digits=2), "\n"))
for ( delta in c(.5, 1, 1.5, 2, 3) )
  cat(paste(delta, "\t",
      round(mewma.arl(r, H, p, delta=delta, ntype="mc", r=25), digits=2), "\n"))
# compare with Fortran program (MEWMA-ARLs.f90) from Molnau et al. (2001) with m1 = m2 = 25
# H4      P     R   DEL  ARL
# 12.73  4.  0.10  0.00 199.78
# 12.73  4.  0.10  0.50  35.05
# 12.73  4.  0.10  1.00  12.17
# 12.73  4.  0.10  1.50   7.22
# 12.73  4.  0.10  2.00   5.19
# 12.73  4.  0.10  3.00   3.42

p <- 20
H <- 37.01
cat(paste(0, "\t",
    round(mewma.arl(r, H, p, delta=0, ntype="mc", r=50), digits=2), "\n"))
for ( delta in c(.5, 1, 1.5, 2, 3) )
  cat(paste(delta, "\t",
      round(mewma.arl(r, H, p, delta=delta, ntype="mc", r=25), digits=2), "\n"))
# compare with Fortran program (MEWMA-ARLs.f90) from Molnau et al. (2001) with m1 = m2 = 25
# H4      P     R   DEL  ARL
# 37.01 20.  0.10  0.00 199.09
# 37.01 20.  0.10  0.50  61.62
# 37.01 20.  0.10  1.00  20.17
# 37.01 20.  0.10  1.50  11.40
# 37.01 20.  0.10  2.00   8.03
# 37.01 20.  0.10  3.00   5.18
}

# Knoth (2017), p. 85, Tab. 3, rows with p=3
\dontrun{
p <- 3
lambda <- 0.05
h4 <- mewma.crit(lambda, 200, p)
benchmark <- mewma.arl(lambda, h4, p, delta=1, r=50)
  
mc.arl  <- mewma.arl(lambda, h4, p, delta=1, r=25, ntype="mc")
ra.arl  <- mewma.arl(lambda, h4, p, delta=1, r=27, ntype="ra")
co.arl  <- mewma.arl(lambda, h4, p, delta=1, r=12, ntype="co2")
gl3.arl <- mewma.arl(lambda, h4, p, delta=1, r=30, ntype="gl3")
gl5.arl <- mewma.arl(lambda, h4, p, delta=1, r=25, ntype="gl5")
  
abs( benchmark - data.frame(mc.arl, ra.arl, co.arl, gl3.arl, gl5.arl) )
}

# Prabhu/Runger (1997), p. 13, Tab. 3
\dontrun{
p <- 2
r <- 0.1
H <- 8.64
cat(paste(0, "\t",
    round(mewma.ad(r, H, p, delta=0, type="cycl", ntype="mc", r=60), digits=2), "\n"))
for ( delta in c(.5, 1, 1.5, 2, 3) )
  cat(paste(delta, "\t",
      round(mewma.ad(r, H, p, delta=delta, type="cycl", ntype="mc", r=30), digits=2), "\n"))

# better accuracy
for ( delta in c(0, .5, 1, 1.5, 2, 3) )
  cat(paste(delta, "\t",
      round(mewma.ad(r, H, p, delta=delta^2, type="cycl", r=30), digits=2), "\n"))
}
}
\keyword{ts}

\name{xsewma.q}
\alias{xsewma.q}
\alias{xsewma.q.crit}
\title{Compute critical values of simultaneous EWMA control charts
(mean and variance charts) for given RL quantile}
\description{Computation of the critical values (similar to alarm limits)
for different types of simultaneous EWMA control charts 
(based on the sample mean and the sample variance \eqn{S^2})
monitoring normal mean and variance.}
\usage{xsewma.q(lx, cx, ls, csu, df, alpha, mu, sigma, hsx=0,
Nx=40, csl=0, hss=1, Ns=40, sided="upper", qm=30)

xsewma.q.crit(lx, ls, L0, alpha, df, mu0=0, sigma0=1, csu=NULL,
hsx=0, hss=1, sided="upper", mode="fixed", Nx=20, Ns=40, qm=30,
c.error=1e-12, a.error=1e-9)}
\arguments{
\item{lx}{smoothing parameter lambda of the two-sided mean EWMA chart.}
\item{cx}{control limit of the two-sided mean EWMA control chart.}
\item{ls}{smoothing parameter lambda of the variance EWMA chart.}
\item{csu}{for two-sided (\code{sided}=\code{"two"}) and fixed upper
control limit (\code{mode}=\code{"fixed"}, only for \code{xsewma.q.crit})
a value larger than \code{sigma0}
has to been given, for all other cases \code{cu} is ignored.
It is the upper control limit of the variance EWMA control chart.}
\item{L0}{in-control RL quantile at level \code{alpha}.}
\item{df}{actual degrees of freedom, corresponds to subgroup size
(for known mean it is equal to the subgroup size,
for unknown mean it is equal to subgroup size minus one.}
\item{alpha}{quantile level.}
\item{mu}{true mean.}
\item{sigma}{true standard deviation.}
\item{mu0}{in-control mean.}
\item{sigma0}{in-control standard deviation.}
\item{hsx}{so-called headstart (enables fast initial response) of the mean chart -- 
do not confuse with the true FIR feature considered in xewma.arl; will be updated.}
\item{Nx}{dimension of the approximating matrix of the mean chart.}
\item{csl}{lower control limit of the variance EWMA control chart; default value is 0;
not considered if \code{sided} is \code{"upper"}.}
\item{hss}{headstart (enables fast initial response) of the variance chart.}
\item{Ns}{dimension of the approximating matrix of the variance chart.}
\item{sided}{distinguishes between one- and two-sided two-sided
EWMA-\eqn{S^2}{S^2} control charts by choosing \code{"upper"} (upper chart without
reflection at \code{cl} -- the actual value of of \code{cl} is not used).}
\item{mode}{only deployed for \code{sided}=\code{"two"} -- with \code{"fixed"}
an upper control limit (see \code{cu}) is set and only the lower is
determined to obtain the in-control ARL \code{L0}, while with \code{"unbiased"}
a certain unbiasedness of the ARL function is guaranteed (here, both the
lower and the upper control limit are calculated).}
\item{qm}{number of quadrature nodes used for the collocation integrals.}
\item{c.error}{error bound for two succeeding values of the critical value
during applying the secant rule.}
\item{a.error}{error bound for the quantile level \code{alpha} during
applying the secant rule.}
}
\details{
Instead of the popular ARL (Average Run Length) quantiles of the EWMA
stopping time (Run Length) are determined. The algorithm is based on
Waldmann's survival function iteration procedure and on Knoth (2007).
\code{xsewma.q.crit} determines the critical values (similar to alarm limits)
for given in-control RL quantile \code{L0} at level \code{alpha} by applying secant
rule and using \code{xsewma.sf()}.
In case of \code{sided}=\code{"two"} and \code{mode}=\code{"unbiased"}
a two-dimensional secant rule is applied that also ensures that the
maximum of the RL cdf for given standard deviation is attained at \code{sigma0}.
}
\value{Returns a single value which resembles the RL quantile of order \code{alpha} and
the critical value of the two-sided mean EWMA chart and
the lower and upper controls limit \code{csl} and \code{csu} of the
variance EWMA chart, respectively.}
\references{

S. Knoth (2007),
Accurate ARL calculation for EWMA control charts monitoring simultaneously
normal mean and variance,
\emph{Sequential Analysis 26}, 251-264.
}
\author{Sven Knoth}
\seealso{\code{xsewma.arl} for calculation of ARL of simultaneous EWMA charts and
\code{xsewma.sf} for the RL survival function.}
\examples{
## will follow
}
\keyword{ts}

\name{imr.RuRl_alone}
\alias{imr.RuRl_alone}
\alias{imr.RuRl_alone_s3}
\alias{imr.RuRl_alone_tail}
\alias{imr.Ru_Rlgiven}
\alias{imr.Rl_Rugiven}
\title{Compute control limits of MR charts for normal data}
\description{Computation of control limits of standalone MR charts.}
\usage{imr.RuRl_alone(L0, N=30, qm=30, M0=12, eps=1e-3)

imr.RuRl_alone_s3(L0, N=30, qm=30, M0=12)

imr.RuRl_alone_tail(L0, N=30, qm=30, M0=12)

imr.Ru_Rlgiven(Rl, L0, N=30, qm=30, M0=12)

imr.Rl_Rugiven(Ru, L0, N=30, qm=30, M0=12)}
\arguments{
\item{L0}{pre-defined in-control ARL, that is, determine \code{Ru} and \code{Rl} so that the mean
number of observations until a false alarm is \code{L0}.}
\item{N}{controls the dimension of the linear equation system and consequently the accuracy of the result. See details.}
\item{qm}{number of quadrature nodes (and weights) to determine the definite collocation integrals.}
\item{M0}{mimics Inf --- by setting \code{M0} to some large value (having a standard normal distribution in mind),
the algorithm for IMR charts could be used as well for the standalone MR chart.}
\item{eps}{resolution parameter, which controls the approximation of the ARL slope at the in-control level of
the monitored standard deviation. It ensures the pattern that is called ARL unbiasedness. A small value is recommended.}
\item{Rl}{lower control limit multiple for moving range chart.}
\item{Ru}{upper control limit multiple for moving range chart.}
}
\details{
Crowder (1987a) provided some math to determine the ARL of the so-called individual moving range (IMR) chart,
which consists of the mean X chart and the standard deviation MR chart. 
Making the alarm threshold, \code{M0}, huge (default value here is 12) for the X chart allows us to utilize Crowder's
setup for standalone MR charts. For details about the IMR numerics see \code{imr.arl}.
The three different versions of \code{imr.RuRl_alone} determine limits that form an ARL unbiased design, follow the restriction
\code{Rl} = \code{1/Ru^3} and feature equal probability tails for the MR's half-normal distribution,
respectively in the order given above).
The other two functions are helper routines for \code{imr.RuRl_alone}.
Note that the elegant approach given in Acosta-Mejia/Pignatiello (2000) is only an approximation,
because the MR series is not Markovian.
}
\value{Returns control limit factors (alias multiples).}
\references{
S. V. Crowder (1987a)
Computation of ARL for Combined Individual Measurement and Moving Range Charts,
\emph{Journal of Quality Technology} 19(2), 98-102.
  
S. V. Crowder (1987b)
A Program for the Computation of ARL for Combined Individual Measurement and Moving Range Charts,
\emph{Journal of Quality Technology} 19(2), 103-106.

D. Radson, L. C. Alwan (1995)
Detecting Variance Reductions Using the Moving Range,
\emph{Quality Engineering} 8(1), 165-178.

C. A. Acosta-Mejia, J. J. Pignatiello (2000)
Monitoring process dispersion without subgrouping,
\emph{Journal of Quality Technology} 32(2), 89-102.
}
\author{Sven Knoth}
\seealso{later.}
\examples{
## Radson, Alwan (1995), Table 2 (Monte Carlo based), half-normal, known parameter case
## two-sided MR-alone chart, hence the ARL results has to be decreased by 1
## Here: a large M0=12 (default of the functions above) is deployed to mimic Inf
alpha <- 0.00915
Ru <- sqrt(2) * qnorm(1-alpha/4)
Rl <- sqrt(2) * qnorm(0.5+alpha/4)
M0 <- 12
\dontrun{
ARL0 <- imr.arl(M0, Ru, 0, 1, vsided="two", Rl=Rl)
RRR1995 <- imr.RuRl_alone_tail(ARL0)
RRRs <- imr.RuRl_alone_s3(ARL0)
RRR <- imr.RuRl_alone(ARL0)
results <- rbind(c(Rl, Ru), RRR1995, RRRs, RRR)
results}
}
\keyword{ts}

\name{sewma.arl.prerun}
\alias{sewma.arl.prerun}
\title{Compute ARLs of EWMA control charts (variance charts) in case of estimated parameters}
\description{Computation of the (zero-state) Average Run Length (ARL)
for EWMA control charts  (based on the sample variance \eqn{S^2})
monitoring normal variance with estimated parameters.}
\usage{sewma.arl.prerun(l, cl, cu, sigma, df1, df2, hs=1, sided="upper",
r=40, qm=30, qm.sigma=30, truncate=1e-10)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{cl}{lower control limit of the EWMA control chart.}
\item{cu}{upper control limit of the EWMA control chart.}
\item{sigma}{true standard deviation.}
\item{df1}{actual degrees of freedom, corresponds to subgroup size (for known mean it is equal to the subgroup size,
for unknown mean it is equal to subgroup size minus one.}
\item{df2}{degrees of freedom of the pre-run variance estimator.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided two-sided EWMA-\eqn{S^2}{S^2} control charts 
by choosing \code{"upper"} (upper chart without reflection at \code{cl} -- the actual value of
\code{cl} is not used), \code{"Rupper"} (upper chart with reflection at \code{cl}),\code{"Rlower"}
(lower chart with reflection at \code{cu}),
and \code{"two"} (two-sided chart), respectively.}
\item{r}{dimension of the resulting linear equation system (highest order of the collocation polynomials).}
\item{qm}{number of quadrature nodes for calculating the collocation definite integrals.}
\item{qm.sigma}{number of quadrature nodes for convoluting the standard deviation uncertainty.}
\item{truncate}{size of truncated tail.}
}
\details{
Essentially, the ARL function \code{sewma.arl} is convoluted with the
distribution of the sample standard deviation.
For details see Jones/Champ/Rigdon (2001) and Knoth (2014?).}
\value{Returns a single value which resembles the ARL.}
\references{
L. A. Jones, C. W. Champ, S. E. Rigdon (2001),
The performance of exponentially weighted moving average charts with estimated parameters,
\emph{Technometrics 43}, 156-167.

S. Knoth (2005),
Accurate ARL computation for EWMA-\eqn{S^2}{S^2} control charts,
\emph{Statistics and Computing 15}, 341-352.

S. Knoth (2006),
Computation of the ARL for CUSUM-\eqn{S^2}{S^2} schemes,
\emph{Computational Statistics & Data Analysis 51}, 499-512.
}
\author{Sven Knoth}
\seealso{
\code{sewma.arl} for zero-state ARL function of EWMA control charts w/o pre run uncertainty.
}
\examples{
## will follow
}
\keyword{ts}

\name{xcusum.ad}
\alias{xcusum.ad}
\title{Compute steady-state ARLs of CUSUM control charts}
\description{Computation of the steady-state Average Run Length (ARL)
for different types of CUSUM control charts monitoring normal mean.}
\usage{xcusum.ad(k, h, mu1, mu0 = 0, sided = "one", r = 30)}
\arguments{
\item{k}{reference value of the CUSUM control chart.}
\item{h}{decision interval (alarm limit, threshold) of the CUSUM control chart.}
\item{mu1}{out-of-control mean.}
\item{mu0}{in-control mean.}
\item{sided}{distinguish between one-, two-sided and Crosier's modified
two-sided CUSUM scheme by choosing \code{"one"}, \code{"two"},
and \code{"Crosier"}, respectively.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-, two-sided) or \code{2r+1}
(Crosier).}
}
\details{
\code{xcusum.ad} determines the steady-state Average Run Length (ARL)
by numerically solving the related ARL integral equation by means
of the Nystroem method based on Gauss-Legendre quadrature
and using the power method for deriving the largest in magnitude
eigenvalue and the related left eigenfunction.
}
\value{Returns a single value which resembles the steady-state ARL.}
\references{
R. B. Crosier (1986),
A new two-sided cumulative quality control scheme,
\emph{Technometrics 28}, 187-194.
}
\note{Be cautious in increasing the dimension parameter \code{r} for
two-sided CUSUM schemes. The resulting matrix dimension is \code{r^2} times
\code{r^2}. Thus, go beyond 30 only on fast machines. This is the only case,
were the package routines are based on the Markov chain approach. Moreover,
the two-sided CUSUM scheme needs a two-dimensional Markov chain.} 
\author{Sven Knoth}
\seealso{
\code{xcusum.arl} for zero-state ARL computation and
\code{xewma.ad} for the steady-state ARL of EWMA control charts.
}
\examples{
## comparison of zero-state (= worst case ) and steady-state performance
## for one-sided CUSUM control charts

k <- .5
h <- xcusum.crit(k,500)
mu <- c(0,.5,1,1.5,2)
arl <- sapply(mu,k=k,h=h,xcusum.arl)
ad <- sapply(mu,k=k,h=h,xcusum.ad)
round(cbind(mu,arl,ad),digits=2)

## Crosier (1986), Crosier's modified two-sided CUSUM
## He introduced the modification and evaluated it by means of
## Markov chain approximation

k <- .5
h2 <- 4
hC <- 3.73
mu <- c(0,.25,.5,.75,1,1.5,2,2.5,3,4,5)
ad2 <- sapply(mu,k=k,h=h2,sided="two",r=20,xcusum.ad)
adC <- sapply(mu,k=k,h=hC,sided="Crosier",xcusum.ad)
round(cbind(mu,ad2,adC),digits=2)

## results in the original paper are (in Table 5)
## 0.00 163.   164.
## 0.25  71.6   69.0
## 0.50  25.2   24.3
## 0.75  12.3   12.1
## 1.00   7.68   7.69
## 1.50   4.31   4.39
## 2.00   3.03   3.12
## 2.50   2.38   2.46
## 3.00   2.00   2.07
## 4.00   1.55   1.60
## 5.00   1.22   1.29
}
\keyword{ts}

\name{xshewhart.ar1.arl}
\alias{xshewhart.ar1.arl}
\title{Compute ARLs of modified Shewhart control charts for AR(1) data}
\description{Computation of the (zero-state) Average Run Length (ARL)
for modified Shewhart charts deployed to the original AR(1) data.}
\usage{xshewhart.ar1.arl(alpha, cS, delta=0, N1=50, N2=30)}
\arguments{
\item{alpha}{lag 1 correlation of the data.}
\item{cS}{critical value (alias to alarm limit) of the Shewhart control chart.}
\item{delta}{potential shift in the data (in-control mean is zero.}
\item{N1}{number of quadrature nodes for solving the ARL integral equation, dimension of the resulting linear equation system is \code{N1}.}
\item{N2}{second number of quadrature nodes for combining the probability density function of the first observation following the margin distribution and the solution of the ARL integral equation.}
}
\details{
Following the idea of Schmid (1995), \code{1- alpha} times the data turns out to be an
EWMA smoothing of the underlying AR(1) residuals. Hence, by combining the solution of
the EWMA ARL integral equation and the stationary distribution of the AR(1) data
(normal distribution is assumed) one gets easily the overall ARL.
}
\value{It returns a single value resembling the zero-state ARL of a modified Shewhart chart.}
\references{
S. Knoth, W. Schmid (2004).
Control charts for time series: A review.
In \emph{Frontiers in Statistical Quality Control 7},
edited by H.-J. Lenz, P.-T. Wilrich, 210-236, Physica-Verlag.

H. Kramer, W. Schmid (2000).
The influence of parameter estimation on the ARL of Shewhart type charts for time series.
\emph{Statistical Papers 41}(2), 173-196.

W. Schmid (1995).
On the run length of a Shewhart chart for correlated data. 
\emph{Statistical Papers 36}(1), 111-130.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} for zero-state ARL computation of EWMA control charts.
}
\examples{
## Table 1 in Kramer/Schmid (2000)

cS <- 3.09023
a  <- seq(0, 4, by=.5)
row1 <- row2 <- row3 <- NULL
for ( i in 1:length(a) ) {
  row1 <- c(row1, round(xshewhart.ar1.arl( 0.4, cS, delta=a[i]), digits=2))
  row2 <- c(row2, round(xshewhart.ar1.arl( 0.2, cS, delta=a[i]), digits=2))
  row3 <- c(row3, round(xshewhart.ar1.arl(-0.2, cS, delta=a[i]), digits=2))
}

results <- rbind(row1, row2, row3)
results

# original values are
# row1 515.44 215.48 61.85 21.63 9.19 4.58 2.61 1.71 1.29
# row2 502.56 204.97 56.72 19.13 7.95 3.97 2.33 1.59 1.25
# row3 502.56 201.41 54.05 17.42 6.89 3.37 2.03 1.46 1.20
}
\keyword{ts}

\name{xDgrsr.arl}
\alias{xDgrsr.arl}
\title{Compute ARLs of Shiryaev-Roberts schemes under drift}
\description{Computation of the (zero-state and other)
Average Run Length (ARL) under drift
for Shiryaev-Roberts schemes monitoring normal mean.}
\usage{xDgrsr.arl(k, g, delta, zr = 0, hs = NULL, sided = "one", m = NULL,
mode = "Gan", q = 1, r = 30, with0 = FALSE)}
\arguments{
\item{k}{reference value of the Shiryaev-Roberts scheme.}
\item{g}{control limit (alarm threshold) of Shiryaev-Roberts scheme.}
\item{delta}{true drift parameter.}
\item{zr}{reflection border for the one-sided chart.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided 
Shiryaev-Roberts schemes
by choosing \code{"one"} and \code{"two"}, respectively. Currentlly,
the two-sided scheme is not implemented.}
\item{m}{parameter used if \code{mode="Gan"}. \code{m} is design
parameter of Gan's approach. If \code{m=NULL}, then \code{m}
will increased until the resulting ARL does not change anymore.}
\item{q}{change point position. For \eqn{q=1} and
\eqn{\mu=\mu_0} and \eqn{\mu=\mu_1}, the usual
zero-state ARLs for the in-control and out-of-control case, respectively,
are calculated. For \eqn{q>1} and \eqn{\mu!=0} conditional delays, that is,
\eqn{E_q(L-q+1|L\geq)}, will be determined.
Note that mu0=0 is implicitely fixed. Deploy large \code{q} to mimic
steady-state. It works only for \code{mode="Knoth"}.}
\item{mode}{decide whether Gan's or Knoth's approach is used. Use
\code{"Gan"} and \code{"Knoth"}, respectively.
\code{"Knoth"} is not implemented yet.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-sided) or \code{r} (two-sided).}
\item{with0}{defines whether the first observation used for the RL calculation
follows already 1*delta or still 0*delta.
With \code{q} additional flexibility is given.}
}
\details{
Based on Gan (1991) or Knoth (2003), the ARL is calculated for Shiryaev-Roberts schemes under drift.
In case of Gan's framework, the usual
ARL function with mu=m*delta is determined and recursively via
m-1, m-2, ... 1 (or 0) the drift ARL determined. The framework
of Knoth allows to calculate ARLs for varying parameters, such as
control limits and distributional parameters.
For details see the cited papers.
}
\value{Returns a single value which resembles the ARL.}
\references{
F. F. Gan (1991),
EWMA control chart under linear drift,
\emph{J. Stat. Comput. Simulation 38}, 181-200.

S. Knoth (2003),
EWMA schemes with non-homogeneous transition kernels,
\emph{Sequential Analysis 22}, 241-255.

S. Knoth (2012),
More on Control Charting under Drift,
in: \emph{Frontiers in Statistical Quality Control 10},
H.-J. Lenz, W. Schmid and P.-T. Wilrich (Eds.),
Physica Verlag, Heidelberg, Germany, 53-68.

C. Zou, Y. Liu and Z. Wang (2009),
Comparisons of control schemes for monitoring
the means of processes subject to drifts,
\emph{Metrika 70}, 141-163.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} and \code{xewma.ad} for zero-state and
steady-state ARL computation of EWMA control charts
for the classical step change model.
}
\examples{
\dontrun{
## Monte Carlo example with 10^8 replicates
#   delta      arl    s.e.
#   0.0001 381.8240   0.0304
#   0.0005 238.4630   0.0148
#   0.001  177.4061   0.0097
#   0.002  125.9055   0.0061
#   0.005   75.7574   0.0031
#   0.01    50.2203   0.0018
#   0.02    32.9458   0.0011
#   0.05    18.9213   0.0005
#   0.1     12.6054   0.0003
#   0.5      5.2157   0.0001
#   1        3.6537   0.0001
#   3        2.0289   0.0000
k <- .5
L0 <- 500
zr <- -7
r <- 50
g <- xgrsr.crit(k, L0, zr=zr, r=r)
DxDgrsr.arl <- Vectorize(xDgrsr.arl, "delta")
deltas <- c(0.0001, 0.0005, 0.001, 0.002, 0.005, 0.01, 0.02, 0.05, 0.1, 0.5, 1, 3)
arls <- round(DxDgrsr.arl(k, g, deltas, zr=zr, r=r), digits=4)
data.frame(deltas, arls)
}
}
\keyword{ts}

\name{xsewma.sf}
\alias{xsewma.sf}
\title{Compute the survival function of simultaneous EWMA control
charts (mean and variance charts)}
\description{Computation of the survival function of the Run Length (RL)
for EWMA control charts monitoring simultaneously normal mean and variance.}
\usage{xsewma.sf(n, lx, cx, ls, csu, df, mu, sigma, hsx=0, Nx=40,
csl=0, hss=1, Ns=40, sided="upper", qm=30) }
\arguments{
\item{n}{calculate sf up to value \code{n}.}
\item{lx}{smoothing parameter lambda of the two-sided mean EWMA chart.}
\item{cx}{control limit of the two-sided mean EWMA control chart.}
\item{ls}{smoothing parameter lambda of the variance EWMA chart.}
\item{csu}{upper control limit of the variance EWMA control chart.}
\item{df}{actual degrees of freedom, corresponds to subgroup size (for known mean it is
equal to the subgroup size, for unknown mean it is equal to subgroup size minus one.}
\item{mu}{true mean.}
\item{sigma}{true standard deviation.}
\item{hsx}{so-called headstart (enables fast initial response) of the mean chart --
do not confuse with the true FIR feature considered in xewma.arl; will be updated.}
\item{Nx}{dimension of the approximating matrix of the mean chart.}
\item{csl}{lower control limit of the variance EWMA control chart; default value is 0;
not considered if \code{sided} is \code{"upper"}.}
\item{hss}{headstart (enables fast initial response) of the variance chart.}
\item{Ns}{dimension of the approximating matrix of the variance chart.}
\item{sided}{distinguishes between one- and two-sided two-sided
EWMA-\eqn{S^2}{S^2} control charts by choosing \code{"upper"} (upper chart
without reflection at \code{cl} -- the actual value of
\code{cl} is not used), \code{"Rupper"} (upper chart with reflection at \code{cl}),
\code{"Rlower"} (lower chart with reflection at \code{cu}), and \code{"two"}
(two-sided chart), respectively.} 
\item{qm}{number of quadrature nodes used for the collocation integrals.}
}
\details{
The survival function P(L>n) and derived from it also the cdf P(L<=n) and
the pmf P(L=n) illustrate
the distribution of the EWMA run length. For large n the geometric tail
could be exploited. That is,
with reasonable large n the complete distribution is characterized.
The algorithm is based on Waldmann's survival function iteration procedure and
on results in Knoth (2007).
}
\value{Returns a vector which resembles the survival function up to a certain point.}
\references{
S. Knoth (2007),
Accurate ARL calculation for EWMA control charts monitoring simultaneously
normal mean and variance,
\emph{Sequential Analysis 26}, 251-264.

K.-H. Waldmann (1986),
Bounds for the distribution of the run length of geometric moving 
average charts, \emph{Appl. Statist. 35}, 151-158.
}
\author{Sven Knoth}
\seealso{
\code{xsewma.arl} for zero-state ARL computation of simultaneous EWMA
control charts.
}
\examples{
## will follow
}
\keyword{ts}

\name{xcusum.crit}
\alias{xcusum.crit}
\title{Compute decision intervals of CUSUM control charts}
\description{Computation of the decision intervals (alarm limits)
for different types of CUSUM control charts monitoring normal mean.}
\usage{xcusum.crit(k, L0, mu0 = 0, hs = 0, sided = "one", r = 30)}
\arguments{
\item{k}{reference value of the CUSUM control chart.}
\item{L0}{in-control ARL.}
\item{mu0}{in-control mean.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one-, two-sided and Crosier's modified
two-sided CUSUM scheme by choosing \code{"one"}, \code{"two"}, and \code{"Crosier"}, respectively.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-, two-sided) or \code{2r+1}
(Crosier).}
}
\details{
\code{xcusum.crit} determines the decision interval (alarm limit)
for given in-control ARL \code{L0} by applying secant rule and using \code{xcusum.arl()}.
}
\value{Returns a single value which resembles the decision interval
\code{h}.}
%\references{Later...}
\author{Sven Knoth}
\seealso{\code{xcusum.arl} for zero-state ARL computation.}
\examples{
k <- .5
incontrolARL <- c(500,5000,50000)
sapply(incontrolARL,k=k,xcusum.crit,r=10) # accuracy with 10 nodes
sapply(incontrolARL,k=k,xcusum.crit,r=20) # accuracy with 20 nodes
sapply(incontrolARL,k=k,xcusum.crit)      # accuracy with 30 nodes
}
\keyword{ts}

\name{xewma.crit}
\alias{xewma.crit}
\title{Compute critical values of EWMA control charts}
\description{Computation of the critical values (similar to alarm limits)
for different types of EWMA control charts monitoring normal mean.}
\usage{xewma.crit(l,L0,mu0=0,zr=0,hs=0,sided="one",limits="fix",r=40,c0=NULL,nmax=10000)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{L0}{in-control ARL.}
\item{mu0}{in-control mean.}
\item{zr}{reflection border for the one-sided chart.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided
two-sided EWMA control chart by choosing \code{"one"} and \code{"two"},
respectively.}
\item{limits}{distinguishes between different control limits behavior.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-sided) or \code{r}
(two-sided).}
\item{c0}{starting value for iteration rule.}
\item{nmax}{maximum number of individual control limit factors for \code{"cfar"}.}
}
\details{
\code{xewma.crit} determines the critical values (similar to alarm limits)
for given in-control ARL \code{L0}
by applying secant rule and using \code{xewma.arl()}.
}
\value{Returns a single value which resembles the critical value
\code{c}.}
\references{
S. V. Crowder (1989), Design of exponentially weighted moving average
schemes, \emph{Journal of Quality Technology 21}, 155-162.
}
\author{Sven Knoth}
\seealso{\code{xewma.arl} for zero-state ARL computation.}
\examples{
l <- .1
incontrolARL <- c(500,5000,50000)
sapply(incontrolARL,l=l,sided="two",xewma.crit,r=35) # accuracy with 35 nodes
sapply(incontrolARL,l=l,sided="two",xewma.crit)      # accuracy with 40 nodes
sapply(incontrolARL,l=l,sided="two",xewma.crit,r=50) # accuracy with 50 nodes

## Crowder (1989)
## two-sided EWMA control charts with fixed limits

l <- c(.05,.1,.15,.2,.25)
L0 <- 250
round(sapply(l,L0=L0,sided="two",xewma.crit),digits=2)

## original values are 2.32, 2.55, 2.65, 2.72, and 2.76.
}
\keyword{ts}

\name{xsewma.arl}
\alias{xsewma.arl}
\title{Compute ARLs of simultaneous EWMA control charts (mean and variance charts)}
\description{Computation of the (zero-state) Average Run Length (ARL)
for different types of simultaneous EWMA control charts 
(based on the sample mean and the sample variance \eqn{S^2})
monitoring normal mean and variance.}
\usage{xsewma.arl(lx, cx, ls, csu, df, mu, sigma, hsx=0, Nx=40, csl=0,
hss=1, Ns=40, s2.on=TRUE, sided="upper", qm=30)}
\arguments{
\item{lx}{smoothing parameter lambda of the two-sided mean EWMA chart.}
\item{cx}{control limit of the two-sided mean EWMA control chart.}
\item{ls}{smoothing parameter lambda of the variance EWMA chart.}
\item{csu}{upper control limit of the variance EWMA control chart.}
\item{df}{actual degrees of freedom, corresponds to subgroup size (for known mean
it is equal to the subgroup size,
for unknown mean it is equal to subgroup size minus one.}
\item{mu}{true mean.}
\item{sigma}{true standard deviation.}
\item{hsx}{so-called headstart (enables fast initial response) of the mean chart -- 
do not confuse with the true FIR feature considered in xewma.arl; will be updated.}
\item{Nx}{dimension of the approximating matrix of the mean chart.}
\item{csl}{lower control limit of the variance EWMA control chart; default value is 0;
not considered if \code{sided} is \code{"upper"}.}
\item{hss}{headstart (enables fast initial response) of the variance chart.}
\item{Ns}{dimension of the approximating matrix of the variance chart.}
\item{s2.on}{distinguishes between \eqn{S^2}{S^2} and \eqn{S}{S} chart.}
\item{sided}{distinguishes between one- and two-sided two-sided EWMA-\eqn{S^2}{S^2}
control charts by choosing \code{"upper"} (upper chart without reflection at \code{cl}
-- the actual value of \code{cl} is not used), \code{"Rupper"} (upper chart with
reflection at \code{cl}),
\code{"Rlower"} (lower chart with reflection at \code{cu}), and \code{"two"}
(two-sided chart), respectively.} 
\item{qm}{number of quadrature nodes used for the collocation integrals.}
}
\details{
\code{xsewma.arl} determines the Average Run Length (ARL) by 
an extension of Gan's (derived from ideas already published by Waldmann)
algorithm. The variance EWMA part is treated
similarly to the ARL calculation method
deployed for the single variance EWMA charts in Knoth (2005), that is, by means of 
collocation (Chebyshev polynomials). For more details see Knoth (2007).}
\value{Returns a single value which resembles the ARL.}
\references{
K. Waldmann (1986),
Bounds for the distribution of the run length of geometric moving average charts,
\emph{J. R. Stat. Soc., Ser. C, Appl. Stat. 35}, 151-158.

F. F. Gan (1995),
Joint monitoring of process mean and variance using exponentially weighted moving
average control charts,
\emph{Technometrics 37}, 446-453.

S. Knoth (2005),
Accurate ARL computation for EWMA-\eqn{S^2}{S^2} control charts,
\emph{Statistics and Computing 15}, 341-352.

S. Knoth (2007),
Accurate ARL calculation for EWMA control charts monitoring simultaneously normal
mean and variance,
\emph{Sequential Analysis 26}, 251-264.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} and \code{sewma.arl} for zero-state ARL computation of
single mean and variance EWMA control charts, respectively.
}
\examples{
## Knoth (2007)
## collocation results in Table 1
## Monte Carlo with 10^9 replicates: 252.307 +/- 0.0078

# process parameters
mu <- 0
sigma <- 1
# subgroup size n=5, df=n-1
df  <- 4
# lambda of mean chart
lx  <- .134
# c_mu^* = .345476571 = cx/sqrt(n) * sqrt(lx/(2-lx)
cx  <- .345476571*sqrt(df+1)/sqrt(lx/(2-lx))
# lambda of variance chart
ls  <- .1
# c_sigma = .477977
csu <- 1 + .477977
# matrix dimensions for mean and variance part
Nx  <- 25
Ns  <- 25
# mode of variance chart
SIDED <- "upper"

arl <- xsewma.arl(lx, cx, ls, csu, df, mu, sigma, Nx=Nx, Ns=Ns, sided=SIDED)
arl
}
\keyword{ts}

\name{sewma.arl}
\alias{sewma.arl}
\title{Compute ARLs of EWMA control charts (variance charts)}
\description{Computation of the (zero-state) Average Run Length (ARL)
for different types of EWMA control charts (based on the sample variance
\eqn{S^2}) monitoring normal variance.}
\usage{sewma.arl(l,cl,cu,sigma,df,s2.on=TRUE,hs=NULL,sided="upper",r=40,qm=30)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{cl}{lower control limit of the EWMA control chart.}
\item{cu}{upper control limit of the EWMA control chart.}
\item{sigma}{true standard deviation.}
\item{df}{actual degrees of freedom, corresponds to subgroup
size (for known mean it is equal to the subgroup size,
for unknown mean it is equal to subgroup size minus one.}
\item{s2.on}{distinguishes between \eqn{S^2}{S^2} and \eqn{S}{S} chart.}
\item{hs}{so-called headstart (enables fast initial response);
the default (\code{NULL}) yields the expected in-control value of
\eqn{S^2}{S^2} (1) and \eqn{S}{S} (\eqn{c_4}{c_4}), respectively.}
\item{sided}{distinguishes between one- and two-sided 
two-sided EWMA-\eqn{S^2}{S^2} control charts 
by choosing \code{"upper"} (upper chart
without reflection at \code{cl} -- the actual value of
\code{cl} is not used),
\code{"Rupper"} (upper chart with reflection at \code{cl}),
\code{"Rlower"} (lower chart with reflection at \code{cu}),
and \code{"two"} (two-sided chart), 
respectively.} 
\item{r}{dimension of the resulting linear equation system (highest order of
the collocation polynomials).}
\item{qm}{number of quadrature nodes for calculating the collocation
definite integrals.}
}
\details{
\code{sewma.arl} determines the Average Run Length (ARL) by numerically
solving the related ARL integral equation by means of 
collocation (Chebyshev polynomials).}
\value{Returns a single value which resembles the ARL.}
\references{
S. Knoth (2005),
Accurate ARL computation for EWMA-\eqn{S^2}{S^2} control charts,
\emph{Statistics and Computing 15}, 341-352.

S. Knoth (2006),
Computation of the ARL for CUSUM-\eqn{S^2}{S^2} schemes,
\emph{Computational Statistics & Data Analysis 51}, 499-512.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} for zero-state ARL computation of EWMA control charts
for monitoring normal mean.
}
\examples{
## Knoth (2005)
## compare with Table 1 (p. 347): 249.9997
## Monte Carlo with 10^9 replicates: 249.9892 +/- 0.008
l <- .025
df <- 1
cu <- 1 + 1.661865*sqrt(l/(2-l))*sqrt(2/df)
sewma.arl(l,0,cu,1,df)

## ARL values for upper and lower EWMA charts with reflecting barriers
## (reflection at in-control level sigma0 = 1)
## examples from Knoth (2006), Tables 4 and 5

Ssewma.arl <- Vectorize("sewma.arl", "sigma")

## upper chart with reflection at sigma0=1 in Table 4
## original entries are
# sigma   ARL
# 1       100.0
# 1.01    85.3
# 1.02    73.4
# 1.03    63.5
# 1.04    55.4
# 1.05    48.7
# 1.1     27.9
# 1.2     12.9
# 1.3     7.86
# 1.4     5.57
# 1.5     4.30
# 2       2.11

\dontrun{
l <- 0.15
df <- 4
cu <- 1 + 2.4831*sqrt(l/(2-l))*sqrt(2/df)
sigmas <- c(1 + (0:5)/100, 1 + (1:5)/10, 2)
arls <- round(Ssewma.arl(l, 1, cu, sigmas, df, sided="Rupper", r=100), digits=2)
data.frame(sigmas, arls)}

## lower chart with reflection at sigma0=1 in Table 5
## original entries are
# sigma   ARL
# 1       200.04
# 0.9     38.47
# 0.8     14.63
# 0.7     8.65
# 0.6     6.31

\dontrun{
l <- 0.115
df <- 5
cl <- 1 - 2.0613*sqrt(l/(2-l))*sqrt(2/df)
sigmas <- c((10:6)/10)
arls <- round(Ssewma.arl(l, cl, 1, sigmas, df, sided="Rlower", r=100), digits=2)
data.frame(sigmas, arls)}
}
\keyword{ts}

\name{scusum.crit}
\alias{scusum.crit}
\title{Compute decision intervals of CUSUM control charts (variance charts)}
\description{omputation of the decision intervals (alarm limits)
for different types of CUSUM control charts (based on the sample
variance \eqn{S^2}) monitoring normal variance.}
\usage{scusum.crit(k, L0, sigma, df, hs=0, sided="upper", mode="eq.tails",
k2=NULL, hs2=0, r=40, qm=30)}
\arguments{
\item{k}{reference value of the CUSUM control chart.}
\item{L0}{in-control ARL.}
\item{sigma}{true standard deviation.}
\item{df}{actual degrees of freedom, corresponds to subgroup size (for known mean it is equal
to the subgroup size, for unknown mean it is equal to subgroup size minus one.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided two-sided CUSUM-\eqn{S^2}{S^2} control charts 
by choosing \code{"upper"} (upper chart), \code{"lower"} (lower chart),
and \code{"two"} (two-sided chart), respectively. Note that for the two-sided chart the parameters
\code{"k2"} and \code{"h2"} have to be set too.}
\item{mode}{only deployed for \code{sided}=\code{"two"} -- with \code{"eq.tails"}
two one-sided CUSUM charts (lower and upper) with the same in-control ARL are coupled.
With \code{"unbiased"} a certain unbiasedness of the ARL function is guaranteed (here, both the
lower and the upper control limit are calculated).}
\item{k2}{in case of a two-sided CUSUM chart for variance the reference value of the lower chart.}
\item{hs2}{in case of a two-sided CUSUM chart for variance the headstart of the lower chart.}
\item{r}{Dimension of the resulting linear equation system (highest order of the collocation
polynomials times number of intervals -- see Knoth 2006).}
\item{qm}{Number of quadrature nodes for calculating the collocation definite integrals.}
}
\details{
\code{scusum.crit} ddetermines the decision interval (alarm limit)
for given in-control ARL \code{L0} by applying secant rule and using \code{scusum.arl()}.}
\value{Returns a single value which resembles the decision interval \code{h}.}
\references{
S. Knoth (2005),
Accurate ARL computation for EWMA-\eqn{S^2}{S^2} control charts,
\emph{Statistics and Computing 15}, 341-352.

S. Knoth (2006),
Computation of the ARL for CUSUM-\eqn{S^2}{S^2} schemes,
\emph{Computational Statistics & Data Analysis 51}, 499-512.
}
\author{Sven Knoth}
\seealso{
\code{xcusum.arl} for zero-state ARL computation of CUSUM control charts monitoring normal mean.
}
\examples{
## Knoth (2006)
## compare with Table 1 (p. 507)
k <- 1.46 # sigma1 = 1.5
df <- 1
L0 <- 260.74
h <- scusum.crit(k, L0, 1, df)
h
# original value is 10
}
\keyword{ts}

\name{pois.cusum.crit}
\alias{pois.cusum.crit}
\title{Compute alarm thresholds and randomization constants of Poisson CUSUM control charts}
\description{Computation of the CUSUM upper limit and, if needed, of the randomization probability, given mean \code{mu0}.}
\usage{pois.cusum.crit(mu0, km, A, m, i0=0, sided="upper", rando=FALSE)}
\arguments{
\item{mu0}{actual in-control mean.}
\item{km}{enumerator of rational approximation of reference value \code{k}.}
\item{A}{target in-control ARL (average run length).}
\item{m}{denominator of rational approximation of reference value.}
\item{i0}{head start value as integer multiple of \code{1/m}; should be an element of \code{0:100} (a more reasonable upper limit will be established soon). It is planned, to set \code{i0} as a fraction of the final threshold.}
\item{sided}{distinguishes between different one- and two-sided CUSUM control chart by choosing
\code{"upper"}, \code{"lower"} and \code{"two"}, respectively.}
\item{rando}{Switch for activating randomization in order to allow continuous ARL control.}
}
\details{
The monitored data follow a Poisson distribution with \code{mu} (here the in-control level \code{mu0}).
The ARL values of the resulting EWMA control chart are determined via Markov chain calculations.
With some grid search, we obtain the smallest value for the integer threshold component \code{hm} so that
the resulting ARL is not smaller than \code{A}. If equality is needed, then activating \code{rando=TRUE}
yields the corresponding randomization probability \code{gamma}.
More details will follow in a paper that will be submitted in 2020.
}
\value{Returns two single values, integer threshold \code{hm} resulting in the final
alarm threshold \code{h=hm/m}, and the randomization probability.}
\references{
J. M. Lucas (1985)
Counted data CUSUM's,
\emph{Technometrics} 27(2), 129-144.

C. H. White and J. B. Keats (1996)
ARLs and Higher-Order Run-Length Moments for the Poisson CUSUM,
\emph{Journal of Quality Technology} 28(3), 363-369.

C. H. White, J. B. Keats and J. Stanley (1997)
Poisson CUSUM versus c chart for defect data,
\emph{Quality Engineering} 9(4), 673-679.

G. Rossi and L. Lampugnani and M. Marchi (1999),
An approximate CUSUM procedure for surveillance of health events,
\emph{Statistics in Medicine} 18(16), 2111-2122.

S. W. Han, K.-L. Tsui, B. Ariyajunya, and S. B. Kim (2010),
A comparison of CUSUM, EWMA, and temporal scan statistics for detection of increases in poisson rates,
\emph{Quality and Reliability Engineering International} 26(3), 279-289.

M. B. Perry and J. J. Pignatiello Jr. (2011)
Estimating the time of step change with Poisson CUSUM and EWMA control charts,
\emph{International Journal of Production Research} 49(10), 2857-2871.

}
\author{Sven Knoth}
\seealso{later.}
\examples{
## Lucas 1985
mu0 <- 0.25
km <- 1
A <- 430
m  <- 4
#cv <- pois.cusum.crit(mu0, km, A, m)
cv <- c(40, 0)
# Lucas reported h = 10 alias hm = 40 (in Table 2, first block, row 10.0 .25 .0 ..., column 1.0
# Recall that Lucas and other trigger an alarm, if the CUSUM statistic is greater than
# or equal to the alarm threshold h
print(cv)
}
\keyword{ts}

\name{phat.ewma.arl}
\alias{phat.ewma.arl}
\alias{phat.ewma.crit}
\alias{phat.ewma.lambda}
\title{Compute ARLs of EWMA phat control charts}
\description{Computation of the (zero-state) Average Run Length (ARL), upper control limit (ucl)
for given in-control ARL, and lambda for minimal out-of control ARL at given shift.}
\usage{phat.ewma.arl(lambda, ucl, mu, n, z0, sigma=1, type="known", LSL=-3, USL=3, N=15,
qm=25, ntype="coll")

phat.ewma.crit(lambda, L0, mu, n, z0, sigma=1, type="known", LSL=-3, USL=3, N=15, qm=25)

phat.ewma.lambda(L0, mu, n, z0, sigma=1, type="known", max_l=1, min_l=.001, LSL=-3, USL=3,
qm=25)
}
\arguments{
\item{lambda}{smoothing parameter of the EWMA control chart.}
\item{ucl}{upper control limit of the EWMA phat control chart.}
\item{L0}{pre-defined in-control ARL (Average Run Length).}
\item{mu}{true mean or mean where the ARL should be minimized (then the in-control mean is simply 0).}
\item{n}{subgroup size.}
\item{z0}{so-called headstart (gives fast initial response).}
\item{type}{choose whether the standard deviation is given and fixed (\code{"known"}) or estimated and potentially monitored (\code{"estimated"}).}
\item{sigma}{actual standard deviation of the data -- the in-control value is 1.}
\item{max_l, min_l}{maximal and minimal value for optimal lambda search.}
\item{LSL,USL}{lower and upper specification limit, respectively.}
\item{N}{size of collocation base, dimension of the resulting linear equation system is equal to \code{N}.}
\item{qm}{number of nodes for collocation quadratures.}
\item{ntype}{switch between the default method \code{coll} (collocation) and the classic one \code{markov} (Markov chain approximation)
for calculating the ARL numerically.}
}
\details{
The three implemented functions allow to apply a new type control chart. Basically, lower and upper
specification limits are given. The monitoring
vehicle then is the empirical probability that an item will not follow these specification given the
sequence of sample means. If
the related EWMA sequence violates the control limits, then the alarm indicates a significant process
deterioration. For details see the
paper mentioned in the references. To be able to construct the control charts, see the first example.
}
\value{Return single values which resemble the ARL, the critical value, and the optimal lambda, respectively.}
\references{
S. Knoth and S. Steinmetz (2013),
EWMA \code{p} charts  under sampling by variables,
\emph{International Journal of Production Research} 51, 3795-3807.
}
\author{Sven Knoth}
\seealso{
\code{sewma.arl} for a further collocation based ARL calculation routine.}
\examples{
## Simple example to demonstrate the chart.

# some functions
h.mu <- function(mu) pnorm(LSL-mu) + pnorm(mu-USL)
ewma <- function(x, lambda=0.1, z0=0) filter(lambda*x, 1-lambda, m="r", init=z0)

# parameters
LSL <- -3       # lower specification limit
USL <-  3	# upper specification limit
n <- 5		# batch size
lambda <- 0.1	# EWMA smoothing parameter
L0 <- 1000	# in-control Average Run Length (ARL)
z0 <- h.mu(0)	# start at minimal defect level
ucl <- phat.ewma.crit(lambda, L0, 0, n, z0, LSL=LSL, USL=USL)

# data
x0 <- matrix(rnorm(50*n), ncol=5)	# in-control data
x1 <- matrix(rnorm(50*n, mean=0.5), ncol=5)# out-of-control data
x <- rbind(x0,x1)			# all data

# create chart
xbar <- apply(x, 1, mean)
phat <- h.mu(xbar)
z <- ewma(phat, lambda=lambda, z0=z0)
plot(1:length(z), z, type="l", xlab="batch", ylim=c(0,.02))
abline(h=z0, col="grey", lwd=.7)
abline(h=ucl, col="red")


## S. Knoth, S. Steinmetz (2013)

# Table 1

lambdas <- c(.5, .25, .2, .1)
L0 <- 370.4
n <- 5
LSL <- -3
USL <- 3

phat.ewma.CRIT <- Vectorize("phat.ewma.crit", "lambda")
p.star <- pnorm( LSL ) + pnorm( -USL ) ## lower bound of the chart
ucls <- phat.ewma.CRIT(lambdas, L0, 0, n, p.star, LSL=LSL, USL=USL)
print(cbind(lambdas, ucls))

# Table 2

mus <- c((0:4)/4, 1.5, 2, 3)
phat.ewma.ARL <- Vectorize("phat.ewma.arl", "mu")
arls <- NULL
for ( i in 1:length(lambdas) ) {
  arls <- cbind(arls, round(phat.ewma.ARL(lambdas[i], ucls[i], mus,
                n, p.star, LSL=LSL, USL=USL), digits=2))
}
arls <- data.frame(arls, row.names=NULL)
names(arls) <- lambdas
print(arls)

# Table 3

\dontrun{
mus <- c(.25, .5, 1, 2)
phat.ewma.LAMBDA <- Vectorize("phat.ewma.lambda", "mu")
lambdas <- phat.ewma.LAMBDA(L0, mus, n, p.star, LSL=LSL, USL=USL)
print(cbind(mus, lambdas))}
}
\keyword{ts}

\name{tewma.arl}
\alias{tewma.arl}
\title{Compute ARLs of Poisson TEWMA control charts}
\description{Computation of the (zero-state) Average Run Length (ARL) at given Poisson mean \code{mu}.}
\usage{tewma.arl(lambda, k, lk, uk, mu, z0, rando=FALSE, gl=0, gu=0)}
\arguments{
\item{lambda}{smoothing parameter of the EWMA p control chart.}
\item{k}{resolution of grid (natural number).}
\item{lk}{lower control limit of the TEWMA control chart, integer.}
\item{uk}{upper control limit of the TEWMA control chart, integer.}
\item{mu}{mean value of Poisson distribution.}
\item{z0}{so-called headstart (give fast initial response) -- it is proposed to use the in-control mean.}
\item{rando}{Distinguish between control chart design without or with randomisation. In the latter case some
meaningful values for \code{gl} and \code{gu} should be provided.}
\item{gl}{randomisation probability at the lower limit.}
\item{gu}{randomisation probability at the upper limit.}
}
\details{
A new idea of applying EWMA smoothing to count data. Here, the thinning operation is
applied to independent Poisson variates is performed. 
Moreover, the original thinning principle is expanded to multiples of one over \code{k} to allow finer
grids and finally better detection perfomance. It is highly recommended to read the
corresponding paper (see below).
}
\value{Return single value which resemble the ARL.}
\references{
M. C. Morais, C. H. Weiss, S. Knoth (2019),
A thinning-based EWMA chart to monitor counts, submitted.
}
\author{Sven Knoth}
\seealso{later.}
\examples{
# MWK (2018)
lambda <- 0.1 # (T)EWMA smoothing constant
mu0 <- 5 # in-control mean
k <- 10 # resolution
z0 <- round(k*mu0) # starting value of (T)EWMA sequence
# (i) without randomisation
lk <- 28
uk <- 75
L0 <- tewma.arl(lambda, k, lk, uk, mu0, z0)
# should be 501.9703
# (ii) with randomisation
uk <- 76 # lk is not changed
gl <- 0.5446310
gu <- 0.1375617
L0 <- tewma.arl(lambda, k, lk, uk, mu0, z0, rando=TRUE, gl=gl, gu=gu)
# should be 500
}
\keyword{ts}

\name{sewma.crit.prerun}
\alias{sewma.crit.prerun}
\title{Compute critical values of of EWMA (variance charts) control charts under pre-run uncertainty}
\description{Computation of quantiles of the Run Length (RL) for EWMA control
charts monitoring normal variance.}
\usage{sewma.crit.prerun(l,L0,df1,df2,sigma0=1,cl=NULL,cu=NULL,hs=1,sided="upper",
mode="fixed",r=40,qm=30,qm.sigma=30,truncate=1e-10,
tail_approx=TRUE,c.error=1e-10,a.error=1e-9)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{L0}{in-control quantile value.}
\item{df1}{actual degrees of freedom, corresponds to subgroup size
(for known mean it is equal to the subgroup size,
for unknown mean it is equal to subgroup size minus one.}
\item{df2}{degrees of freedom of the pre-run variance estimator.}
\item{sigma,sigma0}{true and in-control standard deviation, respectively.}
\item{cl}{deployed for \code{sided}=\code{"Rupper"}, that is, upper variance control chart with lower
reflecting barrier \code{cl}.}
\item{cu}{for two-sided (\code{sided}=\code{"two"}) and fixed upper control limit
(\code{mode}=\code{"fixed"}) a value larger than \code{sigma0}
has to been given, for all other cases \code{cu} is ignored.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided two-sided EWMA-\eqn{S^2}{S^2} control charts
by choosing \code{"upper"} (upper chart without reflection at \code{cl} -- the actual value of \code{cl}
is not used), \code{"Rupper"} (upper chart with reflection at \code{cl}), \code{"Rlower"} (lower chart
with reflection at \code{cu}),and \code{"two"} (two-sided chart), respectively.}
\item{mode}{only deployed for \code{sided}=\code{"two"} -- with \code{"fixed"} an upper control limit
(see \code{cu}) is set and only the lower is calculated to obtain the in-control ARL \code{L0}, while
with \code{"unbiased"} a certain unbiasedness of the ARL function is guaranteed (here, both the
lower and the upper control limit are calculated).}
\item{r}{dimension of the resulting linear equation system (highest order
of the collocation polynomials).}
\item{qm}{number of quadrature nodes for calculating the collocation definite integrals.}
\item{qm.sigma}{number of quadrature nodes for convoluting the standard deviation uncertainty.}
\item{truncate}{size of truncated tail.}
\item{tail_approx}{controls whether the geometric tail approximation is used (is faster) or not.}
\item{c.error}{error bound for two succeeding values of the critical value
during applying the secant rule.}
\item{a.error}{error bound for the quantile level \code{alpha} during applying the secant rule.}
}
\details{
\code{sewma.crit.prerun} determines the critical values (similar to alarm limits)
for given in-control ARL \code{L0}
by applying secant rule and using \code{sewma.arl.prerun()}.
In case of \code{sided}=\code{"two"} and \code{mode}=\code{"unbiased"}
a two-dimensional secant rule is applied that also ensures that the
maximum of the ARL function for given standard deviation is attained
at \code{sigma0}. See Knoth (2010) for some details of the algorithm involved.
}
\value{Returns the lower and upper control limit \code{cl} and \code{cu}.}
\references{
H.-J. Mittag and D. Stemann and B. Tewes (1998),
EWMA-Karten zur \"Uberwachung der Streuung von Qualit\"atsmerkmalen,
\emph{Allgemeines Statistisches Archiv 82}, 327-338,
S. Knoth (2005),
Accurate ARL computation for EWMA-\eqn{S^2}{S^2} control charts,
\emph{Statistics and Computing 15}, 341-352.

S. Knoth (2010),
Control Charting Normal Variance -- Reflections, Curiosities, and Recommendations,
in \emph{Frontiers in Statistical Quality Control 9},
H.-J. Lenz and P.-T. Wilrich (Eds.),
Physica Verlag, Heidelberg, Germany, 3-18.
}
\author{Sven Knoth}
\seealso{\code{sewma.arl.prerun} for calculation of ARL of variance charts under
pre-run uncertainty and \code{sewma.crit} for
the algorithm w/o pre-run uncertainty.}
\examples{
## will follow
}
\keyword{ts}

\name{xcusum.sf}
\alias{xcusum.sf}
\title{Compute the survival function of CUSUM run length}
\description{Computation of the survival function of the Run Length (RL) for CUSUM control charts monitoring normal mean.}
\usage{xcusum.sf(k, h, mu, n, hs=0, sided="one", r=40)}
\arguments{
\item{k}{reference value of the CUSUM control chart.}
\item{h}{decision interval (alarm limit, threshold) of the CUSUM control chart.}
\item{mu}{true mean.}
\item{n}{calculate sf up to value \code{n}.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided CUSUM control chart by choosing \code{"one"} and \code{"two"}, respectively.}
\item{r}{number of quadrature nodes, dimension of the resulting linear equation system is equal to \code{r+1}.}
}
\details{
The survival function P(L>n) and derived from it also the cdf P(L<=n) and the pmf P(L=n) illustrate
the distribution of the CUSUM run length. For large n the geometric tail could be exploited. That is,
with reasonable large n the complete distribution is characterized.
The algorithm is based on Waldmann's survival function iteration procedure.
}
\value{Returns a vector which resembles the survival function up to a certain point.}
\references{
K.-H. Waldmann (1986),
Bounds for the distribution of the run length of one-sided and two-sided CUSUM quality control schemes,
\emph{Technometrics 28}, 61-67.
}
\author{Sven Knoth}
\seealso{
\code{xcusum.q} for computation of CUSUM run length quantiles.
}
\examples{
## Waldmann (1986), one-sided CUSUM, Table 2

k <- .5
h <- 3
mu <- 0 # corresponds to Waldmann's -0.5
SF <- xcusum.sf(k, h, 0, 1000)
plot(1:length(SF), SF, type="l", xlab="n", ylab="P(L>n)", ylim=c(0,1))
#
}
\keyword{ts}

\name{pois.cusum.crit.L0L1}
\alias{pois.cusum.crit.L0L1}
\title{Compute the CUSUM k and h for given in-control ARL L0 and out-of-control ARL L1, Poisson case}
\description{Computation of the reference value k and the alarm threshold h
for one-sided CUSUM control charts monitoring Poisson data, if the in-control ARL L0 and the out-of-control ARL L1 are given.}
\usage{pois.cusum.crit.L0L1(mu0, L0, L1, sided="upper", OUTPUT=FALSE)}
\arguments{
\item{mu0}{in-control Poisson mean.}
\item{L0}{in-control ARL.}
\item{L1}{out-of-control ARL.}
\item{sided}{distinguishes between \code{"upper"} and \code{"lower"} CUSUM designs.}
\item{OUTPUT}{controls whether iteration details are printed.}
}
\details{
\code{pois.cusum.crit.L0L1} determines the reference value k and the alarm threshold h
for given in-control ARL \code{L0} and out-of-control ARL \code{L1}
by applying grid search and using \code{pois.cusum.arl()} and \code{pois.cusum.crit()}.
These CUSUM design rules were firstly (and quite rarely afterwards) used by Ewan and Kemp.
In the Poisson case, Rossi et al. applied them while analyzing three different normal
approximations of the Poisson distribution. See the example which illustrates
the validity of all these approaches.
}
\value{Returns a data frame with results for the denominator \code{m} of the rational approximation,
\code{km} as (integer) enumerator of the reference value (approximation), the corresponding
out-of-control mean \code{mu1}, the final approximation \code{k} of the reference value,
the threshold values \code{hm} (integer) and \code{h} (\code{=hm/m}), and the randomization constant
\code{gamma} (the target in-control ARL is exactly matched).}
\references{
W. D. Ewan and K. W. Kemp (1960),
Sampling inspection of continuous processes with no autocorrelation between successive results,
\emph{Biometrika} 47 (3/4), 363-380.

K. W. Kemp (1962),
The Use of Cumulative Sums for Sampling Inspection Schemes,
\emph{Journal of the Royal Statistical Sociecty C, Applied Statistics} 11(1), 16-31.

G. Rossi, L. Lampugnani and M. Marchi (1999),
An approximate CUSUM procedure for surveillance of health events,
\emph{Statistics in Medicine} 18(16), 2111-2122.
}
\author{Sven Knoth}
\seealso{\code{pois.cusum.arl} for zero-state ARL and \code{pois.cusum.crit} for threshold h computation.}
\examples{
## Table 1 from Rossi et al. (1999) -- one-sided CUSUM
La <- 500 # in-control ARL
Lr <- 7 # out-of-control ARL
m_a <- 0.52 # in-control mean of the Poisson variate
\dontrun{kh <- xcusum.crit.L0L1(La, Lr, sided="one")}
# kh <- ...: instead of deploying EK1960, one could use more accurate numbers
EK_k <- 0.60 # EK1960 results in
EK_h <- 3.80 # Table 2 on p. 372
eZR <- 2*EK_h # reproduce normal ooc mean from reference value k
m_r <- 1.58 # EK1960 Table 3 on p. 377 for m_a = 0.52
R1 <- round( eZR/sqrt(m_a) + 1, digits=2)
R2 <- round( ( eZR/2/sqrt(m_a) + 1 )^2, digits=2)
R3 <- round(( sqrt(4 + 2*eZR/sqrt(m_a)) - 1 )^2, digits=2)
RS <- round( m_r / m_a, digits=2 )
\dontrun{K_hk <- pois.cusum.crit.L0L1(m_a, La, Lr)} # 'our' 'exact' approach
K_hk <- data.frame(m=1000, km=948, mu1=1.563777, k=0.948, hm=3832, h=3.832, gamma=0.1201901)
# get k for competing means mu0 (m_a) and mu1 (m_r)
k_m01 <- function(mu0, mu1) (mu1 - mu0) / (log(mu1) - log(mu0))
# get ooc mean mu1 (m_r) for given mu0 (m_a) and reference value k
m1_km0 <- function(mu0, k) {
  zero <- function(x) k - k_m01(mu0,x)
  upper <- mu0 + .5
  while ( zero(upper) > 0 ) upper <- upper + 0.5
  mu1 <- uniroot(zero, c(mu0*1.00000001, upper), tol=1e-9)$root
  mu1
}
K_m_r <- m1_km0(m_a, K_hk$k)
RK <- round( K_m_r / m_a, digits=2 )
cat(paste(m_a, R1, R2, R3, RS, RK, "\n", sep="\t"))
}
\keyword{ts}

\name{xgrsr.crit}
\alias{xgrsr.crit}
\title{Compute alarm thresholds for Shiryaev-Roberts schemes}
\description{Computation of the alarm thresholds (alarm limits)
for Shiryaev-Roberts schemes monitoring normal mean.}
\usage{xgrsr.crit(k, L0, mu0 = 0, zr = 0, hs = NULL, sided = "one", MPT = FALSE, r = 30)}
\arguments{
\item{k}{reference value of the Shiryaev-Roberts scheme.}
\item{L0}{in-control ARL.}
\item{mu0}{in-control mean.}
\item{zr}{reflection border to enable the numerical algorithms used here.}
\item{hs}{so-called headstart (enables fast initial response). If \code{hs=NULL}, then
the classical headstart -Inf is used (corresponds to 0 for the non-log scheme).}
\item{sided}{distinguishes between one- and two-sided schemes by choosing 
\code{"one"} and\code{"two"}, respectively. Currently only one-sided schemes are
implemented.}
\item{MPT}{switch between the old implementation (\code{FALSE}) and the new one (\code{TRUE}) that considers the completed
likelihood ratio. MPT contains the initials of G. Moustakides, A. Polunchenko and A. Tartakovsky.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1}.}
}
\details{
\code{xgrsr.crit} determines the alarm threshold (alarm limit)
for given in-control ARL \code{L0}
by applying secant rule and using \code{xgrsr.arl()}.
}
\value{Returns a single value which resembles the alarm limit \code{g}.}
\references{
G. Moustakides, A. Polunchenko, A. Tartakovsky (2009),
Numerical comparison of CUSUM and Shiryaev-Roberts procedures for detecting changes in distributions,
\emph{Communications in Statistics: Theory and Methods 38}, 3225-3239.r.
}
\author{Sven Knoth}
\seealso{\code{xgrsr.arl} for zero-state ARL computation.}
\examples{
## Table 4 from Moustakides et al. (2009)
## original values are
# gamma/L0  A/exp(g)
# 50        28.02
# 100       56.04
# 500       280.19
# 1000      560.37
# 5000      2801.75
# 10000     5603.7
theta <- 1
zr <- -6
r <- 100
Lxgrsr.crit  <- Vectorize("xgrsr.crit", "L0")
L0s <- c(50, 100, 500, 1000, 5000, 10000)
gs <- Lxgrsr.crit(theta/2, L0s, zr=zr, r=r)
data.frame(L0s, gs, A=round(exp(gs), digits=2))
}
\keyword{ts}

\name{xDcusum.arl}
\alias{xDcusum.arl}
\title{Compute ARLs of CUSUM control charts under drift}
\description{Computation of the (zero-state and other) Average Run Length (ARL)
under drift for one-sided CUSUM control charts monitoring normal mean.}
\usage{xDcusum.arl(k, h, delta, hs = 0, sided = "one",
    mode = "Gan", m = NULL, q = 1, r = 30, with0 = FALSE)}
\arguments{
\item{k}{reference value of the CUSUM control chart.}
\item{h}{decision interval (alarm limit, threshold) of the CUSUM control chart.}
\item{delta}{true drift parameter.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided CUSUM control chart
by choosing \code{"one"} and \code{"two"}, respectively. Currentlly,
the two-sided scheme is not implemented.}
\item{mode}{decide whether Gan's or Knoth's approach is used. Use
\code{"Gan"} and \code{"Knoth"}, respectively.}
\item{m}{parameter used if \code{mode="Gan"}. \code{m} is design
parameter of Gan's approach. If \code{m=NULL}, then \code{m}
will increased until the resulting ARL does not change anymore.}
\item{q}{change point position. For \eqn{q=1} and
\eqn{\mu=\mu_0} and \eqn{\mu=\mu_1}, the usual
zero-state ARLs for the in-control and out-of-control case, respectively,
are calculated. For \eqn{q>1} and \eqn{\mu!=0} conditional delays, that is,
\eqn{E_q(L-q+1|L\geq)}, will be determined.
Note that mu0=0 is implicitely fixed. Deploy large \code{q} to mimic
steady-state. It works only for \code{mode="Knoth"}.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-sided) or \code{r}
(two-sided).}
\item{with0}{defines whether the first observation used for the RL
calculation follows already 1*delta or still 0*delta.
With \code{q} additional flexibility is given.}
}
\details{
Based on Gan (1991) or Knoth (2003), the ARL is calculated for
one-sided CUSUM
control charts under drift. In case of Gan's framework, the usual
ARL function with mu=m*delta is determined and recursively via
m-1, m-2, ... 1 (or 0) the drift ARL determined. The framework
of Knoth allows to calculate ARLs for varying parameters, such as
control limits and distributional parameters.
For details see the cited papers. Note that two-sided CUSUM charts
under drift are difficult to treat.
}
\value{Returns a single value which resembles the ARL.}
\references{
F. F. Gan (1992),
CUSUM control charts under linear drift,
\emph{Statistician 41}, 71-84.

F. F. Gan (1996),
Average Run Lengths for Cumulative Sum control chart under linear trend,
\emph{Applied Statistics 45}, 505-512.

S. Knoth (2003),
EWMA schemes with non-homogeneous transition kernels,
\emph{Sequential Analysis 22}, 241-255.

S. Knoth (2012),
More on Control Charting under Drift,
in: \emph{Frontiers in Statistical Quality Control 10},
H.-J. Lenz, W. Schmid and P.-T. Wilrich (Eds.),
Physica Verlag, Heidelberg, Germany, 53-68.

C. Zou, Y. Liu and Z. Wang (2009),
Comparisons of control schemes for monitoring
the means of processes subject to drifts,
\emph{Metrika 70}, 141-163.
}
\author{Sven Knoth}
\seealso{
\code{xcusum.arl} and \code{xcusum.ad} for zero-state and
steady-state ARL computation of CUSUM control charts
for the classical step change model.
}
\examples{
## Gan (1992)
## Table 1
## original values are
#   deltas  arl
#   0.0001  475
#   0.0005  261
#   0.0010  187
#   0.0020  129
#   0.0050  76.3
#   0.0100  52.0
#   0.0200  35.2
#   0.0500  21.4
#   0.1000  15.0
#   0.5000  6.95
#   1.0000  5.16
#   3.0000  3.30
\dontrun{k <- .25
h <- 8
r <- 50
DxDcusum.arl <- Vectorize(xDcusum.arl, "delta")
deltas <- c(0.0001, 0.0005, 0.001, 0.002, 0.005, 0.01, 0.02, 0.05, 0.1, 0.5, 1, 3)
arl.like.Gan   <-
  round(DxDcusum.arl(k, h, deltas, r=r, with0=TRUE), digits=2)
arl.like.Knoth <-
  round(DxDcusum.arl(k, h, deltas, r=r, mode="Knoth", with0=TRUE), digits=2)
data.frame(deltas, arl.like.Gan, arl.like.Knoth)}

## Zou et al. (2009)
## Table 1
## original values are
#  delta   arl1  arl2  arl3
#  0           ~ 1730
#  0.0005  345   412   470
#  0.001   231   275   317
#  0.005   86.6  98.6  112
#  0.01    56.9  61.8  69.3
#  0.05    22.6  21.6  22.7
#  0.1     15.4  14.7  14.2
#  0.5     6.60  5.54  5.17
#  1.0     4.63  3.80  3.45
#  2.0     3.17  2.67  2.32
#  3.0     2.79  2.04  1.96
#  4.0     2.10  1.98  1.74
\dontrun{
k1 <- 0.25
k2 <- 0.5
k3 <- 0.75
h1 <- 9.660
h2 <- 5.620
h3 <- 3.904
deltas <- c(0.0005, 0.001, 0.005, 0.01, 0.05, 0.1, 0.5, 1:4)
arl1 <- c(round(xcusum.arl(k1, h1, 0, r=r), digits=2),
          round(DxDcusum.arl(k1, h1, deltas, r=r), digits=2))
arl2 <- c(round(xcusum.arl(k2, h2, 0), digits=2),
          round(DxDcusum.arl(k2, h2, deltas, r=r), digits=2))
arl3 <- c(round(xcusum.arl(k3, h3, 0, r=r), digits=2),
          round(DxDcusum.arl(k3, h3, deltas, r=r), digits=2))
data.frame(delta=c(0, deltas), arl1, arl2, arl3)}
}
\keyword{ts}

\name{xtshewhart.ar1.arl}
\alias{xtshewhart.ar1.arl}
\title{Compute ARLs of modified Shewhart control charts for AR(1) data with Student t residuals}
\description{Computation of the (zero-state) Average Run Length (ARL)
for modified Shewhart charts deployed to the original AR(1) data where the residuals follow a Student t distribution.}
\usage{xtshewhart.ar1.arl(alpha, cS, df, delta=0, N1=50, N2=30, N3=2*N2, INFI=10, mode="tan")}
\arguments{
\item{alpha}{lag 1 correlation of the data.}
\item{cS}{critical value (alias to alarm limit) of the Shewhart control chart.}
\item{df}{degrees of freedom -- parameter of the t distribution.}
\item{delta}{potential shift in the data (in-control mean is zero.}
\item{N1}{number of quadrature nodes for solving the ARL integral equation, dimension of the resulting linear equation system is \code{N1}.}
\item{N2}{second number of quadrature nodes for combining the probability density function of the first observation following the margin distribution and the solution of the ARL integral equation.}
\item{N3}{third number of quadrature nodes for solving the left eigenfunction integral equation to determine the margin density (see Andel/Hrach, 2000),
dimension of the resulting linear equation system is \code{N3}.}
\item{INFI}{substitute of \code{Inf} -- the left eigenfunction integral equation is defined on the whole real axis; now it is reduced to \code{(-INFI,INFI)}.}
\item{mode}{Controls the type of variables substitution that might improve the numerical performance. Currently, \code{"identity"}, \code{"sin"}, \code{"sinh"}, and \code{"tan"} (default) are provided.}
}
\details{
Following the idea of Schmid (1995), \code{1-alpha} times the data turns out to be an
EWMA smoothing of the underlying AR(1) residuals. Hence, by combining the solution of
the EWMA ARL integral equation and the stationary distribution of the AR(1) data
(here Student t distribution is assumed) one gets easily the overall ARL.
}
\value{It returns a single value resembling the zero-state ARL of a modified Shewhart chart.}
\references{
J. Andel, K. Hrach (2000).
On calculation of stationary density of autoregressive processes.
\emph{Kybernetika, Institute of Information Theory and Automation AS CR 36}(3), 311-319.

H. Kramer, W. Schmid (2000).
The influence of parameter estimation on the ARL of Shewhart type charts for time series.
\emph{Statistical Papers 41}(2), 173-196.

W. Schmid (1995).
On the run length of a Shewhart chart for correlated data. 
\emph{Statistical Papers 36}(1), 111-130.
}
\author{Sven Knoth}
\seealso{
\code{xtewma.arl} for zero-state ARL computation of EWMA control charts in case of Student t distributed data.
}
\examples{
## will follow
}
\keyword{ts}

\name{xcusum.crit.L0h}
\alias{xcusum.crit.L0h}
\title{Compute the CUSUM reference value k for given in-control ARL and threshold h}
\description{Computation of the reference value k
for one-sided CUSUM control charts monitoring normal mean, if the in-control ARL L0 and
the alarm threshold h are given.}
\usage{xcusum.crit.L0h(L0, h, hs=0, sided="one", r=30, L0.eps=1e-6, k.eps=1e-8)}
\arguments{
\item{L0}{in-control ARL.}
\item{h}{alarm level of the CUSUM control chart.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one-, two-sided and Crosier's modified
two-sided CUSUM scheme choosing \code{"one"}, \code{"two"}, and \code{"Crosier"}, respectively.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-, two-sided) or \code{2r+1} (Crosier).}
\item{L0.eps}{error bound for the L0 error.}
\item{k.eps}{bound for the difference of two successive values of k.}
}
\details{
\code{xcusum.crit.L0h} determines the reference value k
for given in-control ARL \code{L0} and alarm level \code{h}
by applying secant rule and using \code{xcusum.arl()}. Note that
not for any combination of \code{L0} and \code{h} a solution exists
-- for given \code{L0} there is a maximal value for \code{h} to get a valid result \code{k}.
}
\value{Returns a single value which resembles the reference value \code{k}.}
%\references{Later...}
\author{Sven Knoth}
\seealso{\code{xcusum.arl} for zero-state ARL computation.}
\examples{
L0 <- 100
h.max <- xcusum.crit(0, L0, 0)
hs <- (300:1)/100
hs <- hs[hs < h.max]
ks <- NULL
for ( h in hs ) ks <- c(ks, xcusum.crit.L0h(L0, h))  
k.max <- qnorm( 1 - 1/L0 )
plot(hs, ks, type="l", ylim=c(0, max(k.max, ks)), xlab="h", ylab="k")
abline(h=c(0, k.max), col="red")
}
\keyword{ts}

\name{xsewma.crit}
\alias{xsewma.crit}
\title{Compute critical values of simultaneous EWMA control charts (mean and variance charts)}
\description{Computation of the critical values (similar to alarm limits)
for different types of simultaneous EWMA control charts 
(based on the sample mean and the sample variance \eqn{S^2})
monitoring normal mean and variance.}
\usage{xsewma.crit(lx, ls, L0, df, mu0=0, sigma0=1, cu=NULL, hsx=0,
hss=1, s2.on=TRUE, sided="upper", mode="fixed", Nx=30, Ns=40, qm=30)}
\arguments{
\item{lx}{smoothing parameter lambda of the two-sided mean EWMA chart.}
\item{ls}{smoothing parameter lambda of the variance EWMA chart.}
\item{L0}{in-control ARL.}
\item{mu0}{in-control mean.}
\item{sigma0}{in-control standard deviation.}
\item{cu}{for two-sided (\code{sided}=\code{"two"}) and fixed upper
control limit (\code{mode}=\code{"fixed"}) a value larger than \code{sigma0}
has to been given, for all other cases \code{cu} is ignored.}
\item{hsx}{so-called headstart (enables fast initial response) of the mean chart -- 
do not confuse with the true FIR feature considered in xewma.arl; will be updated.}
\item{hss}{headstart (enables fast initial response) of the variance chart.}
\item{df}{actual degrees of freedom, corresponds to subgroup size
(for known mean it is equal to the subgroup size,
for unknown mean it is equal to subgroup size minus one.}
\item{s2.on}{distinguishes between \eqn{S^2}{S^2} and \eqn{S}{S} chart.}
\item{sided}{distinguishes between one- and two-sided two-sided EWMA-\eqn{S^2}{S^2}
control charts by choosing \code{"upper"} (upper chart without reflection at \code{cl}
-- the actual value of \code{cl} is not used), \code{"Rupper"} (upper chart with
reflection at \code{cl}), \code{"Rlower"} (lower chart with reflection at \code{cu}),
and \code{"two"} (two-sided chart), respectively.}
\item{mode}{only deployed for \code{sided}=\code{"two"} -- with \code{"fixed"}
an upper control limit (see \code{cu}) is set and only the lower is
determined to obtain the in-control ARL \code{L0}, while with \code{"unbiased"}
a certain unbiasedness of the ARL function is guaranteed (here, both the
lower and the upper control limit are calculated).}
\item{Nx}{dimension of the approximating matrix of the mean chart.}
\item{Ns}{dimension of the approximating matrix of the variance chart.}
\item{qm}{number of quadrature nodes used for the collocation integrals.}
}
\details{
\code{xsewma.crit} determines the critical values (similar to alarm limits)
for given in-control ARL \code{L0}
by applying secant rule and using \code{xsewma.arl()}.
In case of \code{sided}=\code{"two"} and \code{mode}=\code{"unbiased"}
a two-dimensional secant rule is applied that also ensures that the
maximum of the ARL function for given standard deviation is attained
at \code{sigma0}. See Knoth (2007) for details and application.
}
\value{Returns the critical value of the two-sided mean EWMA chart and
the lower and upper controls limit \code{cl} and \code{cu} of the variance EWMA chart.}
\references{

S. Knoth (2007),
Accurate ARL calculation for EWMA control charts monitoring simultaneously
normal mean and variance,
\emph{Sequential Analysis 26}, 251-264.
}
\author{Sven Knoth}
\seealso{\code{xsewma.arl} for calculation of ARL of simultaneous EWMA charts.}
\examples{
## Knoth (2007)
## results in Table 2

# subgroup size n=5, df=n-1
df  <- 4
# lambda of mean chart
lx  <- .134
# lambda of variance chart
ls  <- .1
# in-control ARL 
L0 <- 252.3
# matrix dimensions for mean and variance part
Nx  <- 25
Ns  <- 25
# mode of variance chart
SIDED <- "upper"

crit <- xsewma.crit(lx, ls, L0, df, sided=SIDED, Nx=Nx, Ns=Ns)
crit

## output as used in Knoth (2007)
crit["cx"]/sqrt(df+1)*sqrt(lx/(2-lx))
crit["cu"] - 1
}
\keyword{ts}

\name{dphat}
\encoding{utf-8}
\alias{dphat}
\alias{pphat}
\alias{qphat}
\title{Percent defective for normal samples}
\description{Density, distribution function and quantile function
for the sample percent defective calculated on normal samples
with mean equal to \code{mu} and standard deviation equal to \code{sigma}.}
\usage{dphat(x, n, mu=0, sigma=1, type="known", LSL=-3, USL=3, nodes=30)

pphat(q, n, mu=0, sigma=1, type="known", LSL=-3, USL=3, nodes=30)

qphat(p, n, mu=0, sigma=1, type="known", LSL=-3, USL=3, nodes=30)}
\arguments{
\item{x, q}{vector of quantiles.}
\item{p}{vector of probabilities.}
\item{n}{sample size.}
\item{mu, sigma}{parameters of the underlying normal distribution.}
\item{type}{choose whether the standard deviation is given and fixed (\code{"known"}) or estimated and potententially monitored (\code{"estimated"}).}
\item{LSL,USL}{lower and upper specification limit, respectively.}
\item{nodes}{number of quadrature nodes needed for \code{type="estimated"}.}
}
\details{Bruhn-Suhr/Krumbholz (1990) derived the cumulative distribution function
of the sample percent defective calculated on normal samples to applying them for a new variables sampling plan.
These results were heavily used in Krumbholz/Zöller (1995) for Shewhart and in Knoth/Steinmetz (2013) for EWMA control charts.
For algorithmic details see, essentially, Bruhn-Suhr/Krumbholz (1990).
Two design variants are treated: The simple case, \code{type="known"}, with known normal variance and the presumably much
more relevant and considerably intricate case, \code{type="estimated"}, where both parameters of
the normal distribution are unknown. Basically, given lower and upper specification limits and the normal distribution,
one estimates the expected yield based on a normal sample of size \code{n}.
}
\value{Returns vector of pdf, cdf or qf values for the statistic phat.}
\references{
M. Bruhn-Suhr and W. Krumbholz (1990),
A new variables sampling plan for normally distributed lots with unknown standard deviation and double specification limits,
\emph{Statistical Papers} 31(1), 195-207.

W. Krumbholz and A. Zöller (1995),
\code{p}-Karten vom Shewhartschen Typ für die messende Prüfung,
\emph{Allgemeines Statistisches Archiv} 79, 347-360.

S. Knoth and S. Steinmetz (2013),
EWMA \code{p} charts  under sampling by variables,
\emph{International Journal of Production Research} 51(13), 3795-3807.
}
\author{Sven Knoth}
\seealso{
\code{phat.ewma.arl} for routines using the herewith considered phat statistic.}
\examples{
# Figures 1 (c) and (d) from Knoth/Steinmetz (2013)
n      <-  5
LSL    <- -3
USL    <-  3

par(mar=c(5, 5, 1, 1) + 0.1)

p.star <- 2*pnorm( (LSL-USL)/2 ) # for p <= p.star pdf and cdf vanish

p_ <- seq(p.star+1e-10, 0.07, 0.0001) # define support of Figure 1

# Figure 1 (c)
pp_ <- pphat(p_, n)
plot(p_, pp_, type="l", xlab="p", ylab=expression(P( hat(p) <= p )),
     xlim=c(0, 0.06), ylim=c(0,1), lwd=2)
abline(h=0:1, v=p.star, col="grey")

# Figure 1 (d)
dp_ <- dphat(p_, n)
plot(p_, dp_, type="l", xlab="p", ylab="f(p)", xlim=c(0, 0.06),
     ylim=c(0,50), lwd=2)
abline(h=0, v=p.star, col="grey")
}
\keyword{ts}

\name{xDewma.arl}
\alias{xDewma.arl}
\title{Compute ARLs of EWMA control charts under drift}
\description{Computation of the (zero-state and other)
Average Run Length (ARL) under drift
for different types of EWMA control charts monitoring normal mean.}
\usage{xDewma.arl(l, c, delta, zr = 0, hs = 0, sided = "one", limits = "fix",
    mode = "Gan", m = NULL, q = 1, r = 40, with0 = FALSE)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{c}{critical value (similar to alarm limit) of the EWMA control chart.}
\item{delta}{true drift parameter.}
\item{zr}{reflection border for the one-sided chart.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguish between one- and two-sided EWMA control chart 
by choosing \code{"one"} and \code{"two"}, respectively.}
\item{limits}{distinguishes between different control limits behavior.}
\item{mode}{decide whether Gan's or Knoth's approach is used. Use
\code{"Gan"} and \code{"Knoth"}, respectively.}
\item{m}{parameter used if \code{mode="Gan"}. \code{m} is design
parameter of Gan's approach. If \code{m=NULL}, then \code{m}
will increased until the resulting ARL does not change anymore.}
\item{q}{change point position. For \eqn{q=1} and
\eqn{\mu=\mu_0} and \eqn{\mu=\mu_1}, the usual
zero-state ARLs for the in-control and out-of-control case, respectively,
are calculated. For \eqn{q>1} and \eqn{\mu!=0} conditional delays, that is,
\eqn{E_q(L-q+1|L\geq)}, will be determined.
Note that mu0=0 is implicitely fixed. Deploy large \code{q} to mimic
steady-state. It works only for \code{mode="Knoth"}.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-sided) or \code{r}
(two-sided).}
\item{with0}{defines whether the first observation used for the RL calculation
follows already 1*delta or still 0*delta.
With \code{q} additional flexibility is given.}
}
\details{
Based on Gan (1991) or Knoth (2003), the ARL is calculated for EWMA
control charts under drift. In case of Gan's framework, the usual
ARL function with mu=m*delta is determined and recursively via
m-1, m-2, ... 1 (or 0) the drift ARL determined. The framework
of Knoth allows to calculate ARLs for varying parameters, such as
control limits and distributional parameters.
For details see the cited papers.
}
\value{Returns a single value which resembles the ARL.}
\references{
F. F. Gan (1991),
EWMA control chart under linear drift,
\emph{J. Stat. Comput. Simulation 38}, 181-200.

L. A. Aerne, C. W. Champ and S. E. Rigdon (1991),
Evaluation of control charts under linear trend,
\emph{Commun. Stat., Theory Methods 20}, 3341-3349.

S. Knoth (2003),
EWMA schemes with non-homogeneous transition kernels,
\emph{Sequential Analysis 22}, 241-255.

H. M. Fahmy and E. A. Elsayed (2006),
Detection of linear trends in process mean,
\emph{International Journal of Production Research 44}, 487-504.

S. Knoth (2012),
More on Control Charting under Drift,
in: \emph{Frontiers in Statistical Quality Control 10},
H.-J. Lenz, W. Schmid and P.-T. Wilrich (Eds.),
Physica Verlag, Heidelberg, Germany, 53-68.

C. Zou, Y. Liu and Z. Wang (2009),
Comparisons of control schemes for monitoring
the means of processes subject to drifts,
\emph{Metrika 70}, 141-163.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} and \code{xewma.ad} for zero-state and
steady-state ARL computation of EWMA control charts
for the classical step change model.
}
\examples{
\dontrun{
DxDewma.arl <- Vectorize(xDewma.arl, "delta")
## Gan (1991)
## Table 1
## original values are
#  delta   arlE1  arlE2  arlE3
#  0       500    500    500
#  0.0001  482    460    424
#  0.0010  289    231    185
#  0.0020  210    162    129
#  0.0050  126     94.6   77.9
#  0.0100   81.7   61.3   52.7
#  0.0500   27.5   21.8   21.9
#  0.1000   17.0   14.2   15.3
#  1.0000    4.09   4.28   5.25
#  3.0000    2.60   2.90   3.43
#
lambda1 <- 0.676
lambda2 <- 0.242
lambda3 <- 0.047
h1 <- 2.204/sqrt(lambda1/(2-lambda1))
h2 <- 1.111/sqrt(lambda2/(2-lambda2))
h3 <- 0.403/sqrt(lambda3/(2-lambda3))
deltas <- c(.0001, .001, .002, .005, .01, .05, .1, 1, 3)
arlE10 <- round(xewma.arl(lambda1, h1, 0, sided="two"), digits=2)
arlE1 <- c(arlE10, round(DxDewma.arl(lambda1, h1, deltas, sided="two", with0=TRUE),
                         digits=2))
arlE20 <- round(xewma.arl(lambda2, h2, 0, sided="two"), digits=2)
arlE2 <- c(arlE20, round(DxDewma.arl(lambda2, h2, deltas, sided="two", with0=TRUE),
                         digits=2))
arlE30 <- round(xewma.arl(lambda3, h3, 0, sided="two"), digits=2)
arlE3 <- c(arlE30, round(DxDewma.arl(lambda3, h3, deltas, sided="two", with0=TRUE),
                         digits=2))
data.frame(delta=c(0, deltas), arlE1, arlE2, arlE3)

## do the same with more digits for the alarm threshold
L0 <- 500
h1 <- xewma.crit(lambda1, L0, sided="two")
h2 <- xewma.crit(lambda2, L0, sided="two")
h3 <- xewma.crit(lambda3, L0, sided="two")
lambdas <- c(lambda1, lambda2, lambda3)
hs <- c(h1, h2, h3) * sqrt(lambdas/(2-lambdas))
hs
# compare with Gan's values 2.204, 1.111, 0.403
round(hs, digits=3)

arlE10 <- round(xewma.arl(lambda1, h1, 0, sided="two"), digits=2)
arlE1 <- c(arlE10, round(DxDewma.arl(lambda1, h1, deltas, sided="two", with0=TRUE),
                         digits=2))
arlE20 <- round(xewma.arl(lambda2, h2, 0, sided="two"), digits=2)
arlE2 <- c(arlE20, round(DxDewma.arl(lambda2, h2, deltas, sided="two", with0=TRUE),
                         digits=2))
arlE30 <- round(xewma.arl(lambda3, h3, 0, sided="two"), digits=2)
arlE3 <- c(arlE30, round(DxDewma.arl(lambda3, h3, deltas, sided="two", with0=TRUE),
                         digits=2))
data.frame(delta=c(0, deltas), arlE1, arlE2, arlE3)

## Aerne et al. (1991) -- two-sided EWMA
## Table I (continued)
## original numbers are
#     delta  arlE1  arlE2  arlE3
#  0.000000  465.0  465.0  465.0
#  0.005623  77.01  85.93  102.68
#  0.007499  64.61  71.78  85.74
#  0.010000  54.20  59.74  71.22
#  0.013335  45.20  49.58  58.90
#  0.017783  37.76  41.06  48.54
#  0.023714  31.54  33.95  39.87
#  0.031623  26.36  28.06  32.68
#  0.042170  22.06  23.19  26.73
#  0.056234  18.49  19.17  21.84
#  0.074989  15.53  15.87  17.83
#  0.100000  13.07  13.16  14.55
#  0.133352  11.03  10.94  11.88
#  0.177828   9.33   9.12   9.71
#  0.237137   7.91   7.62   7.95
#  0.316228   6.72   6.39   6.52
#  0.421697   5.72   5.38   5.37
#  0.562341   4.88   4.54   4.44
#  0.749894   4.18   3.84   3.68
#  1.000000   3.58   3.27   3.07
#
lambda1 <- .133
lambda2 <- .25
lambda3 <- .5
cE1 <- 2.856
cE2 <- 2.974
cE3 <- 3.049
deltas <- 10^(-(18:0)/8)
arlE10 <- round(xewma.arl(lambda1, cE1, 0, sided="two"), digits=2)
arlE1 <- c(arlE10, round(DxDewma.arl(lambda1, cE1, deltas, sided="two"), digits=2))
arlE20 <- round(xewma.arl(lambda2, cE2, 0, sided="two"), digits=2)
arlE2 <- c(arlE20, round(DxDewma.arl(lambda2, cE2, deltas, sided="two"), digits=2))
arlE30 <- round(xewma.arl(lambda3, cE3, 0, sided="two"), digits=2)
arlE3 <- c(arlE30, round(DxDewma.arl(lambda3, cE3, deltas, sided="two"), digits=2))
data.frame(delta=c(0, round(deltas, digits=6)), arlE1, arlE2, arlE3)


## Fahmy/Elsayed (2006) -- two-sided EWMA
## Table 4 (Monte Carlo results, 10^4 replicates, change point at t=51!)
## original numbers are
#   delta     arl  s.e.
#   0.00  365.749  3.598
#   0.10   12.971  0.029
#   0.25    7.738  0.015
#   0.50    5.312  0.009
#   0.75    4.279  0.007
#   1.00    3.680  0.006
#   1.25    3.271  0.006
#   1.50    2.979  0.005
#   1.75    2.782  0.004
#   2.00    2.598  0.005
#
lambda <- 0.1
cE <- 2.7
deltas <- c(.1, (1:8)/4)
arlE1 <- c(round(xewma.arl(lambda, cE, 0, sided="two"), digits=3),
           round(DxDewma.arl(lambda, cE, deltas, sided="two"), digits=3))
arlE51 <- c(round(xewma.arl(lambda, cE, 0, sided="two", q=51)[51], digits=3),
     round(DxDewma.arl(lambda, cE, deltas, sided="two", mode="Knoth", q=51),
           digits=3))
data.frame(delta=c(0, deltas), arlE1, arlE51)

## additional Monte Carlo results with 10^8 replicates
#   delta   arl.q=1   s.e.    arl.q=51  s.e.
#   0.00    368.910   0.036   361.714   0.038
#   0.10     12.986   0.000    12.781   0.000
#   0.25      7.758   0.000     7.637   0.000
#   0.50      5.318   0.000     5.235   0.000
#   0.75      4.285   0.000     4.218   0.000
#   1.00      3.688   0.000     3.628   0.000
#   1.25      3.274   0.000     3.233   0.000
#   1.50      2.993   0.000     2.942   0.000
#   1.75      2.808   0.000     2.723   0.000
#   2.00      2.616   0.000     2.554   0.000

## Zou et al. (2009) -- one-sided EWMA
## Table 1
## original values are
#  delta   arl1  arl2  arl3
#  0           ~ 1730
#  0.0005  317   377   440
#  0.001   215   253   297
#  0.005   83.6  92.6  106
#  0.01    55.6  58.8  66.1
#  0.05    22.6  21.1  22.0
#  0.1     15.5  13.9  13.8
#  0.5     6.65  5.56  5.09
#  1.0     4.67  3.83  3.43
#  2.0     3.21  2.74  2.32
#  3.0     2.86  2.06  1.98
#  4.0     2.14  2.00  1.83
l1 <- 0.03479
l2 <- 0.11125
l3 <- 0.23052
c1 <- 2.711
c2 <- 3.033
c3 <- 3.161
zr <- -6
r  <- 50
deltas <- c(0.0005, 0.001, 0.005, 0.01, 0.05, 0.1, 0.5, 1:4)
arl1 <- c(round(xewma.arl(l1, c1, 0, zr=zr, r=r), digits=2),
          round(DxDewma.arl(l1, c1, deltas, zr=zr, r=r), digits=2))
arl2 <- c(round(xewma.arl(l2, c2, 0, zr=zr), digits=2),
          round(DxDewma.arl(l2, c2, deltas, zr=zr, r=r), digits=2))
arl3 <- c(round(xewma.arl(l3, c3, 0, zr=zr, r=r), digits=2),
          round(DxDewma.arl(l3, c3, deltas, zr=zr, r=r), digits=2))
data.frame(delta=c(0, deltas), arl1, arl2, arl3)
}
}
\keyword{ts}

\name{xewma.arl}
\alias{xewma.arl}
\title{Compute ARLs of EWMA control charts}
\description{Computation of the (zero-state) Average Run Length (ARL)
for different types of EWMA control charts monitoring normal mean.}
\usage{xewma.arl(l,cE,mu,zr=0,hs=0,sided="one",limits="fix",q=1,
steady.state.mode="conditional",r=40)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{cE}{critical value (similar to alarm limit) of the EWMA control chart.}
\item{mu}{true mean.}
\item{zr}{reflection border for the one-sided chart.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided EWMA control chart 
by choosing \code{"one"} and \code{"two"}, respectively.}
\item{limits}{distinguishes between different control limits behavior.}
\item{q}{change point position. For \eqn{q=1} and
\eqn{\mu=\mu_0} and \eqn{\mu=\mu_1}, the usual
zero-state ARLs for the in-control and out-of-control case, respectively,
are calculated. For \eqn{q>1} and \eqn{\mu!=0} conditional delays, that is,
\eqn{E_q(L-q+1|L\ge q)}, will be determined. Note that mu0=0 is implicitely fixed.}
\item{steady.state.mode}{distinguishes between two steady-state modes -- conditional and cyclical
(needed for \code{q>1}).}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-sided) or \code{r}
(two-sided).}
}
\details{
In case of the EWMA chart with fixed control limits,
\code{xewma.arl} determines the Average Run Length (ARL) by numerically
solving the related ARL integral equation by means of the Nystroem method
based on Gauss-Legendre quadrature.
If \code{limits} is not \code{"fix"}, then the method presented in Knoth (2003) is utilized.
Note that for one-sided EWMA charts (\code{sided}=\code{"one"}), only
\code{"vacl"} and \code{"stat"} are deployed, while for two-sided ones
(\code{sided}=\code{"two"}) also \code{"fir"}, \code{"both"}
(combination of \code{"fir"} and \code{"vacl"}), \code{"Steiner"} and \code{"cfar"} are implemented.
For details see Knoth (2004).
}
\value{Except for the fixed limits EWMA charts it returns a single value which resembles the ARL.
For fixed limits charts, it returns a vector of length \code{q} which resembles the ARL and the
sequence of conditional expected delays for
\code{q}=1 and \code{q}>1, respectively.}
\references{
K.-H. Waldmann (1986),
Bounds for the distribution of the run length of geometric moving 
average charts, \emph{Appl. Statist. 35}, 151-158.

S. V. Crowder (1987),
A simple method for studying run-length distributions of
exponentially weighted moving average charts,
\emph{Technometrics 29}, 401-407.

J. M. Lucas and M. S. Saccucci (1990),
Exponentially weighted moving average control schemes: Properties
and enhancements, \emph{Technometrics 32}, 1-12.

S. Chandrasekaran, J. R. English and R. L. Disney (1995), 
Modeling and analysis of EWMA control schemes with variance-adjusted 
control limits, \emph{IIE Transactions 277}, 282-290.

T. R. Rhoads, D. C. Montgomery and C. M. Mastrangelo (1996),
Fast initial response scheme for exponentially weighted moving average
control chart, \emph{Quality Engineering 9}, 317-327.

S. H. Steiner (1999),
EWMA control charts with time-varying control limits and fast initial response, 
\emph{Journal of Quality Technology 31}, 75-86.

S. Knoth (2003),
EWMA schemes with non-homogeneous transition kernels,
\emph{Sequential Analysis 22}, 241-255.

S. Knoth (2004),
Fast initial response features for EWMA Control Charts,
\emph{Statistical Papers 46}, 47-64.
}
\author{Sven Knoth}
\seealso{
\code{xcusum.arl} for zero-state ARL computation of CUSUM control charts
and \code{xewma.ad} for the steady-state ARL.
}
\examples{
## Waldmann (1986), one-sided EWMA
l <- .75
round(xewma.arl(l,2*sqrt((2-l)/l),0,zr=-4*sqrt((2-l)/l)),digits=1)
l <- .5
round(xewma.arl(l,2*sqrt((2-l)/l),0,zr=-4*sqrt((2-l)/l)),digits=1)
## original values are 209.3 and 3907.5 (in Table 2).

## Waldmann (1986), two-sided EWMA with fixed control limits
l <- .75
round(xewma.arl(l,2*sqrt((2-l)/l),0,sided="two"),digits=1)
l <- .5
round(xewma.arl(l,2*sqrt((2-l)/l),0,sided="two"),digits=1)
## original values are 104.0 and 1952 (in Table 1).

## Crowder (1987), two-sided EWMA with fixed control limits
l1 <- .5
l2 <- .05
cE <- 2
mu <- (0:16)/4
arl1 <- sapply(mu,l=l1,cE=cE,sided="two",xewma.arl)
arl2 <- sapply(mu,l=l2,cE=cE,sided="two",xewma.arl)
round(cbind(mu,arl1,arl2),digits=2)

## original results are (in Table 1)
## 0.00 26.45 127.53
## 0.25 20.12  43.94
## 0.50 11.89  18.97
## 0.75  7.29  11.64
## 1.00  4.91   8.38
## 1.25  3.95*  6.56
## 1.50  2.80   5.41
## 1.75  2.29   4.62
## 2.00  1.94   4.04
## 2.25  1.70   3.61
## 2.50  1.51   3.26
## 2.75  1.37   2.99
## 3.00  1.26   2.76
## 3.25  1.18   2.56
## 3.50  1.12   2.39
## 3.75  1.08   2.26
## 4.00  1.05   2.15  (* -- in Crowder (1987) typo!?)

## Lucas/Saccucci (1990)
## two-sided EWMA

## with fixed limits
l1 <- .5
l2 <- .03
c1 <- 3.071
c2 <- 2.437
mu <- c(0,.25,.5,.75,1,1.5,2,2.5,3,3.5,4,5)
arl1 <- sapply(mu,l=l1,cE=c1,sided="two",xewma.arl)
arl2 <- sapply(mu,l=l2,cE=c2,sided="two",xewma.arl)
round(cbind(mu,arl1,arl2),digits=2)

## original results are (in Table 3)
## 0.00 500.   500.
## 0.25 255.    76.7
## 0.50  88.8   29.3
## 0.75  35.9   17.6
## 1.00  17.5   12.6
## 1.50   6.53   8.07
## 2.00   3.63   5.99
## 2.50   2.50   4.80
## 3.00   1.93   4.03
## 3.50   1.58   3.49
## 4.00   1.34   3.11
## 5.00   1.07   2.55

\dontrun{
## with fir feature
l1 <- .5
l2 <- .03
c1 <- 3.071
c2 <- 2.437
hs1 <- c1/2
hs2 <- c2/2
mu <- c(0,.5,1,2,3,5)
arl1 <- sapply(mu,l=l1,cE=c1,hs=hs1,sided="two",limits="fir",xewma.arl)
arl2 <- sapply(mu,l=l2,cE=c2,hs=hs2,sided="two",limits="fir",xewma.arl)
round(cbind(mu,arl1,arl2),digits=2)

## original results are (in Table 5)
## 0.0 487.   406.
## 0.5  86.1   18.4
## 1.0  15.9    7.36
## 2.0   2.87   3.43
## 3.0   1.45   2.34
## 5.0   1.01   1.57

## Chandrasekaran, English, Disney (1995)
## two-sided EWMA with fixed and variance adjusted limits (vacl)

l1 <- .25
l2 <- .1
c1s <- 2.9985
c1n <- 3.0042
c2s <- 2.8159
c2n <- 2.8452
mu <- c(0,.25,.5,.75,1,2)
arl1s <- sapply(mu,l=l1,cE=c1s,sided="two",xewma.arl)
arl1n <- sapply(mu,l=l1,cE=c1n,sided="two",limits="vacl",xewma.arl)
arl2s <- sapply(mu,l=l2,cE=c2s,sided="two",xewma.arl)
arl2n <- sapply(mu,l=l2,cE=c2n,sided="two",limits="vacl",xewma.arl)
round(cbind(mu,arl1s,arl1n,arl2s,arl2n),digits=2)

## original results are (in Table 2)
## 0.00 500.   500.   500.   500.
## 0.25 170.09 167.54 105.90  96.6
## 0.50  48.14  45.65  31.08  24.35
## 0.75  20.02  19.72  15.71  10.74
## 1.00  11.07   9.37  10.23   6.35
## 2.00   3.59   2.64   4.32   2.73

## The results in Chandrasekaran, English, Disney (1995) are not
## that accurate. Let us consider the more appropriate comparison

c1s <- xewma.crit(l1,500,sided="two")
c1n <- xewma.crit(l1,500,sided="two",limits="vacl")
c2s <- xewma.crit(l2,500,sided="two")
c2n <- xewma.crit(l2,500,sided="two",limits="vacl")
mu <- c(0,.25,.5,.75,1,2)
arl1s <- sapply(mu,l=l1,cE=c1s,sided="two",xewma.arl)
arl1n <- sapply(mu,l=l1,cE=c1n,sided="two",limits="vacl",xewma.arl)
arl2s <- sapply(mu,l=l2,cE=c2s,sided="two",xewma.arl)
arl2n <- sapply(mu,l=l2,cE=c2n,sided="two",limits="vacl",xewma.arl)
round(cbind(mu,arl1s,arl1n,arl2s,arl2n),digits=2)

## which demonstrate the abilities of the variance-adjusted limits
## scheme more explicitely.

## Rhoads, Montgomery, Mastrangelo (1996)
## two-sided EWMA with fixed and variance adjusted limits (vacl),
## with fir and both features

l <- .03
cE <- 2.437
mu <- c(0,.5,1,1.5,2,3,4)
sl <- sqrt(l*(2-l))
arlfix  <- sapply(mu,l=l,cE=cE,sided="two",xewma.arl)
arlvacl <- sapply(mu,l=l,cE=cE,sided="two",limits="vacl",xewma.arl)
arlfir  <- sapply(mu,l=l,cE=cE,hs=c/2,sided="two",limits="fir",xewma.arl)
arlboth <- sapply(mu,l=l,cE=cE,hs=c/2*sl,sided="two",limits="both",xewma.arl)
round(cbind(mu,arlfix,arlvacl,arlfir,arlboth),digits=1)

## original results are (in Table 1)
## 0.0 477.3* 427.9* 383.4* 286.2*
## 0.5  29.7   20.0   18.6   12.8
## 1.0  12.5    6.5    7.4    3.6
## 1.5   8.1    3.3    4.6    1.9
## 2.0   6.0    2.2    3.4    1.4
## 3.0   4.0    1.3    2.4    1.0
## 4.0   3.1    1.1    1.9    1.0
## * -- the in-control values differ sustainably from the true values!

## Steiner (1999)
## two-sided EWMA control charts with various modifications

## fixed vs. variance adjusted limits

l <- .05
cE <- 3
mu <- c(0,.25,.5,.75,1,1.5,2,2.5,3,3.5,4)
arlfix <- sapply(mu,l=l,cE=cE,sided="two",xewma.arl)
arlvacl <- sapply(mu,l=l,cE=cE,sided="two",limits="vacl",xewma.arl)
round(cbind(mu,arlfix,arlvacl),digits=1)

## original results are (in Table 2)
## 0.00 1379.0   1353.0
## 0.25  135.0    127.0
## 0.50   37.4     32.5 
## 0.75   20.0     15.6
## 1.00   13.5      9.0
## 1.50    8.3      4.5
## 2.00    6.0      2.8
## 2.50    4.8      2.0
## 3.00    4.0      1.6
## 3.50    3.4      1.3
## 4.00    3.0      1.1

## fir, both, and Steiner's modification

l <- .03
cfir <- 2.44
cboth <- 2.54
cstein <- 2.55
hsfir <- cfir/2
hsboth <- cboth/2*sqrt(l*(2-l))
mu <- c(0,.5,1,1.5,2,3,4)
arlfir <- sapply(mu,l=l,cE=cfir,hs=hsfir,sided="two",limits="fir",xewma.arl)
arlboth <- sapply(mu,l=l,cE=cboth,hs=hsboth,sided="two",limits="both",xewma.arl)
arlstein <- sapply(mu,l=l,cE=cstein,sided="two",limits="Steiner",xewma.arl)
round(cbind(mu,arlfir,arlboth,arlstein),digits=1)

## original values are (in Table 5)
## 0.0 383.0   384.0   391.0
## 0.5  18.6    14.9    13.8
## 1.0   7.4     3.9     3.6
## 1.5   4.6     2.0     1.8
## 2.0   3.4     1.4     1.3
## 3.0   2.4     1.1     1.0
## 4.0   1.9     1.0     1.0

## SAS/QC manual 1999
## two-sided EWMA control charts with fixed limits

l <- .25
cE <- 3
mu <- 1
print(xewma.arl(l,cE,mu,sided="two"),digits=11)

# original value is 11.154267016.

## Some recent examples for one-sided EWMA charts
## with varying limits and in the so-called stationary mode

# 1. varying limits = "vacl"

lambda <- .1
L0 <- 500

## Monte Carlo results (10^9 replicates)
# mu    ARL      s.e.
# 0     500.00   0.0160
# 0.5   21.637   0.0006
# 1     6.7596   0.0001
# 1.5   3.5398   0.0001
# 2     2.3038   0.0000
# 2.5   1.7004   0.0000
# 3     1.3675   0.0000

zr <- -6
r <- 50
cE <- xewma.crit(lambda, L0, zr=zr, limits="vacl", r=r)
Mxewma.arl <- Vectorize(xewma.arl, "mu")
mus <- (0:6)/2
arls <- round(Mxewma.arl(lambda, cE, mus, zr=zr, limits="vacl", r=r), digits=4)
data.frame(mus, arls)

# 2. stationary mode, i. e. limits = "stat"

## Monte Carlo results (10^9 replicates)
# mu    ARL      s.e.
# 0     500.00   0.0159
# 0.5   22.313   0.0006
# 1     7.2920   0.0001
# 1.5   3.9064   0.0001
# 2     2.5131   0.0000
# 2.5   1.7983   0.0000
# 3     1.4029   0.0000

cE <- xewma.crit(lambda, L0, zr=zr, limits="stat", r=r)
arls <- round(Mxewma.arl(lambda, cE, mus, zr=zr, limits="stat", r=r), digits=4)
data.frame(mus, arls)
}
}
\keyword{ts}

\name{pois.cusum.arl}
\alias{pois.cusum.arl}
\title{Compute ARLs of Poisson CUSUM control charts}
\description{Computation of the (zero-state) Average Run Length (ARL) at given mean \code{mu}.}
\usage{pois.cusum.arl(mu, km, hm, m, i0=0, sided="upper", rando=FALSE,
gamma=0, km2=0, hm2=0, m2=0, i02=0, gamma2=0)}
\arguments{
\item{mu}{actual mean.}
\item{km}{enumerator of rational approximation of reference value \code{k}.}
\item{hm}{enumerator of rational approximation of reference value \code{h}.}
\item{m}{denominator of rational approximation of reference value.}
\item{i0}{head start value as integer multiple of \code{1/m}; should be an element of \code{0:hm}.}
\item{sided}{distinguishes between different one- and two-sided CUSUM control chart by choosing
\code{"upper"}, \code{"lower"} and \code{"two"}, respectively.}
\item{rando}{Switch for activating randomization in order to allow continuous ARL control.}
\item{gamma}{Randomization probability. If the CUSUM statistic is equal to the threshold \code{h},
an control chart alarm is triggered with probability \code{gamma}.}
\item{km2,hm2,m2,i02,gamma2}{corresponding values of the second CUSUM chart (to building a two-sided CUSUM scheme).}
}
\details{
The monitored data follow a Poisson distribution with \code{mu}.
The ARL values of the resulting EWMA control chart are determined via Markov chain calculations.
We follow the algorithm given in Lucas (1985) expanded with some arithmetic 'tricks' (e.g., by deploying
Toeplitz matrix algebra). A paper explaining it is under preparation.
}
\value{Returns a single value which resembles the ARL.}
\references{
J. M. Lucas (1985)
Counted data CUSUM's,
\emph{Technometrics} 27(2), 129-144.

C. H. White and J. B. Keats (1996)
ARLs and Higher-Order Run-Length Moments for the Poisson CUSUM,
\emph{Journal of Quality Technology} 28(3), 363-369.

C. H. White, J. B. Keats and J. Stanley (1997)
Poisson CUSUM versus c chart for defect data,
\emph{Quality Engineering} 9(4), 673-679.

G. Rossi and L. Lampugnani and M. Marchi (1999),
An approximate CUSUM procedure for surveillance of health events,
\emph{Statistics in Medicine} 18(16), 2111-2122.

S. W. Han, K.-L. Tsui, B. Ariyajunya, and S. B. Kim (2010),
A comparison of CUSUM, EWMA, and temporal scan statistics for detection of increases in poisson rates,
\emph{Quality and Reliability Engineering International} 26(3), 279-289.

M. B. Perry and J. J. Pignatiello Jr. (2011)
Estimating the time of step change with Poisson CUSUM and EWMA control charts,
\emph{International Journal of Production Research} 49(10), 2857-2871.

}
\author{Sven Knoth}
\seealso{later.}
\examples{
## Lucas 1985, upper chart (Tables 2 and 3)
k   <- .25
h   <- 10
m   <- 4
km  <- m * k
hm  <- m * h
mu0 <- 1 * k
ARL <- pois.cusum.arl(mu0, km, hm-1, m)
# Lucas reported 438 (in Table 2, first block, row 10.0 .25 .0 ..., column 1.0
# Recall that Lucas and other trigger an alarm, if the CUSUM statistic is greater than
# or equal to the alarm threshold h
print(ARL)

ARL <- pois.cusum.arl(mu0, km, hm-1, m, i0=round((hm-1)/2))
# Lucas reported 333 (in Table 3, first block, row 10.0 .25 .0 ..., column 1.0
print(ARL)

## Lucas 1985, lower chart (Tables 4 and 5)
ARL <- pois.cusum.arl(mu0, km, hm-1, m, sided="lower")
# Lucas reported 437 (in Table 4, first block, row 10.0 .25 .0 ..., column 1.0
print(ARL)

ARL <- pois.cusum.arl(mu0, km, hm-1, m, i0=round((hm-1)/2), sided="lower")
# Lucas reported 318 (in Table 5, first block, row 10.0 .25 .0 ..., column 1.0
print(ARL)
}
\keyword{ts}

\name{euklid.ewma.arl}
\alias{euklid.ewma.arl}
\title{Compute ARLs of Poisson NCS-EWMA control charts}
\description{Computation of the (zero-state) Average Run Length (ARL) at given Poisson mean \code{mu}.}
\usage{euklid.ewma.arl(gX, gY, kL, kU, mu, y0, r0=0)}
\arguments{
\item{gX}{first and}
\item{gY}{second integer forming the rational lambda = gX/(gX+gY), lambda mimics the usual EWMA smoothing constant.}
\item{kL}{lower control limit of the NCS-EWMA control chart, integer.}
\item{kU}{upper control limit of the NCS-EWMA control chart, integer.}
\item{mu}{mean value of Poisson distribution.}
\item{y0}{headstart like value -- it is proposed to use the in-control mean.}
\item{r0}{further element of the headstart -- deviating from the default should be done only in case of full understanding of the scheme.}
}
\details{
A new idea of applying EWMA smoothing to count data based on integer divison with remainders.
It is highly recommended to read the corresponding paper (see below).
}
\value{Return single value which resemble the ARL.}
\references{
A. C. Rakitzis, P. Castagliola, P. E. Maravelakis (2015),
A new memory-type monitoring technique for count data,
Computers and Industrial Engineering 85, 235-247.
}
\author{Sven Knoth}
\seealso{later.}
\examples{
# RCM (2015), Table 12, page 243, first NCS column
gX <- 5
gY <- 24
kL <- 16
kU <- 24
mu0 <- 20
#L0 <- euklid.ewma.arl(gX, gY, kL, kU, mu0, mu0)
# should be 1219.2
 
}
\keyword{ts}

\name{xcusum.q}
\alias{xcusum.q}
\title{Compute RL quantiles of CUSUM control charts}
\description{Computation of quantiles of the Run Length (RL)for CUSUM control charts monitoring normal mean.}
\usage{xcusum.q(k, h, mu, alpha, hs=0, sided="one", r=40)}
\arguments{
\item{k}{reference value of the CUSUM control chart.}
\item{h}{decision interval (alarm limit, threshold) of the CUSUM control chart.}
\item{mu}{true mean.}
\item{alpha}{quantile level.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided CUSUM control chart by choosing \code{"one"} and \code{"two"}, respectively.}
\item{r}{number of quadrature nodes, dimension of the resulting linear equation system is equal to \code{r+1}.}
}
\details{
Instead of the popular ARL (Average Run Length) quantiles of the CUSUM
stopping time (Run Length) are determined. The algorithm is based on
Waldmann's survival function iteration procedure.
}
\value{Returns a single value which resembles the RL quantile of order \code{q}.}
\references{
K.-H. Waldmann (1986),
Bounds for the distribution of the run length of one-sided and two-sided CUSUM quality control schemes,
\emph{Technometrics 28}, 61-67.
}
\author{Sven Knoth}
\seealso{
\code{xcusum.arl} for zero-state ARL computation of CUSUM control charts.
}
\examples{
## Waldmann (1986), one-sided CUSUM, Table 2
## original values are 345, 82, 9

XCUSUM.Q <- Vectorize("xcusum.q", "alpha")
k <- .5
h <- 3
mu <- 0 # corresponds to Waldmann's -0.5
a.list <- c(.95, .5, .05)
rl.quantiles <- ceiling(XCUSUM.Q(k, h, mu, a.list))
cbind(a.list, rl.quantiles)
}
\keyword{ts}

\name{xewma.sf}
\alias{xewma.sf}
\title{Compute the survival function of EWMA run length}
\description{Computation of the survival function of the Run Length (RL) for EWMA control charts monitoring normal mean.}
\usage{xewma.sf(l, c, mu, n, zr=0, hs=0, sided="one", limits="fix", q=1, r=40)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{c}{critical value (similar to alarm limit) of the EWMA control chart.}
\item{mu}{true mean.}
\item{n}{calculate sf up to value \code{n}.}
\item{zr}{reflection border for the one-sided chart.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided EWMA control chart 
by choosing \code{"one"} and \code{"two"}, respectively.}
\item{limits}{distinguishes between different control limits behavior.}
\item{q}{change point position. For \eqn{q=1} and
\eqn{\mu=\mu_0} and \eqn{\mu=\mu_1}, the usual
zero-state situation for the in-control and out-of-control case, respectively,
are calculated. Note that mu0=0 is implicitely fixed.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-sided) or \code{r} (two-sided).}
}
\details{
The survival function P(L>n) and derived from it also the cdf P(L<=n) and the pmf P(L=n) illustrate
the distribution of the EWMA run length. For large n the geometric tail could be exploited. That is,
with reasonable large n the complete distribution is characterized.
The algorithm is based on Waldmann's survival function iteration procedure.
For varying limits and for change points after 1 the algorithm from Knoth (2004) is applied.
Note that for one-sided EWMA charts (\code{sided}=\code{"one"}), only
\code{"vacl"} and \code{"stat"} are deployed, while for two-sided ones
(\code{sided}=\code{"two"}) also \code{"fir"}, \code{"both"}
(combination of \code{"fir"} and \code{"vacl"}), and \code{"Steiner"} are implemented.
For details see Knoth (2004).
}
\value{Returns a vector which resembles the survival function up to a certain point.}
\references{
F. F. Gan (1993),
An optimal design of EWMA control charts based on the median run length,
\emph{J. Stat. Comput. Simulation 45}, 169-184.

S. Knoth (2003),
EWMA schemes with non-homogeneous transition kernels,
\emph{Sequential Analysis 22}, 241-255.

S. Knoth (2004),
Fast initial response features for EWMA Control Charts,
\emph{Statistical Papers 46}, 47-64.

K.-H. Waldmann (1986),
Bounds for the distribution of the run length of geometric moving 
average charts, \emph{Appl. Statist. 35}, 151-158.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} for zero-state ARL computation of EWMA control charts.
}
\examples{
## Gan (1993), two-sided EWMA with fixed control limits
## some values of his Table 1 -- any median RL should be 500

G.lambda <- c(.05, .1, .15, .2, .25)
G.h <- c(.441, .675, .863, 1.027, 1.177)/sqrt(G.lambda/(2-G.lambda))

for ( i in 1:length(G.lambda) ) {
  SF <- xewma.sf(G.lambda[i], G.h[i], 0, 1000)
  if (i==1) plot(1:length(SF), SF, type="l", xlab="n", ylab="P(L>n)")
  else lines(1:length(SF), SF, col=i)
}
}
\keyword{ts}

\name{xgrsr.ad}
\alias{xgrsr.ad}
\title{Compute steady-state ARLs of Shiryaev-Roberts schemes}
\description{Computation of the steady-state Average Run Length (ARL)
for Shiryaev-Roberts schemes monitoring normal mean.}
\usage{xgrsr.ad(k, g, mu1, mu0 = 0, zr = 0, sided = "one", MPT = FALSE, r = 30)}
\arguments{
\item{k}{reference value of the Shiryaev-Roberts scheme.}
\item{g}{control limit (alarm threshold) of Shiryaev-Roberts scheme.}
\item{mu1}{out-of-control mean.}
\item{mu0}{in-control mean.}
\item{zr}{reflection border to enable the numerical algorithms used here.}
\item{sided}{distinguishes between one- and two-sided schemes by choosing 
\code{"one"} and\code{"two"}, respectively. Currently only one-sided schemes are
implemented.}
\item{MPT}{switch between the old implementation (\code{FALSE}) and the new one (\code{TRUE}) that considers the completed
likelihood ratio. MPT contains the initials of G. Moustakides, A. Polunchenko and A. Tartakovsky.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1}.}
}
\details{
\code{xgrsr.ad} determines the steady-state Average Run Length (ARL) by numerically
solving the related ARL integral equation by means of the Nystroem method
based on Gauss-Legendre quadrature.
}
\value{Returns a single value which resembles the steady-state ARL.}
\references{
S. Knoth (2006),
The art of evaluating monitoring schemes --
how to measure the performance of control charts?
S. Lenz, H. & Wilrich, P. (ed.),
\emph{Frontiers in Statistical Quality Control 8}, Physica Verlag, Heidelberg, Germany, 74-99.

G. Moustakides, A. Polunchenko, A. Tartakovsky (2009),
Numerical comparison of CUSUM and Shiryaev-Roberts procedures for
detectin changes in distributions,
\emph{Communications in Statistics: Theory and Methods 38}, 3225-3239.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} and \code{xcusum-arl} for zero-state ARL computation of EWMA and CUSUM control charts,
respectively, and \code{xgrsr.arl} for the zero-state ARL.
}
\examples{
## Small study to identify appropriate reflection border to mimic unreflected schemes
k <- .5
g <- log(390)
zrs <- -(0:10)
ZRxgrsr.ad <- Vectorize(xgrsr.ad, "zr")
ads <- ZRxgrsr.ad(k, g, 0, zr=zrs)
data.frame(zrs, ads)

## Table 2 from Knoth (2006)
## original values are
#  mu   arl
#  0    689
#  0.5  30
#  1    8.9
#  1.5  5.1
#  2    3.6
#  2.5  2.8
#  3    2.4
#
k <- .5
g <- log(390)
zr <- -5 # see first example
mus <- (0:6)/2
Mxgrsr.ad <- Vectorize(xgrsr.ad, "mu1")
ads <- round(Mxgrsr.ad(k, g, mus, zr=zr), digits=1)
data.frame(mus, ads)

## Table 4 from Moustakides et al. (2009)
## original values are
# gamma  A        STADD/steady-state ARL
# 50     28.02    4.37
# 100    56.04    5.46
# 500    280.19   8.33
# 1000   560.37   9.64
# 5000   2801.75  12.79
# 10000  5603.7   14.17
Gxgrsr.ad  <- Vectorize("xgrsr.ad", "g")
As <- c(28.02, 56.04, 280.19, 560.37, 2801.75, 5603.7)
gs <- log(As)
theta <- 1
zr <- -6
ads <- round(Gxgrsr.ad(theta/2, gs, theta, zr=zr, r=100), digits=2)
data.frame(As, ads)
}
\keyword{ts}

\name{xcusum.crit.L0L1}
\alias{xcusum.crit.L0L1}
\title{Compute the CUSUM k and h for given in-control ARL L0 and out-of-control L1}
\description{Computation of the reference value k and the alarm threshold h
for one-sided CUSUM control charts monitoring normal mean, if the in-control ARL L0 and the out-of-control L1 are given.}
\usage{xcusum.crit.L0L1(L0, L1, hs=0, sided="one", r=30, L1.eps=1e-6, k.eps=1e-8)}
\arguments{
\item{L0}{in-control ARL.}
\item{L1}{out-of-control ARL.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one-, two-sided and Crosier's modified
two-sided CUSUM schemoosing \code{"one"}, \code{"two"}, and \code{"Crosier"}, respectively.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-, two-sided) or \code{2r+1} (Crosier).}
\item{L1.eps}{error bound for the L1 error.}
\item{k.eps}{bound for the difference of two successive values of k.}
}
\details{
\code{xcusum.crit.L0L1} determines the reference value k and the alarm threshold h
for given in-control ARL \code{L0} and out-of-control ARL \code{L1}
by applying secant rule and using \code{xcusum.arl()} and \code{xcusum.crit()}.
These CUSUM design rules were firstly (and quite rarely afterwards) used by Ewan and Kemp.
}
\value{Returns two values which resemble the reference value \code{k} and the threshold \code{h}.}
\references{
W. D. Ewan and K. W. Kemp (1960),
Sampling inspection of continuous processes with no autocorrelation between successive results,
\emph{Biometrika 47}, 363-380.

K. W. Kemp (1962),
The Use of Cumulative Sums for Sampling Inspection Schemes,
\emph{Journal of the Royal Statistical Sociecty C, Applied Statistics, 10}, 16-31.
}
\author{Sven Knoth}
\seealso{\code{xcusum.arl} for zero-state ARL and \code{xcusum.crit} for threshold h computation.}
\examples{
## Table 2 from Ewan/Kemp (1960) -- one-sided CUSUM
#
# A.R.L. at A.Q.L.   A.R.L. at A.Q.L.     k      h
#       1000                3           1.12   2.40
#       1000                7           0.65   4.06
#        500                3           1.04   2.26
#        500                7           0.60   3.80
#        250                3           0.94   2.11
#        250                7           0.54   3.51
#
L0.set <- c(1000, 500, 250)
L1.set <- c(3, 7)
cat("\nL0\tL1\tk\th\n")
for ( L0 in L0.set ) {
  for ( L1 in L1.set ) {
    result <- round(xcusum.crit.L0L1(L0, L1), digits=2)
    cat(paste(L0, L1, result[1], result[2], sep="\t"), "\n")
  }
}
#
# two confirmation runs
xcusum.arl(0.54, 3.51, 0) # Ewan/Kemp
xcusum.arl(result[1], result[2], 0) # here
xcusum.arl(0.54, 3.51, 2*0.54) # Ewan/Kemp
xcusum.arl(result[1], result[2], 2*result[1]) # here
#
## Table II from Kemp (1962) -- two-sided CUSUM
#
#    Lr                  k
#             La=250   La=500   La=1000
#    2.5       1.05     1.17     1.27
#    3.0       0.94     1.035    1.13
#    4.0       0.78     0.85     0.92
#    5.0       0.68     0.74     0.80
#    6.0       0.60     0.655    0.71
#    7.5       0.52     0.57     0.62
#   10.0       0.43     0.48     0.52
#
L0.set <- c(250, 500, 1000)
L1.set <- c(2.5, 3:6, 7.5, 10)
cat("\nL1\tL0=250\tL0=500\tL0=1000\n")
for ( L1 in L1.set ) {
  cat(L1)
  for ( L0 in L0.set ) {
    result <- round(xcusum.crit.L0L1(L0, L1, sided="two"), digits=2)
    cat("\t", result[1])
  }
  cat("\n")
}
}
\keyword{ts}

\name{mewma.crit}
\alias{mewma.crit}
\title{Compute alarm threshold of MEWMA control charts}
\description{Computation of the alarm threshold for multivariate exponentially weighted
moving average (MEWMA) charts monitoring multivariate normal mean.}
\usage{mewma.crit(l, L0, p, hs=0, r=20)}
\arguments{
\item{l}{smoothing parameter lambda of the MEWMA control chart.}
\item{L0}{in-control ARL.}
\item{p}{dimension of multivariate normal distribution.}
\item{hs}{so-called headstart (enables fast initial response) -- must be non-negative.}
\item{r}{number of quadrature nodes -- dimension of the resulting linear equation system.}
}
\details{
\code{mewma.crit} determines the alarm threshold of for given in-control ARL \code{L0}
by applying secant rule and using \code{mewma.arl()} with \code{ntype="gl2"}.
}
\value{Returns a single value which resembles the critical value \code{c}.}
\references{
Sven Knoth (2017),
ARL Numerics for MEWMA Charts,
\emph{Journal of Quality Technology 49(1)}, 78-89.

Steven E. Rigdon (1995),
An integral equation for the in-control average run length of a multivariate exponentially weighted moving average control chart,
\emph{J. Stat. Comput. Simulation 52(4)}, 351-365.
}
\author{Sven Knoth}
\seealso{\code{mewma.arl} for zero-state ARL computation.}
\examples{
# Rigdon (1995), p. 358, Tab. 1
p <- 4
L0 <- 500
r <- .25
h4 <- mewma.crit(r, L0, p)
h4
## original value is 16.38.

# Knoth (2017), p. 82, Tab. 2
p <- 3
L0 <- 1e3
lambda <- c(0.25, 0.2, 0.15, 0.1, 0.05)
h4 <- rep(NA, length(lambda) )
for ( i in 1:length(lambda) ) h4[i] <- mewma.crit(lambda[i], L0, p, r=20)
round(h4, digits=2)
## original values are
## 15.82 15.62 15.31 14.76 13.60
}
\keyword{ts}

\name{lns2ewma.crit}
\alias{lns2ewma.crit}
\title{Compute critical values of EWMA ln \eqn{S^2}{S^2} control charts (variance charts)}
\description{Computation of the critical values (similar to alarm limits)
for different types of EWMA control charts 
(based on the log of the sample variance \eqn{S^2}) monitoring normal variance.}
\usage{lns2ewma.crit(l,L0,df,sigma0=1,cl=NULL,cu=NULL,hs=NULL,sided="upper",mode="fixed",r=40)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{L0}{in-control ARL.}
\item{df}{actual degrees of freedom, corresponds to subsample size
(for known mean it is equal to the subsample size,
for unknown mean it is equal to subsample size minus one.}
\item{sigma0}{in-control standard deviation.}
\item{cl}{deployed for \code{sided}=\code{"upper"}, that is, upper variance control chart with
lower reflecting barrier \code{cl}.}
\item{cu}{for two-sided (\code{sided}=\code{"two"}) and fixed upper control limit (\code{mode}=\code{"fixed"}),
for all other cases \code{cu} is ignored.}
\item{hs}{so-called headstart (enables fast initial response) -- the default value (hs=NULL) corresponds to the
in-control mean of ln \eqn{S^2}{S^2}.}
\item{sided}{distinguishes between one- and two-sided two-sided EWMA-\eqn{S^2}{S^2} control charts by choosing
\code{"upper"} (upper chart with reflection at \code{cl}), 
\code{"lower"} (lower chart with reflection at \code{cu}), and \code{"two"} (two-sided chart),  respectively.} 
\item{mode}{only deployed for \code{sided}=\code{"two"} -- with \code{"fixed"} an upper control limit (see \code{cu})
is set and only the lower is
calculated to obtain the in-control ARL \code{L0}, while with \code{"unbiased"} a certain unbiasedness of the ARL
function is guaranteed (here, both the
lower and the upper control limit are calculated). With \code{"vanilla"} limits symmetric around the in-control
mean of ln \eqn{S^2}{S^2}
are determined, while for \code{"eq.tails"} the in-control ARL values of two single EWMA variance charts
(decompose the two-sided scheme into one lower and one upper scheme) are matched.}
\item{r}{dimension of the resulting linear equation system: the larger the more accurate.}
}
\details{
\code{lns2ewma.crit} determines the critical values (similar to alarm limits) for given in-control ARL \code{L0}
by applying secant rule and using \code{lns2ewma.arl()}.
In case of \code{sided}=\code{"two"} and \code{mode}=\code{"unbiased"}
a two-dimensional secant rule is applied that also ensures that the
maximum of the ARL function for given standard deviation is attained
at \code{sigma0}. See Knoth (2010) and the related example.
}
\value{Returns the lower and upper control limit \code{cl} and \code{cu}.}
\references{
C. A. Acosta-Mej\'ia and J. J. Pignatiello Jr. and B. V. Rao (1999),
A comparison of control charting procedures for monitoring process dispersion,
\emph{IIE Transactions 31}, 569-579.

S. V. Crowder and M. D. Hamilton (1992),
An EWMA for monitoring a process standard deviation,
\emph{Journal of Quality Technology 24}, 12-21.

S. Knoth (2005),
Accurate ARL computation for EWMA-\eqn{S^2}{S^2} control charts,
\emph{Statistics and Computing 15}, 341-352.

S. Knoth (2010),
Control Charting Normal Variance -- Reflections, Curiosities, and Recommendations,
in \emph{Frontiers in Statistical Quality Control 9},
H.-J. Lenz and P.-T. Wilrich (Eds.),
Physica Verlag, Heidelberg, Germany, 3-18.
}
\author{Sven Knoth}
\seealso{\code{lns2ewma.arl} for calculation of ARL of EWMA ln \eqn{S^2}{S^2} control charts.}
\examples{
## Knoth (2005)
## compare with 1.05521 mentioned on page 350 third line from below
L0 <- 200
lambda <- .05
df <- 4
limits <- lns2ewma.crit(lambda, L0, df, cl=0, hs=0)
limits["cu"]/sqrt( lambda/(2-lambda)*(2/df+2/df^2+4/3/df^3-16/15/df^5) )
}
\keyword{ts}

\name{xtcusum.arl}
\alias{xtcusum.arl}
\title{Compute ARLs of CUSUM control charts}
\description{Computation of the (zero-state) Average Run Length (ARL)
for different types of CUSUM control charts monitoring normal mean.}
\usage{xtcusum.arl(k, h, df, mu, hs = 0, sided="one", mode="tan", r=30)}
\arguments{
\item{k}{reference value of the CUSUM control chart.}
\item{h}{decision interval (alarm limit, threshold) of the CUSUM control chart.}
\item{df}{degrees of freedom -- parameter of the t distribution.}
\item{mu}{true mean.}
\item{hs}{so-called headstart (give fast initial response).}
\item{sided}{distinguish between one- and  two-sided CUSUM schemes by choosing \code{"one"} and \code{"two"}, respectively.}
\item{r}{number of quadrature nodes, dimension of the resulting linear equation system is equal to \code{r+1}.}
\item{mode}{Controls the type of variables substitution that might improve the numerical performance. Currently, \code{"identity"}, \code{"sin"}, \code{"sinh"}, and \code{"tan"} (default) are provided.}
}
\details{
\code{xtcusum.arl} determines the Average Run Length (ARL) by numerically
solving the related ARL integral equation by means of the Nystroem method
based on Gauss-Legendre quadrature.
}
\value{Returns a single value which resembles the ARL.}
\references{
A. L. Goel, S. M. Wu (1971),
Determination of A.R.L. and a contour nomogram for CUSUM charts to
control normal mean, \emph{Technometrics 13}, 221-230.

D. Brook, D. A. Evans (1972),
An approach to the probability distribution of cusum run length,
\emph{Biometrika 59}, 539-548.

J. M. Lucas, R. B. Crosier (1982),
Fast initial response for cusum quality-control schemes: 
Give your cusum a headstart, \emph{Technometrics 24}, 199-205.

L. C. Vance (1986), 
Average run lengths of cumulative sum control charts for controlling 
normal means, \emph{Journal of Quality Technology 18}, 189-193.

K.-H. Waldmann (1986),
Bounds for the distribution of the run length of one-sided and
two-sided CUSUM quality control schemes,
\emph{Technometrics 28}, 61-67.

R. B. Crosier (1986),
A new two-sided cumulative quality control scheme,
\emph{Technometrics 28}, 187-194.
}
\author{Sven Knoth}
\seealso{
\code{xtewma.arl} for zero-state ARL computation of EWMA control charts and \code{xtcusum.arl} for the zero-state ARL of CUSUM for normal data.
}
\examples{
## will follow
}
\keyword{ts}

\name{p.ewma.arl}
\alias{p.ewma.arl}
\title{Compute ARLs of binomial EWMA p control charts}
\description{Computation of the (zero-state) Average Run Length (ARL) at given rate \code{p}.}
\usage{p.ewma.arl(lambda, ucl, n, p, z0, sided="upper", lcl=NULL, d.res=1,
r.mode="ieee.round", i.mode="integer")}
\arguments{
\item{lambda}{smoothing parameter of the EWMA p control chart.}
\item{ucl}{upper control limit of the EWMA p control chart.}
\item{n}{subgroup size.}
\item{p}{(failure/success) rate.}
\item{z0}{so-called headstart (give fast initial response).}
\item{sided}{distinguishes between one- and two-sided EWMA control chart by choosing \code{"upper"}, \code{"lower"}, and \code{"two"}, respectively.}
\item{lcl}{lower control limit of the EWMA p control chart; needed for two-sided design.}
\item{d.res}{resolution (see details).}
\item{r.mode}{round mode -- allowed modes are \code{"gan.floor"}, \code{"floor"}, \code{"ceil"},
\code{"ieee.round"}, \code{"round"}, \code{"mix"}.}
\item{i.mode}{type of interval center -- \code{"integer"} or \code{"half"} integer.}
}
\details{
The monitored data follow a binomial distribution with size \code{n} and failure/success probability \code{p}.
The ARL values of the resulting EWMA control chart are determined by Markov chain approximation.
Here, the original EWMA values are approximated by
multiples of one over \code{d.res}. Different ways of rounding (see \code{r.mode}) to the next multiple are implemented.
Besides Gan's paper nothing is published about the numerical subtleties.
}
\value{Return single value which resemble the ARL.}
\references{
F. F. Gan (1990),
Monitoring observations generated from a binomial distribution using modified
exponentially weighted moving average control chart,
\emph{J. Stat. Comput. Simulation} 37, 45-60.

S. Knoth and S. Steinmetz (2013),
EWMA \code{p} charts  under sampling by variables,
\emph{International Journal of Production Research} 51, 3795-3807.
}
\author{Sven Knoth}
\seealso{later.}
\examples{
## Gan (1990)

# Table 1

n <- 150
p0 <- .1
z0 <- n*p0

lambda <- c(1, .51, .165)
hu <- c(27, 22, 18)

p.value <- .1 + (0:20)/200

p.EWMA.arl <- Vectorize(p.ewma.arl, "p")

arl1.value <- round(p.EWMA.arl(lambda[1], hu[1], n, p.value, z0, r.mode="round"), digits=2)
arl2.value <- round(p.EWMA.arl(lambda[2], hu[2], n, p.value, z0, r.mode="round"), digits=2)
arl3.value <- round(p.EWMA.arl(lambda[3], hu[3], n, p.value, z0, r.mode="round"), digits=2)

arls <- matrix(c(arl1.value, arl2.value, arl3.value), ncol=length(lambda))
rownames(arls) <- p.value
colnames(arls) <- paste("lambda =", lambda)
arls

## Knoth/Steinmetz (2013)

n <- 5
p0 <- 0.02
z0 <- n*p0
lambda <- 0.3
ucl <- 0.649169922 ## in-control ARL 370.4 (determined with d.res = 2^14 = 16384)

res.list <- 2^(1:11)
arl.list <- NULL
for ( res in res.list ) {
  arl <- p.ewma.arl(lambda, ucl, n, p0, z0, d.res=res)
  arl.list <- c(arl.list, arl)
}
cbind(res.list, arl.list)
}
\keyword{ts}

\name{xcusum.arl}
\alias{xcusum.arl}
\title{Compute ARLs of CUSUM control charts}
\description{Computation of the (zero-state) Average Run Length (ARL)
for different types of CUSUM control charts monitoring normal mean.}
\usage{xcusum.arl(k, h, mu, hs = 0, sided = "one", method = "igl", q = 1, r = 30)}
\arguments{
\item{k}{reference value of the CUSUM control chart.}
\item{h}{decision interval (alarm limit, threshold) of the CUSUM control chart.}
\item{mu}{true mean.}
\item{hs}{so-called headstart (give fast initial response).}
\item{sided}{distinguish between one-, two-sided and Crosier's modified
two-sided CUSUM scheme by choosing \code{"one"}, \code{"two"},
and \code{"Crosier"}, respectively.}
\item{method}{deploy the integral equation (\code{"igl"}) or Markov chain approximation
(\code{"mc"}) method to calculate the ARL (currently only for two-sided CUSUM implemented).}
\item{q}{change point position. For \eqn{q=1} and
\eqn{\mu=\mu_0} and \eqn{\mu=\mu_1}, the usual
zero-state ARLs for the in-control and out-of-control case, respectively,
are calculated. For \eqn{q>1} and \eqn{\mu!=0} conditional delays, that is,
\eqn{E_q(L-q+1|L\ge q)}, will be determined.
Note that mu0=0 is implicitely fixed.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-, two-sided) or \code{2r+1}
(Crosier).}
}
\details{
\code{xcusum.arl} determines the Average Run Length (ARL) by numerically
solving the related ARL integral equation by means of the Nystroem method
based on Gauss-Legendre quadrature.
}
\value{Returns a vector of length \code{q} which resembles the ARL and the sequence of conditional expected delays for
\code{q}=1 and \code{q}>1, respectively.}
\references{
A. L. Goel, S. M. Wu (1971),
Determination of A.R.L. and a contour nomogram for CUSUM charts to
control normal mean, \emph{Technometrics 13}, 221-230.

D. Brook, D. A. Evans (1972),
An approach to the probability distribution of cusum run length,
\emph{Biometrika 59}, 539-548.

J. M. Lucas, R. B. Crosier (1982),
Fast initial response for cusum quality-control schemes: 
Give your cusum a headstart, \emph{Technometrics 24}, 199-205.

L. C. Vance (1986), 
Average run lengths of cumulative sum control charts for controlling 
normal means, \emph{Journal of Quality Technology 18}, 189-193.

K.-H. Waldmann (1986),
Bounds for the distribution of the run length of one-sided and
two-sided CUSUM quality control schemes,
\emph{Technometrics 28}, 61-67.

R. B. Crosier (1986),
A new two-sided cumulative quality control scheme,
\emph{Technometrics 28}, 187-194.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} for zero-state ARL computation of EWMA control charts
and \code{xcusum.ad} for the steady-state ARL.
}
\examples{
## Brook/Evans (1972), one-sided CUSUM
## Their results are based on the less accurate Markov chain approach.

k <- .5
h <- 3
round(c( xcusum.arl(k,h,0), xcusum.arl(k,h,1.5) ),digits=2)

## results in the original paper are L0 = 117.59, L1 = 3.75 (in Subsection 4.3).

## Lucas, Crosier (1982)
## (one- and) two-sided CUSUM with possible headstarts

k <- .5
h <- 4
mu <- c(0,.25,.5,.75,1,1.5,2,2.5,3,4,5)
arl1 <- sapply(mu,k=k,h=h,sided="two",xcusum.arl)
arl2 <- sapply(mu,k=k,h=h,hs=h/2,sided="two",xcusum.arl)
round(cbind(mu,arl1,arl2),digits=2)

## results in the original paper are (in Table 1)
## 0.00 168.   149.
## 0.25  74.2   62.7
## 0.50  26.6   20.1
## 0.75  13.3    8.97
## 1.00   8.38   5.29
## 1.50   4.75   2.86
## 2.00   3.34   2.01
## 2.50   2.62   1.59
## 3.00   2.19   1.32
## 4.00   1.71   1.07
## 5.00   1.31   1.01

## Vance (1986), one-sided CUSUM
## The first paper on using Nystroem method and Gauss-Legendre quadrature
## for solving the ARL integral equation (see as well Goel/Wu, 1971)

k <- 0
h <- 10
mu <- c(-.25,-.125,0,.125,.25,.5,.75,1)
round(cbind(mu,sapply(mu,k=k,h=h,xcusum.arl)),digits=2)

## results in the original paper are (in Table 1 incl. Goel/Wu (1971) results)
##  -0.25  2071.51
##  -0.125  400.28
##   0.0    124.66
##   0.125   59.30
##   0.25    36.71
##   0.50    20.37
##   0.75    14.06
##   1.00    10.75

## Waldmann (1986),
## one- and two-sided CUSUM

## one-sided case

k <- .5
h <- 3
mu <- c(-.5,0,.5)
round(sapply(mu,k=k,h=h,xcusum.arl),digits=2)

## results in the original paper are 1963, 117.4, and 17.35, resp.
## (in Tables 3, 1, and 5, resp.).

## two-sided case

k <- .6
h <- 3
round(xcusum.arl(k,h,-.2,sided="two"),digits=1)  # fits to Waldmann's setup

## result in the original paper is 65.4 (in Table 6).

## Crosier (1986), Crosier's modified two-sided CUSUM
## He introduced the modification and evaluated it by means of
## Markov chain approximation

k <- .5
h <- 3.73
mu <- c(0,.25,.5,.75,1,1.5,2,2.5,3,4,5)
round(cbind(mu,sapply(mu,k=k,h=h,sided="Crosier",xcusum.arl)),digits=2)

## results in the original paper are (in Table 3)
## 0.00 168.
## 0.25  70.7
## 0.50  25.1
## 0.75  12.5
## 1.00   7.92
## 1.50   4.49
## 2.00   3.17
## 2.50   2.49
## 3.00   2.09
## 4.00   1.60
## 5.00   1.22

## SAS/QC manual 1999
## one- and two-sided CUSUM schemes

## one-sided

k <- .25
h <- 8
mu <- 2.5
print(xcusum.arl(k,h,mu),digits=12)
print(xcusum.arl(k,h,mu,hs=.1),digits=12)

## original results are 4.1500836225 and 4.1061588131.

## two-sided

print(xcusum.arl(k,h,mu,sided="two"),digits=12)

## original result is 4.1500826715.
}
\keyword{ts}

\name{lns2ewma.arl}
\alias{lns2ewma.arl}
\title{Compute ARLs of EWMA ln \eqn{S^2}{S^2} control charts (variance charts)}
\description{Computation of the (zero-state) Average Run Length (ARL)
for different types of EWMA control charts 
(based on the log of the sample variance \eqn{S^2}) monitoring normal variance.}
\usage{lns2ewma.arl(l,cl,cu,sigma,df,hs=NULL,sided="upper",r=40)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{cl}{lower control limit of the EWMA control chart.}
\item{cu}{upper control limit of the EWMA control chart.}
\item{sigma}{true standard deviation.}
\item{df}{actual degrees of freedom, corresponds to subsample size (for known mean it is equal to the subsample size,
for unknown mean it is equal to subsample size minus one.}
\item{hs}{so-called headstart (enables fast initial response) -- the default value (hs=NULL) corresponds to the in-control
mean of ln \eqn{S^2}{S^2}.}
\item{sided}{distinguishes between one- and two-sided two-sided EWMA-\eqn{S^2}{S^2} control charts 
by choosing \code{"upper"} (upper chart with reflection at \code{cl}), \code{"lower"} (lower chart with reflection at \code{cu}),
and \code{"two"} (two-sided chart), respectively.} 
\item{r}{dimension of the resulting linear equation system: the larger the better.}
}
\details{
\code{lns2ewma.arl} determines the Average Run Length (ARL) by numerically
solving the related ARL integral equation by means of the Nystroem method
based on Gauss-Legendre quadrature.}
\value{Returns a single value which resembles the ARL.}
\references{
S. V. Crowder and M. D. Hamilton (1992),
An EWMA for monitoring a process standard deviation,
\emph{Journal of Quality Technology 24}, 12-21.

S. Knoth (2005),
Accurate ARL computation for EWMA-\eqn{S^2}{S^2} control charts,
\emph{Statistics and Computing 15}, 341-352.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} for zero-state ARL computation of EWMA control charts for monitoring normal mean.
}
\examples{
lns2ewma.ARL <- Vectorize("lns2ewma.arl", "sigma")

## Crowder/Hamilton (1992)
## moments of ln S^2
E_log_gamma <- function(df) log(2/df) + digamma(df/2)
V_log_gamma <- function(df) trigamma(df/2)
E_log_gamma_approx <- function(df) -1/df - 1/3/df^2 + 2/15/df^4
V_log_gamma_approx <- function(df) 2/df + 2/df^2 + 4/3/df^3 - 16/15/df^5

## results from Table 3 ( upper chart with reflection at 0 = log(sigma0=1) )
## original entries are (lambda = 0.05, K = 1.06, df=n-1=4)
# sigma   ARL
# 1       200
# 1.1      43
# 1.2      18
# 1.3      11
# 1.4       7.6
# 1.5       6.0
# 2         3.2

df <- 4
lambda <- .05
K <- 1.06
cu <- K * sqrt( lambda/(2-lambda) * V_log_gamma_approx(df) )

sigmas <- c(1 + (0:5)/10, 2)
arls <- round(lns2ewma.ARL(lambda, 0, cu, sigmas, df, hs=0, sided="upper"), digits=1)
data.frame(sigmas, arls)

## Knoth (2005)
## compare with Table 3 (p. 351)
lambda <- .05
df <- 4
K <- 1.05521
cu <- 1.05521 * sqrt( lambda/(2-lambda) * V_log_gamma_approx(df) )

## upper chart with reflection at sigma0=1 in Table 4
## original entries are
# sigma   ARL_0    ARL_-.267
# 1       200.0    200.0
# 1.1      43.04    41.55
# 1.2      18.10    19.92
# 1.3      10.75    13.11
# 1.4       7.63     9.93
# 1.5       5.97     8.11
# 2         3.17     4.67

M <- -0.267
cuM <- lns2ewma.crit(lambda, 200, df, cl=M, hs=M, r=60)[2]
arls1 <- round(lns2ewma.ARL(lambda, 0, cu, sigmas, df, hs=0, sided="upper"), digits=2)
arls2 <- round(lns2ewma.ARL(lambda, M, cuM, sigmas, df, hs=M, sided="upper", r=60), digits=2)
data.frame(sigmas, arls1, arls2)
}
\keyword{ts}

\name{xtewma.ad}
\alias{xtewma.ad}
\title{Compute steady-state ARLs of EWMA control charts, t distributed data}
\description{Computation of the steady-state Average Run Length (ARL)
for different types of EWMA control charts monitoring the mean of t distributed data.}
\usage{xtewma.ad(l, c, df, mu1, mu0=0, zr=0, z0=0, sided="one", limits="fix",
steady.state.mode="conditional", mode="tan", r=40)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{c}{critical value (similar to alarm limit) of the EWMA control chart.}
\item{df}{degrees of freedom -- parameter of the t distribution.}
\item{mu1}{in-control mean.}
\item{mu0}{out-of-control mean.}
\item{zr}{reflection border for the one-sided chart.}
\item{z0}{restarting value of the EWMA sequence in case of a false alarm in
\code{steady.state.mode="cyclical"}.}
\item{sided}{distinguishes between one- and two-sided two-sided EWMA control
chart by choosing \code{"one"} and \code{"two"}, respectively.}
\item{limits}{distinguishes between different control limits behavior.}
\item{steady.state.mode}{distinguishes between two steady-state modes -- conditional and cyclical.}
\item{mode}{Controls the type of variables substitution that might improve the numerical performance. Currently,
\code{"identity"}, \code{"sin"}, \code{"sinh"}, and \code{"tan"} (default) are provided.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-sided) or \code{r}
(two-sided).}
}
\details{
\code{xtewma.ad} determines the steady-state Average Run Length (ARL)
by numerically solving the related ARL integral equation by means
of the Nystroem method based on Gauss-Legendre quadrature
and using the power method for deriving the largest in magnitude
eigenvalue and the related left eigenfunction.
}
\value{Returns a single value which resembles the steady-state ARL.}
\references{
R. B. Crosier (1986),
A new two-sided cumulative quality control scheme,
\emph{Technometrics 28}, 187-194.

S. V. Crowder (1987),
A simple method for studying run-length distributions of exponentially weighted
moving average charts,
\emph{Technometrics 29}, 401-407.

J. M. Lucas and M. S. Saccucci (1990),
Exponentially weighted moving average control schemes: Properties and enhancements,
\emph{Technometrics 32}, 1-12.
}
\author{Sven Knoth}
\seealso{
\code{xtewma.arl} for zero-state ARL computation and
\code{xewma.ad} for the steady-state ARL for normal data.}
\examples{
## will follow
}
\keyword{ts}

\name{sewma.sf.prerun}
\alias{sewma.sf.prerun}
\title{Compute the survival function of EWMA run length}
\description{Computation of the survival function of the Run Length (RL)
for EWMA control charts monitoring normal variance.}
\usage{sewma.sf.prerun(n, l, cl, cu, sigma, df1, df2, hs=1, sided="upper",
qm=30, qm.sigma=30, truncate=1e-10, tail_approx=TRUE)}
\arguments{
\item{n}{calculate sf up to value \code{n}.}
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{cl}{lower control limit of the EWMA control chart.}
\item{cu}{upper control limit of the EWMA control chart.}
\item{sigma}{true standard deviation.}
\item{df1}{actual degrees of freedom, corresponds to subgroup size
(for known mean it is equal to the subgroup size,
for unknown mean it is equal to subgroup size minus one.}
\item{df2}{degrees of freedom of the pre-run variance estimator.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided two-sided
EWMA-\eqn{S^2}{S^2} control charts 
by choosing \code{"upper"} (upper chart without reflection at \code{cl} --
the actual value of \code{cl} is not used),
\code{"Rupper"} (upper chart with reflection at \code{cl}), \code{"Rlower"}
(lower chart with reflection at \code{cu}),
and \code{"two"} (two-sided chart), respectively.} 
\item{qm}{number of quadrature nodes for calculating the collocation
definite integrals.}
\item{qm.sigma}{number of quadrature nodes for convoluting the
standard deviation uncertainty.}
\item{truncate}{size of truncated tail.}
\item{tail_approx}{Controls whether the geometric tail approximation
is used (is faster) or not.}
}
\details{
The survival function P(L>n) and derived from it also the cdf P(L<=n) and
the pmf P(L=n) illustrate
the distribution of the EWMA run length. For large n the geometric tail
could be exploited. That is,
with reasonable large n the complete distribution is characterized.
The algorithm is based on Waldmann's survival function iteration
procedure and on results in Knoth (2007)...
}
\value{Returns a vector which resembles the survival function up to a certain point.}
\references{
S. Knoth (2007),
Accurate ARL calculation for EWMA control charts monitoring simultaneously normal mean and variance,
\emph{Sequential Analysis 26}, 251-264.

K.-H. Waldmann (1986),
Bounds for the distribution of the run length of geometric moving 
average charts, \emph{Appl. Statist. 35}, 151-158.
}
\author{Sven Knoth}
\seealso{
\code{sewma.sf} for the RL survival function of EWMA control charts w/o pre-run uncertainty.
}
\examples{
## will follow
}
\keyword{ts}

\name{pois.ewma.arl}
\alias{pois.ewma.arl}
\title{Compute ARLs of Poisson EWMA control charts}
\description{Computation of the (zero-state) Average Run Length (ARL) at given mean \code{mu}.}
\usage{pois.ewma.arl(lambda, AL, AU, mu0, z0, mu, sided="two", rando=FALSE, gL=0, gU=0,
mcdesign="transfer", N=101, OLD=FALSE)}
\arguments{
\item{lambda}{smoothing parameter of the EWMA p control chart.}
\item{AL, AU}{factors to build the lower and upper control limit, respectively, of the Poisson EWMA control chart.}
\item{mu0}{in-control mean.}
\item{z0}{so-called headstart (give fast initial response).}
\item{mu}{actual mean.}
\item{sided}{distinguishes between one- and two-sided EWMA control chart by choosing
\code{"upper"}, \code{"lower"}, and \code{"two"}, and \code{"zwei"}, respectively.}
\item{rando}{Switch between the standard limit treatment, \code{FALSE},
and an additional randomisation (to allow `perfect' ARL calibration) by setting \code{TRUE}.
If randomisation is used, then set the corresponding probailities, \code{gL} and \code{gU}, appropriately.}
\item{gL, gU}{If the EWMA statistic is at the limit (approximately), then an alarm is triggered with probability
\code{gL} and \code{gU} for the lower and upper limit, respectively.}
\item{mcdesign}{choose either \code{"classic"} which follows Borror, Champ and Rigdon (1998), or the more
sophisticated \code{"transfer"} which improves the accuracy heavily.}
\item{N}{number of states of the approximating Markov chain; is equal to the dimension of the
resulting linear equation system.}
\item{OLD}{Switch between the original implemenation, used in Morais and Knoth (2020), and a slightly modified new one (more accurate).}
}
\details{
The monitored data follow a Poisson distribution with \code{mu}.
The ARL values of the resulting EWMA control chart are determined by Markov chain approximation.
We follow the algorithm given in Borror, Champ and Rigdon (1998).
However, by setting \code{mcdesign="transfer"} (now the default) from Morais and Knoth (2020),
the accuracy is considerably improved.
}
\value{Return single value which resembles the ARL.}
\references{
C. M. Borror, C. W. Champ and S. E. Rigdon (1998)
Poisson EWMA control charts,
\emph{Journal of Quality Technonlogy} 30(4), 352-361.

M. C. Morais and S. Knoth (2020)
Improving the ARL profile and the accuracy of its calculation for Poisson EWMA charts,
\emph{Quality and Reliability Engineering International} 36(3), 876-889.
}
\author{Sven Knoth}
\seealso{later.}
\examples{
## Borror, Champ and Rigdon (1998), Table 2, PEWMA column
mu0 <- 20
lambda <- 0.27
A <- 3.319
mu1  <- c(2*(3:15), 35)
ARL1 <- rep(NA, length(mu1))
for ( i in 1:length(mu1) )
  ARL1[i] <- pois.ewma.arl(lambda, A, A, mu0, mu0, mu1[i], mcdesign="classic")
print(cbind(mu1, round(ARL1, digits=1)))

## the same numbers with improved accuracy
ARL2 <- rep(NA, length(mu1))
for ( i in 1:length(mu1) )
  ARL2[i] <- pois.ewma.arl(lambda, A, A, mu0, mu0, mu1[i], mcdesign="transfer")
print(cbind(mu1, round(ARL2, digits=1)))

## Morais and Knoth (2020), Table 2, lambda = 0.27 column
lambda <- 0.27
AL <- 3.0870
AU <- 3.4870
gL <- 0.001029
gU <- 0.000765
mu0 <- 20
mu1  <- c(16, 18, 19.99, mu0, 20.01, 22, 24)
ARL3 <- rep(NA, length(mu1))
for ( i in 1:length(mu1) )
  ARL3[i] <- pois.ewma.arl(lambda,AL,AU,mu0,mu0,mu1[i],rando=TRUE,gL=gL,gU=gU, N=101, OLD=TRUE)
print(cbind(mu1, round(ARL3, digits=1)))
}
\keyword{ts}

\name{sewma.q.prerun}
\alias{sewma.q.prerun}
\alias{sewma.q.crit.prerun}
\title{Compute RL quantiles of EWMA (variance charts) control charts under pre-run uncertainty}
\description{Computation of quantiles of the Run Length (RL) for EWMA control charts monitoring
normal variance.}
\usage{sewma.q.prerun(l,cl,cu,sigma,df1,df2,alpha,hs=1,sided="upper",
r=40,qm=30,qm.sigma=30,truncate=1e-10)

sewma.q.crit.prerun(l,L0,alpha,df1,df2,sigma0=1,cl=NULL,cu=NULL,hs=1,
sided="upper",mode="fixed",r=40, qm=30,qm.sigma=30,truncate=1e-10,
tail_approx=TRUE,c.error=1e-10,a.error=1e-9)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{cl}{deployed for \code{sided}=\code{"Rupper"}, that is, upper variance control chart
with lower reflecting barrier \code{cl}.}
\item{cu}{for two-sided (\code{sided}=\code{"two"}) and fixed upper control limit
(\code{mode}=\code{"fixed"}) a value larger than \code{sigma0}
has to been given, for all other cases \code{cu} is ignored.}
\item{sigma,sigma0}{true and in-control standard deviation, respectively.}
\item{L0}{in-control quantile value.}
\item{alpha}{quantile level.}
\item{df1}{actual degrees of freedom, corresponds to subgroup size (for known mean it is
equal to the subgroup size, for unknown mean it is equal to subgroup size minus one.}
\item{df2}{degrees of freedom of the pre-run variance estimator.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided two-sided EWMA-\eqn{S^2}{S^2}
control charts by choosing \code{"upper"} (upper chart
without reflection at \code{cl} -- the actual value of \code{cl} is not used),
\code{"Rupper"} (upper chart with reflection at \code{cl}),
\code{"Rlower"} (lower chart with reflection at \code{cu}),
and \code{"two"} (two-sided chart), respectively.}
\item{mode}{only deployed for \code{sided}=\code{"two"} -- with \code{"fixed"}
an upper control limit (see \code{cu}) is set and only the lower is
calculated to obtain the in-control ARL \code{L0}, while with \code{"unbiased"} a certain unbiasedness
of the ARL function is guaranteed (here, both the
lower and the upper control limit are calculated).}
\item{r}{dimension of the resulting linear equation system (highest order
of the collocation polynomials).}
\item{qm}{number of quadrature nodes for calculating the collocation definite integrals.}
\item{qm.sigma}{number of quadrature nodes for convoluting the standard deviation uncertainty.}
\item{truncate}{size of truncated tail.}
\item{tail_approx}{controls whether the geometric tail approximation is used (is faster) or not.}
\item{c.error}{error bound for two succeeding values of the critical value
during applying the secant rule.}
\item{a.error}{error bound for the quantile level \code{alpha} during applying the secant rule.}
}
\details{
Instead of the popular ARL (Average Run Length) quantiles of the EWMA
stopping time (Run Length) are determined. The algorithm is based on
Waldmann's survival function iteration procedure.
Thereby the ideas presented in Knoth (2007) are used.
\code{sewma.q.crit.prerun} determines the critical values (similar to alarm limits)
for given in-control RL quantile \code{L0} at level \code{alpha} by applying secant
rule and using \code{sewma.sf()}.
In case of \code{sided}=\code{"two"} and \code{mode}=\code{"unbiased"}
a two-dimensional secant rule is applied that also ensures that the
minimum of the cdf for given standard deviation is attained at \code{sigma0}.
}
\value{Returns a single value which resembles the RL quantile of order \code{alpha} and
the lower and upper control limit \code{cl} and \code{cu}, respectively.}
\references{
S. Knoth (2007),
Accurate ARL calculation for EWMA control charts monitoring simultaneously normal mean and variance,
\emph{Sequential Analysis 26}, 251-264.

K.-H. Waldmann (1986),
Bounds for the distribution of the run length of geometric moving 
average charts, \emph{Appl. Statist. 35}, 151-158.
}
\author{Sven Knoth}
\seealso{\code{sewma.q}  and \code{sewma.q.crit} for the version w/o pre-run uncertainty.}
\examples{
## will follow
}
\keyword{ts}

\name{xewma.arl.prerun}
\alias{xewma.arl.prerun}
\alias{xewma.crit.prerun}
\title{Compute ARLs of EWMA control charts in case of estimated parameters}
\description{Computation of the (zero-state) Average Run Length (ARL)
for different types of EWMA control charts monitoring normal mean
if the in-control mean, standard deviation, or both are estimated by a pre run.}
\usage{xewma.arl.prerun(l, c, mu, zr=0, hs=0, sided="two", limits="fix", q=1,
size=100, df=NULL, estimated="mu", qm.mu=30, qm.sigma=30, truncate=1e-10)

xewma.crit.prerun(l, L0, mu, zr=0, hs=0, sided="two", limits="fix", size=100,
df=NULL, estimated="mu", qm.mu=30, qm.sigma=30, truncate=1e-10,
c.error=1e-12, L.error=1e-9, OUTPUT=FALSE)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{c}{critical value (similar to alarm limit) of the EWMA control chart.}
\item{mu}{true mean shift.}
\item{zr}{reflection border for the one-sided chart.}
\item{hs}{so-called headstart (give fast initial response).}
\item{sided}{distinguish between one- and two-sided EWMA control chart 
by choosing \code{"one"} and \code{"two"}, respectively.}
\item{limits}{distinguish between different control limits behavior.}
\item{q}{change point position. For \eqn{q=1} and
\eqn{\mu=\mu_0} and \eqn{\mu=\mu_1}, the usual
zero-state ARLs for the in-control and out-of-control case, respectively,
are calculated. For \eqn{q>1} and \eqn{\mu!=0} conditional delays, that is,
\eqn{E_q(L-q+1|L\ge q)}, will be determined.
Note that mu0=0 is implicitely fixed.}
\item{size}{pre run sample size.}
\item{df}{Degrees of freedom of the pre run variance estimator. Typically it is simply \code{size} - 1.
If the pre run is collected in batches, then also other values are needed.}
\item{estimated}{name the parameter to be estimated within
the \code{"mu"}, \code{"sigma"}, \code{"both"}.}
\item{qm.mu}{number of quadrature nodes for convoluting the mean uncertainty.}
\item{qm.sigma}{number of quadrature nodes for convoluting the standard deviation uncertainty.}
\item{truncate}{size of truncated tail.}
\item{L0}{in-control ARL.}
\item{c.error}{error bound for two succeeding values of the critical value
during applying the secant rule.}
\item{L.error}{error bound for the ARL level \code{L0} during applying the secant rule.}
\item{OUTPUT}{activate or deactivate additional output.}
}
\details{
Essentially, the ARL function \code{xewma.arl} is convoluted with the
distribution of the sample mean, standard deviation or both.
For details see Jones/Champ/Rigdon (2001) and Knoth (2014?).
}
\value{Returns a single value which resembles the ARL.}
\references{
L. A. Jones, C. W. Champ, S. E. Rigdon (2001),
The performance of exponentially weighted moving average charts
with estimated parameters,
\emph{Technometrics 43}, 156-167.

S. Knoth (2003),
EWMA schemes with non-homogeneous transition kernels,
\emph{Sequential Analysis 22}, 241-255.

S. Knoth (2004),
Fast initial response features for EWMA Control Charts,
\emph{Statistical Papers 46}, 47-64.

S. Knoth (2014?),
tbd,
\emph{tbd}, tbd-tbd.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} for the usual zero-state ARL computation.
}
\examples{
## Jones/Champ/Rigdon (2001)

c4m <- function(m, n) sqrt(2)*gamma( (m*(n-1)+1)/2 )/sqrt( m*(n-1) )/gamma( m*(n-1)/2 )

n <- 5 # sample size
m <- 20 # pre run with 20 samples of size n = 5
C4m <- c4m(m, n) # needed for bias correction

# Table 1, 3rd column
lambda <- 0.2
L <- 2.636

xewma.ARL <- Vectorize("xewma.arl", "mu")
xewma.ARL.prerun <- Vectorize("xewma.arl.prerun", "mu")

mu <- c(0, .25, .5, 1, 1.5, 2)
ARL <- round(xewma.ARL(lambda, L, mu, sided="two"), digits=2)
p.ARL <- round(xewma.ARL.prerun(lambda, L/C4m, mu, sided="two",
size=m, df=m*(n-1), estimated="both", qm.mu=70), digits=2)

# Monte-Carlo with 10^8 repetitions: 200.325 (0.020) and 144.458 (0.022)
cbind(mu, ARL, p.ARL)

\dontrun{
# Figure 5, subfigure r = 0.2
mu_ <- (0:85)/40
ARL_ <- round(xewma.ARL(lambda, L, mu_, sided="two"), digits=2)
p.ARL_ <- round(xewma.ARL.prerun(lambda, L/C4m, mu_, sided="two",
size=m, df=m*(n-1), estimated="both"), digits=2)

plot(mu_, ARL_, type="l", xlab=expression(delta), ylab="ARL", xlim=c(0,2))
abline(v=0, h=0, col="grey", lwd=.7)
points(mu, ARL, pch=5)
lines(mu_, p.ARL_, col="blue")
points(mu, p.ARL, pch=18, col ="blue")
legend("topright", c("Known", "Estimated"), col=c("black", "blue"),
lty=1, pch=c(5, 18))
}
}
\keyword{ts}

\name{xtewma.arl}
\alias{xtewma.arl}
\title{Compute ARLs of EWMA control charts, t distributed data}
\description{Computation of the (zero-state) Average Run Length (ARL)
for different types of EWMA control charts monitoring the mean of t distributed data.}
\usage{xtewma.arl(l,c,df,mu,zr=0,hs=0,sided="two",limits="fix",mode="tan",q=1,r=40)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{c}{critical value (similar to alarm limit) of the EWMA control chart.}
\item{df}{degrees of freedom -- parameter of the t distribution.}
\item{mu}{true mean.}
\item{zr}{reflection border for the one-sided chart.}
\item{hs}{so-called headstart (enables fast initial response).}
\item{sided}{distinguishes between one- and two-sided EWMA control chart 
by choosing \code{"one"} and \code{"two"}, respectively.}
\item{limits}{distinguishes between different control limits behavior.}
\item{mode}{Controls the type of variables substitution that might improve the numerical performance. Currently,
\code{"identity"}, \code{"sin"}, \code{"sinh"}, and \code{"tan"} (default) are provided.}
\item{q}{change point position. For \eqn{q=1} and
\eqn{\mu=\mu_0} and \eqn{\mu=\mu_1}, the usual
zero-state ARLs for the in-control and out-of-control case, respectively,
are calculated. For \eqn{q>1} and \eqn{\mu!=0} conditional delays, that is,
\eqn{E_q(L-q+1|L\ge q)}, will be determined.
Note that mu0=0 is implicitely fixed.}
\item{r}{number of quadrature nodes, dimension of the resulting linear
equation system is equal to \code{r+1} (one-sided) or \code{r}
(two-sided).}
}
\details{
In case of the EWMA chart with fixed control limits,
\code{xtewma.arl} determines the Average Run Length (ARL) by numerically
solving the related ARL integral equation by means of the Nystroem method
based on Gauss-Legendre quadrature.
If \code{limits} is \code{"vacl"}, then the method presented in Knoth (2003) is utilized.
Other values (normal case) for \code{limits} are not yet supported.
}
\value{Except for the fixed limits EWMA charts it returns a single value which resembles the ARL.
For fixed limits charts, it returns a vector of length \code{q} which resembles the ARL and the
sequence of conditional expected delays for \code{q}=1 and \code{q}>1, respectively.}
\references{
K.-H. Waldmann (1986),
Bounds for the distribution of the run length of geometric moving 
average charts, \emph{Appl. Statist. 35}, 151-158.

S. V. Crowder (1987),
A simple method for studying run-length distributions of
exponentially weighted moving average charts,
\emph{Technometrics 29}, 401-407.

J. M. Lucas and M. S. Saccucci (1990),
Exponentially weighted moving average control schemes: Properties
and enhancements, \emph{Technometrics 32}, 1-12.

C. M. Borror, D. C. Montgomery, and G. C. Runger (1999),
Robustness of the EWMA control chart to non-normality ,
\emph{Journal of Quality Technology 31}, 309-316.

S. Knoth (2003),
EWMA schemes with non-homogeneous transition kernels,
\emph{Sequential Analysis 22}, 241-255.

S. Knoth (2004),
Fast initial response features for EWMA Control Charts,
\emph{Statistical Papers 46}, 47-64.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} for zero-state ARL computation of EWMA control charts in the normal case.
}
\examples{
##  Borror/Montgomery/Runger (1999), Table 3
lambda <- 0.1
cE <- 2.703
df <- c(4, 6, 8, 10, 15, 20, 30, 40, 50)
L0 <- rep(NA, length(df))
for ( i in 1:length(df) ) {
  L0[i] <- round(xtewma.arl(lambda, cE*sqrt(df[i]/(df[i]-2)), df[i], 0), digits=0)
}
data.frame(df, L0)
}
\keyword{ts}

\name{x.res.ewma.arl}
\alias{x.res.ewma.arl}
\alias{s.res.ewma.arl}
\alias{xs.res.ewma.arl}
\alias{xs.res.ewma.pms}
\title{Compute ARLs of EWMA residual control charts}
\description{Computation of the (zero-state) Average Run Length (ARL)
for  EWMA residual control charts monitoring normal mean,
variance, or mean and variance simultaneously. Additionally,
the probability of misleading signals (PMS) is calculated.}
\usage{x.res.ewma.arl(l, c, mu, alpha=0, n=5, hs=0, r=40)

s.res.ewma.arl(l, cu, sigma, mu=0, alpha=0, n=5, hs=1, r=40, qm=30)

xs.res.ewma.arl(lx, cx, ls, csu, mu, sigma, alpha=0,
n=5, hsx=0, rx=40, hss=1, rs=40, qm=30)

xs.res.ewma.pms(lx, cx, ls, csu, mu, sigma, type="3",
alpha=0, n=5, hsx=0, rx=40, hss=1, rs=40, qm=30)
}
\arguments{
\item{l, lx, ls}{smoothing parameter(s) lambda of the EWMA control chart.}
\item{c, cu, cx, csu}{critical value (similar to alarm limit) of the EWMA control charts.}
\item{mu}{true mean.}
\item{sigma}{true standard deviation.}
\item{alpha}{the AR(1) coefficient -- first order autocorrelation of the original data.}
\item{n}{batch size.}
\item{hs, hsx, hss}{so-called headstart (enables fast initial response).}
\item{r, rx, rs}{number of quadrature nodes or size of collocation base,
dimension of the resulting linear
equation system is equal to \code{r} (two-sided).}
\item{qm}{number of nodes for collocation quadratures.}
\item{type}{PMS type, for \code{PMS}="3" (the default) the probability of
getting a mean signal despite the variance
changed, and for \code{PMS}="4" the opposite case is dealt with.}
}
\details{
The above list of functions provides the application of
algorithms developed for iid data to
the residual case. To be more precise, the underlying model is a sequence of normally
distributed batches with size \code{n} with autocorrelation within
the batch and independence between the batches
(see also the references below). It is restricted to the
classical EWMA chart types, that
is two-sided for the mean, upper charts for the variance,
and all equipped with fixed limits.
The autocorrelation is modeled by an AR(1) process with parameter
\code{alpha}. Additionally,
with \code{xs.res.ewma.pms} the probability of misleading signals
(PMS) of \code{type} is
calculated. This is offered exclusively in this small
collection so that for iid data
this function has to be used too (with \code{alpha=0}).
}
\value{Return single values which resemble the ARL and the PMS, respectively.}
\references{
S. Knoth, M. C. Morais, A. Pacheco, W. Schmid (2009),
Misleading Signals in Simultaneous Residual Schemes for the Mean and
Variance of a Stationary Process,
\emph{Commun. Stat., Theory Methods 38}, 2923-2943.

S. Knoth, W. Schmid, A. Schoene (2001),
Simultaneous Shewhart-Type Charts for the Mean and the Variance of a Time Series,
\emph{Frontiers of Statistical Quality Control 6,
A. Lenz, H.-J. & Wilrich, P.-T. (Eds.)}, 6, 61-79.

S. Knoth, W. Schmid (2002)
Monitoring the mean and the variance of a stationary process,
\emph{Statistica Neerlandica 56}, 77-100.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl}, \code{sewma.arl}, and \code{xsewma.arl} as more
elaborated functions in the iid case.}
\examples{
\dontrun{
## S. Knoth, W. Schmid (2002)

cat("\nFragments of Table 2 (n=5, lambda.1=lambda.2)\n")

lambdas <- c(.5, .25, .1, .05)
L0 <- 500
n <- 5

crit <- NULL
for ( lambda in lambdas ) {
  cs <- xsewma.crit(lambda, lambda, L0, n-1) 
  x.e <- round(cs[1], digits=4)
  names(x.e) <- NULL
  s.e <- round((cs[3]-1) * sqrt((2-lambda)/lambda)*sqrt((n-1)/2), digits=4)
  names(s.e) <- NULL
  crit <- rbind(crit, data.frame(lambda, x.e, s.e))
}


## orinal values are (Markov chain approximation with 50 states)
# lambda x.e    s.e
#   0.50 3.2765 4.6439
#   0.25 3.2168 4.0149
#   0.10 3.0578 3.3376
#   0.05 2.8817 2.9103

print(crit)


cat("\nFragments of Table 4 (n=5, lambda.1=lambda.2=0.1)\n\n")

lambda <- .1
# the algorithm used in Knoth/Schmid is less accurate -- proceed with their values
cx <- x.e <- 3.0578
s.e <- 3.3376
csu <- 1 + s.e * sqrt(lambda/(2-lambda))*sqrt(2/(n-1))

alpha <- .3

a.values <- c((0:6)/4, 2)
d.values <- c(1 + (0:5)/10, 1.75 , 2)

arls <- NULL
for ( delta in d.values ) {
  row <- NULL
  for ( mu in a.values ) {
    arl <- round(xs.res.ewma.arl(lambda, cx, lambda, csu, mu*sqrt(n), delta, alpha=alpha, n=n),
                 digits=2)
    names(arl) <- NULL
    row <- c(row, arl)   
  }
  arls <- rbind(arls, data.frame(t(row)))
}
names(arls) <- a.values
rownames(arls) <- d.values

## orinal values are (now Monte-Carlo with 10^6 replicates)
#          0  0.25   0.5 0.75    1 1.25  1.5    2
#1    502.44 49.50 14.21 7.93 5.53 4.28 3.53 2.65
#1.1   73.19 32.91 13.33 7.82 5.52 4.29 3.54 2.66
#1.2   24.42 18.88 11.37 7.44 5.42 4.27 3.54 2.67
#1.3   13.11 11.83  9.09 6.74 5.18 4.17 3.50 2.66
#1.4    8.74  8.31  7.19 5.89 4.81 4.00 3.41 2.64
#1.5    6.50  6.31  5.80 5.08 4.37 3.76 3.28 2.59
#1.75   3.94  3.90  3.78 3.59 3.35 3.09 2.83 2.40
#2      2.85  2.84  2.80 2.73 2.63 2.51 2.39 2.14

print(arls)


## S. Knoth, M. C. Morais, A. Pacheco, W. Schmid (2009)

cat("\nFragments of Table 5 (n=5, lambda=0.1)\n\n")

d.values <- c(1.02, 1 + (1:5)/10, 1.75 , 2)

arl.x <- arl.s <- arl.xs <- PMS.3 <- NULL
for ( delta in d.values ) {
  arl.x  <- c(arl.x,  round(x.res.ewma.arl(lambda, cx/delta, 0, n=n),
                            digits=3))
  arl.s  <- c(arl.s,  round(s.res.ewma.arl(lambda, csu, delta, n=n),
                            digits=3))
  arl.xs <- c(arl.xs, round(xs.res.ewma.arl(lambda, cx, lambda, csu, 0, delta, n=n),
                            digits=3))
  PMS.3  <- c(PMS.3,  round(xs.res.ewma.pms(lambda, cx, lambda, csu, 0, delta, n=n),
                            digits=6))
}

## orinal values are (Markov chain approximation)
# delta   arl.x   arl.s  arl.xs PMS.3
#  1.02 833.086 518.935 323.324 0.381118
#  1.10 454.101  84.208  73.029 0.145005
#  1.20 250.665  25.871  24.432 0.071024
#  1.30 157.343  13.567  13.125 0.047193
#  1.40 108.112   8.941   8.734 0.035945
#  1.50  79.308   6.614   6.493 0.029499
#  1.75  44.128   3.995   3.942 0.021579
#  2.00  28.974   2.887   2.853 0.018220

print(cbind(delta=d.values, arl.x, arl.s, arl.xs, PMS.3))


cat("\nFragments of Table 6 (n=5, lambda=0.1)\n\n")

alphas <- c(-0.9, -0.5, -0.3, 0, 0.3, 0.5, 0.9)
deltas <- c(0.05, 0.25, 0.5, 0.75, 1, 1.25, 1.5, 2)

PMS.4 <- NULL
for ( ir in 1:length(deltas) ) {
  mu <- deltas[ir]*sqrt(n)
  pms <- NULL
  for ( alpha in alphas ) {
    pms <- c(pms, round(xs.res.ewma.pms(lambda, cx, lambda, csu, mu, 1, type="4", alpha=alpha, n=n),
                        digits=6))
  }
  PMS.4 <- rbind(PMS.4, data.frame(delta=deltas[ir], t(pms)))
}
names(PMS.4) <- c("delta", alphas)
rownames(PMS.4) <- NULL

## orinal values are (Markov chain approximation)
#  delta     -0.9     -0.5     -0.3        0      0.3      0.5      0.9
#   0.05 0.055789 0.224521 0.279842 0.342805 0.391299 0.418915 0.471386
#   0.25 0.003566 0.009522 0.014580 0.025786 0.044892 0.066584 0.192023
#   0.50 0.002994 0.001816 0.002596 0.004774 0.009259 0.015303 0.072945
#   0.75 0.006967 0.000703 0.000837 0.001529 0.003400 0.006424 0.046602
#   1.00 0.005098 0.000402 0.000370 0.000625 0.001589 0.003490 0.039978
#   1.25 0.000084 0.000266 0.000202 0.000300 0.000867 0.002220 0.039773
#   1.50 0.000000 0.000256 0.000120 0.000163 0.000531 0.001584 0.042734
#   2.00 0.000000 0.000311 0.000091 0.000056 0.000259 0.001029 0.054543

print(PMS.4)
}
}
\keyword{ts}

\name{xewma.arl.f}
\alias{xewma.arl.f}
\title{Compute ARL function of EWMA control charts}
\description{Computation of the (zero-state) Average Run Length (ARL) function for different types of EWMA control charts monitoring normal mean.}
\usage{xewma.arl.f(l,c,mu,zr=0,sided="one",limits="fix",r=40)}
\arguments{
\item{l}{smoothing parameter lambda of the EWMA control chart.}
\item{c}{critical value (similar to alarm limit) of the EWMA control chart.}
\item{mu}{true mean.}
\item{zr}{reflection border for the one-sided chart.}
\item{sided}{distinguishes between one- and two-sided EWMA control chart by choosing \code{"one"} and \code{"two"}, respectively.}
\item{limits}{distinguishes between different control limits behavior.}
\item{r}{number of quadrature nodes, dimension of the resulting linear equation system is equal to \code{r+1} (one-sided) or \code{r} (two-sided).}
}
\details{
It is a convenience function to yield the ARL as function of the head start \code{hs}. For more details see \code{xewma.arl}.
}
\value{It returns a function of a single argument, \code{hs=x} which maps the head-start value \code{hs} to the ARL.}
\references{
S. V. Crowder (1987),
A simple method for studying run-length distributions of exponentially weighted moving average charts,
\emph{Technometrics 29}, 401-407.
}
\author{Sven Knoth}
\seealso{
\code{xewma.arl} for zero-state ARL for one specific head-start \code{hs}.
}
\examples{
# will follow
}
\keyword{ts}

\name{pois.ewma.ad}
\alias{pois.ewma.ad}
\title{Compute steady-state ARLs of Poisson EWMA control charts}
\description{Computation of the steady-state Average Run Length (ARL) at given mean \code{mu}.}
\usage{pois.ewma.ad(lambda, AL, AU, mu0, mu, sided="two", rando=FALSE, gL=0, gU=0,
mcdesign="classic", N=101, OLD=FALSE)}
\arguments{
\item{lambda}{smoothing parameter of the EWMA p control chart.}
\item{AL, AU}{factors to build the lower and upper control limit, respectively, of the Poisson EWMA control chart.}
\item{mu0}{in-control mean.}
\item{mu}{actual mean.}
\item{sided}{distinguishes between one- and two-sided EWMA control chart by choosing
\code{"upper"}, \code{"lower"}, and \code{"two"}, and \code{"zwei"}, respectively.}
\item{rando}{Switch between the standard limit treatment, \code{FALSE},
and an additional randomisation (to allow `perfect' ARL calibration) by setting \code{TRUE}.
If randomisation is used, then set the corresponding probailities, \code{gL} and \code{gU}, appropriately.}
\item{gL, gU}{If the EWMA statistic is at the limit (approximately), then an alarm is triggered with probability
\code{gL} and \code{gU} for the lower and upper limit, respectively.}
\item{mcdesign}{choose either \code{"classic"} which follows Borror, Champ and Rigdon (1998), or the more
sophisticated \code{"transfer"} which improves the accuracy heavily.}
\item{N}{number of states of the approximating Markov chain; is equal to the dimension of the
resulting linear equation system.}
\item{OLD}{Switch between the original implemenation, used in Morais and Knoth (2020), and a slightly modified new one (more accurate).}
}
\details{
The monitored data follow a Poisson distribution with \code{mu}.
The ARL values of the resulting EWMA control chart are determined by Markov chain approximation.
We follow the algorithm given in Borror, Champ and Rigdon (1998). The function is in an early development phase.
}
\value{Return single value which resembles the steady-state ARL.}
\references{
C. M. Borror, C. W. Champ and S. E. Rigdon (1998)
Poisson EWMA control charts,
\emph{Journal of Quality Technonlogy} 30(4), 352-361.

M. C. Morais and S. Knoth (2020)
Improving the ARL profile and the accuracy of its calculation for Poisson EWMA charts,
\emph{Quality and Reliability Engineering International} 36(3), 876-889.
}
\author{Sven Knoth}
\seealso{later.}
\examples{
## Borror, Champ and Rigdon (1998), Table 2, PEWMA column
mu0 <- 20
lambda <- 0.27
A <- 3.319
mu1  <- c(2*(3:15), 35)
ARL1 <- AD1 <- rep(NA, length(mu1))
for ( i in 1:length(mu1) ) {
  ARL1[i] <- round(pois.ewma.arl(lambda,A,A,mu0,mu0,mu1[i],mcdesign="classic"),digits=1)
  AD1[i]  <- round(pois.ewma.ad(lambda,A,A,mu0,mu1[i],mcdesign="classic"),digits=1)
}
print( cbind(mu1, ARL1, AD1) )

## Morais and Knoth (2020), Table 2, lambda = 0.27 column
## randomisation not implemented for pois.ewma.ad()
lambda <- 0.27
AL <- 3.0870
AU <- 3.4870
gL <- 0.001029
gU <- 0.000765
mu2  <- c(16, 18, 19.99, mu0, 20.01, 22, 24)
ARL2 <- AD2 <- rep(NA, length(mu2))
for ( i in 1:length(mu2) ) {
  ARL2[i] <- round(pois.ewma.arl(lambda,AL,AU,mu0,mu0,mu2[i],rando=FALSE,OLD=TRUE), digits=1)
  AD2[i] <- round(pois.ewma.ad(lambda,AL,AU,mu0,mu2[i],rando=FALSE,OLD=TRUE), digits=1)
}
print( cbind(mu2, ARL2, AD2) )
}
\keyword{ts}
